@@ -19,19 +19,10 @@
  * MA 02110-1301 USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <string.h> // memcmp, memcpy
 #include <stdlib.h> // malloc, free
 
-#ifdef HAVE_LIBUSB
-#ifdef _WIN32
-#define NOGDI
-#endif
-#include <libusb-1.0/libusb.h>
-#endif
+#include <libdivecomputer/usb.h>
 
 #include "atomics_cobalt.h"
 #include "context-private.h"
@@ -41,7 +32,8 @@
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &atomics_cobalt_device_vtable)
 
-#define EXITCODE(rc) (rc == LIBUSB_ERROR_TIMEOUT ? DC_STATUS_TIMEOUT : DC_STATUS_IO)
+#define COBALT1 0
+#define COBALT2 2
 
 #define VID 0x0471
 #define PID 0x0888
@@ -49,15 +41,13 @@
 
 #define FP_OFFSET 20
 
-#define SZ_MEMORY (29 * 64 * 1024)
+#define SZ_MEMORY1 (29 * 64 * 1024) // Cobalt 1
+#define SZ_MEMORY2 (41 * 64 * 1024) // Cobalt 2
 #define SZ_VERSION 14
 
 typedef struct atomics_cobalt_device_t {
 	dc_device_t base;
-#ifdef HAVE_LIBUSB
-	libusb_context *context;
-	libusb_device_handle *handle;
-#endif
+	dc_iostream_t *iostream;
 	unsigned int simulation;
 	unsigned char fingerprint[6];
 	unsigned char version[SZ_VERSION];
@@ -65,7 +55,6 @@ typedef struct atomics_cobalt_device_t {
 
 static dc_status_t atomics_cobalt_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t atomics_cobalt_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t atomics_cobalt_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t atomics_cobalt_device_vtable = {
 	sizeof(atomics_cobalt_device_t),
@@ -76,22 +65,19 @@ static const dc_device_vtable_t atomics_cobalt_device_vtable = {
 	NULL, /* dump */
 	atomics_cobalt_device_foreach, /* foreach */
 	NULL, /* timesync */
-	atomics_cobalt_device_close /* close */
+	NULL /* close */
 };
 
 
 dc_status_t
-atomics_cobalt_device_open (dc_device_t **out, dc_context_t *context)
+atomics_cobalt_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
-#ifdef HAVE_LIBUSB
 	dc_status_t status = DC_STATUS_SUCCESS;
 	atomics_cobalt_device_t *device = NULL;
-#endif
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-#ifdef HAVE_LIBUSB
 	// Allocate memory.
 	device = (atomics_cobalt_device_t *) dc_device_allocate (context, &atomics_cobalt_device_vtable);
 	if (device == NULL) {
@@ -100,67 +86,30 @@ atomics_cobalt_device_open (dc_device_t **out, dc_context_t *context)
 	}
 
 	// Set the default values.
-	device->context = NULL;
-	device->handle = NULL;
+	device->iostream = iostream;
 	device->simulation = 0;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	int rc = libusb_init (&device->context);
-	if (rc < 0) {
-		ERROR (context, "Failed to initialize usb support.");
-		status = DC_STATUS_IO;
+	// Set the timeout for receiving data (2000 ms).
+	status = dc_iostream_set_timeout (device->iostream, TIMEOUT);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to set the timeout.");
 		goto error_free;
 	}
 
-	device->handle = libusb_open_device_with_vid_pid (device->context, VID, PID);
-	if (device->handle == NULL) {
-		ERROR (context, "Failed to open the usb device.");
-		status = DC_STATUS_IO;
-		goto error_usb_exit;
-	}
-
-	rc = libusb_claim_interface (device->handle, 0);
-	if (rc < 0) {
-		ERROR (context, "Failed to claim the usb interface.");
-		status = DC_STATUS_IO;
-		goto error_usb_close;
-	}
-
 	status = atomics_cobalt_device_version ((dc_device_t *) device, device->version, sizeof (device->version));
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to identify the dive computer.");
-		goto error_usb_close;
+		goto error_free;
 	}
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_usb_close:
-	libusb_close (device->handle);
-error_usb_exit:
-	libusb_exit (device->context);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
-#else
-	return DC_STATUS_UNSUPPORTED;
-#endif
-}
-
-
-static dc_status_t
-atomics_cobalt_device_close (dc_device_t *abstract)
-{
-	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
-
-#ifdef HAVE_LIBUSB
-	libusb_release_interface(device->handle, 0);
-	libusb_close (device->handle);
-	libusb_exit (device->context);
-#endif
-
-	return DC_STATUS_SUCCESS;
 }
 
 
@@ -198,6 +147,7 @@ atomics_cobalt_device_set_simulation (dc_device_t *abstract, unsigned int simula
 dc_status_t
 atomics_cobalt_device_version (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
 	if (!ISINSTANCE (abstract))
@@ -206,31 +156,31 @@ atomics_cobalt_device_version (dc_device_t *abstract, unsigned char data[], unsi
 	if (size < SZ_VERSION)
 		return DC_STATUS_INVALIDARGS;
 
-#ifdef HAVE_LIBUSB
 	// Send the command to the dive computer.
-	uint8_t bRequest = 0x01;
-	int rc = libusb_control_transfer (device->handle,
-		LIBUSB_RECIPIENT_DEVICE | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_OUT,
-		bRequest, 0, 0, NULL, 0, TIMEOUT);
-	if (rc != LIBUSB_SUCCESS) {
-		ERROR (abstract->context, "Failed to send the command.");
-		return EXITCODE(rc);
-	}
+	unsigned char bRequest = 0x01;
+	dc_usb_control_t control = {
+		DC_USB_REQUEST_VENDOR | DC_USB_RECIPIENT_DEVICE | DC_USB_ENDPOINT_OUT, /* bmRequestType */
+		bRequest, /* bRequest */
+		0, /* wValue */
+		0, /* wIndex */
+		0, /* wLength */
+	};
 
-	HEXDUMP (abstract->context, DC_LOGLEVEL_INFO, "Write", &bRequest, 1);
+	status = dc_iostream_ioctl (device->iostream, DC_IOCTL_USB_CONTROL_WRITE, &control, sizeof(control));
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to send the command.");
+		return status;
+	}
 
 	// Receive the answer from the dive computer.
-	int length = 0;
+	size_t length = 0;
 	unsigned char packet[SZ_VERSION + 2] = {0};
-	rc = libusb_bulk_transfer (device->handle, 0x82,
-		packet, sizeof (packet), &length, TIMEOUT);
-	if (rc != LIBUSB_SUCCESS || length != sizeof (packet)) {
+	status = dc_iostream_read (device->iostream, packet, sizeof(packet), &length);
+	if (status != DC_STATUS_SUCCESS || length != sizeof (packet)) {
 		ERROR (abstract->context, "Failed to receive the answer.");
-		return EXITCODE(rc);
+		return status;
 	}
 
-	HEXDUMP (abstract->context, DC_LOGLEVEL_INFO, "Read", packet, length);
-
 	// Verify the checksum of the packet.
 	unsigned short crc = array_uint16_le (packet + SZ_VERSION);
 	unsigned short ccrc = checksum_add_uint16 (packet, SZ_VERSION, 0x0);
@@ -242,16 +192,13 @@ atomics_cobalt_device_version (dc_device_t *abstract, unsigned char data[], unsi
 	memcpy (data, packet, SZ_VERSION);
 
 	return DC_STATUS_SUCCESS;
-#else
-	return DC_STATUS_UNSUPPORTED;
-#endif
 }
 
 
 static dc_status_t
 atomics_cobalt_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc_event_progress_t *progress)
 {
-#ifdef HAVE_LIBUSB
+	dc_status_t status = DC_STATUS_SUCCESS;
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
 	if (device_is_cancelled (abstract))
@@ -263,36 +210,47 @@ atomics_cobalt_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init,
 		return DC_STATUS_NOMEMORY;
 	}
 
+	// Adjust the maximum value to take into account the two byte checksum and
+	// the 8 byte serial number. Those extra bytes are not stored inside the
+	// dive header and are added dynamically during the data transfer. Since we
+	// don't know the total number of dives in advance, we can't calculate the
+	// total number of extra bytes and adjust the maximum on the fly.
+	if (progress) {
+		progress->maximum += 2 + 8;
+	}
+
 	// Send the command to the dive computer.
-	uint8_t bRequest = 0;
+	unsigned char bRequest = 0;
 	if (device->simulation)
 		bRequest = init ? 0x02 : 0x03;
 	else
 		bRequest = init ? 0x09 : 0x0A;
-	int rc = libusb_control_transfer (device->handle,
-		LIBUSB_RECIPIENT_DEVICE | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_OUT,
-		bRequest, 0, 0, NULL, 0, TIMEOUT);
-	if (rc != LIBUSB_SUCCESS) {
-		ERROR (abstract->context, "Failed to send the command.");
-		return EXITCODE(rc);
-	}
 
-	HEXDUMP (abstract->context, DC_LOGLEVEL_INFO, "Write", &bRequest, 1);
+	dc_usb_control_t control = {
+		DC_USB_REQUEST_VENDOR | DC_USB_RECIPIENT_DEVICE | DC_USB_ENDPOINT_OUT, /* bmRequestType */
+		bRequest, /* bRequest */
+		0, /* wValue */
+		0, /* wIndex */
+		0, /* wLength */
+	};
+
+	status = dc_iostream_ioctl (device->iostream, DC_IOCTL_USB_CONTROL_WRITE, &control, sizeof(control));
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to send the command.");
+		return status;
+	}
 
 	unsigned int nbytes = 0;
 	while (1) {
 		// Receive the answer from the dive computer.
-		int length = 0;
+		size_t length = 0;
 		unsigned char packet[8 * 1024] = {0};
-		rc = libusb_bulk_transfer (device->handle, 0x82,
-			packet, sizeof (packet), &length, TIMEOUT);
-		if (rc != LIBUSB_SUCCESS && rc != LIBUSB_ERROR_TIMEOUT) {
+		status = dc_iostream_read (device->iostream, packet, sizeof(packet), &length);
+		if (status != DC_STATUS_SUCCESS && status != DC_STATUS_TIMEOUT) {
 			ERROR (abstract->context, "Failed to receive the answer.");
-			return EXITCODE(rc);
+			return status;
 		}
 
-		HEXDUMP (abstract->context, DC_LOGLEVEL_INFO, "Read", packet, length);
-
 		// Update and emit a progress event.
 		if (progress) {
 			progress->current += length;
@@ -339,9 +297,6 @@ atomics_cobalt_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init,
 	dc_buffer_slice (buffer, 0, nbytes - 2);
 
 	return DC_STATUS_SUCCESS;
-#else
-	return DC_STATUS_UNSUPPORTED;
-#endif
 }
 
 
@@ -350,9 +305,12 @@ atomics_cobalt_device_foreach (dc_device_t *abstract, dc_dive_callback_t callbac
 {
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
+	// Get the model number.
+	unsigned int model = array_uint16_le (device->version + 12);
+
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
-	progress.maximum = SZ_MEMORY + 2;
+	progress.maximum = (model == COBALT2 ? SZ_MEMORY2 : SZ_MEMORY1) + 2;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Emit a vendor event.
@@ -399,12 +357,6 @@ atomics_cobalt_device_foreach (dc_device_t *abstract, dc_dive_callback_t callbac
 			return DC_STATUS_SUCCESS;
 		}
 
-		// Adjust the maximum value to take into account the two checksum bytes
-		// for the next dive. Since we don't know the total number of dives in
-		// advance, we can't calculate the total number of checksum bytes and
-		// adjust the maximum on the fly.
-		progress.maximum += 2;
-
 		ndives++;
 	}
 
