@@ -39,7 +39,22 @@ unsigned char
 checksum_xor_uint8 (const unsigned char data[], unsigned int size, unsigned char init);
 
 unsigned short
-checksum_crc_ccitt_uint16 (const unsigned char data[], unsigned int size);
+checksum_crc16_ccitt (const unsigned char data[], unsigned int size, unsigned short init, unsigned short xorout);
+
+unsigned short
+checksum_crc16r_ccitt (const unsigned char data[], unsigned int size, unsigned short init, unsigned short xorout);
+
+unsigned short
+checksum_crc16_ansi (const unsigned char data[], unsigned int size, unsigned short init, unsigned short xorout);
+
+unsigned short
+checksum_crc16r_ansi (const unsigned char data[], unsigned int size, unsigned short init, unsigned short xorout);
+
+unsigned int
+checksum_crc32r (const unsigned char data[], unsigned int size);
+
+unsigned int
+checksum_crc32 (const unsigned char data[], unsigned int size);
 
 #ifdef __cplusplus
 }
