@@ -26,13 +26,10 @@
 #include "cochran_commander.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "array.h"
 #include "ringbuffer.h"
 #include "rbstream.h"
 
-#define C_ARRAY_SIZE(array) (sizeof (array) / sizeof *(array))
-
 #define MAXRETRIES 2
 
 #define COCHRAN_MODEL_COMMANDER_TM 0
@@ -42,6 +39,8 @@
 #define COCHRAN_MODEL_EMC_16 4
 #define COCHRAN_MODEL_EMC_20 5
 
+#define UNDEFINED 0xFFFFFFFF
+
 typedef enum cochran_endian_t {
 	ENDIAN_LE,
 	ENDIAN_BE,
@@ -58,8 +57,8 @@ typedef struct cochran_data_t {
 	unsigned char *logbook;
 
 	unsigned short int dive_count;
-	int fp_dive_num;
-	int invalid_profile_dive_num;
+	unsigned int fp_dive_num;
+	unsigned int invalid_profile_dive_num;
 
 	unsigned int logbook_size;
 } cochran_data_t;
@@ -94,7 +93,7 @@ typedef struct cochran_device_layout_t {
 
 typedef struct cochran_commander_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	const cochran_device_layout_t *layout;
 	unsigned char id[67];
 	unsigned char fingerprint[6];
@@ -104,7 +103,6 @@ static dc_status_t cochran_commander_device_set_fingerprint (dc_device_t *device
 static dc_status_t cochran_commander_device_read (dc_device_t *device, unsigned int address, unsigned char data[], unsigned int size);
 static dc_status_t cochran_commander_device_dump (dc_device_t *device, dc_buffer_t *data);
 static dc_status_t cochran_commander_device_foreach (dc_device_t *device, dc_dive_callback_t callback, void *userdata);
-static dc_status_t cochran_commander_device_close (dc_device_t *device);
 
 static const dc_device_vtable_t cochran_commander_device_vtable = {
 	sizeof (cochran_commander_device_t),
@@ -115,7 +113,7 @@ static const dc_device_vtable_t cochran_commander_device_vtable = {
 	cochran_commander_device_dump, /* dump */
 	cochran_commander_device_foreach, /* foreach */
 	NULL, /* timesync */
-	cochran_commander_device_close /* close */
+	NULL /* close */
 };
 
 // Cochran Commander TM, pre-dates pre-21000 s/n
@@ -305,30 +303,30 @@ cochran_commander_serial_setup (cochran_commander_device_t *device)
 	dc_status_t status = DC_STATUS_SUCCESS;
 
 	// Set the serial communication protocol (9600 8N2, no FC).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (device->base.context, "Failed to set the terminal attributes.");
 		return status;
 	}
 
 	// Set the timeout for receiving data (5000 ms).
-	status = dc_serial_set_timeout (device->port, 5000);
+	status = dc_iostream_set_timeout (device->iostream, 5000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (device->base.context, "Failed to set the timeout.");
 		return status;
 	}
 
 	// Wake up DC and trigger heartbeat
-	dc_serial_set_break(device->port, 1);
-	dc_serial_sleep(device->port, 16);
-	dc_serial_set_break(device->port, 0);
+	dc_iostream_set_break(device->iostream, 1);
+	dc_iostream_sleep(device->iostream, 16);
+	dc_iostream_set_break(device->iostream, 0);
 
 	// Clear old heartbeats
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Wait for heartbeat byte before send
 	unsigned char answer = 0;
-	status = dc_serial_read(device->port, &answer, 1, NULL);
+	status = dc_iostream_read(device->iostream, &answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (device->base.context, "Failed to receive device heartbeat.");
 		return status;
@@ -359,9 +357,9 @@ cochran_commander_packet (cochran_commander_device_t *device, dc_event_progress_
 	// has no buffering.
 	for (unsigned int i = 0; i < csize; i++) {
 		// Give the DC time to read the character.
-		if (i) dc_serial_sleep(device->port, 16); // 16 ms
+		if (i) dc_iostream_sleep(device->iostream, 16); // 16 ms
 
-		status = dc_serial_write(device->port, command + i, 1, NULL);
+		status = dc_iostream_write(device->iostream, command + i, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to send the command.");
 			return status;
@@ -370,10 +368,10 @@ cochran_commander_packet (cochran_commander_device_t *device, dc_event_progress_
 
 	if (high_speed && device->layout->baudrate != 9600) {
 		// Give the DC time to process the command.
-		dc_serial_sleep(device->port, 45);
+		dc_iostream_sleep(device->iostream, 45);
 
 		// Rates are odd, like 850400 for the EMC, 115200 for commander
-		status = dc_serial_configure(device->port, device->layout->baudrate, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
+		status = dc_iostream_configure(device->iostream, device->layout->baudrate, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to set the high baud rate.");
 			return status;
@@ -388,7 +386,7 @@ cochran_commander_packet (cochran_commander_device_t *device, dc_event_progress_
 		if (len > 1024)
 			len = 1024;
 
-		status = dc_serial_read (device->port, answer + nbytes, len, NULL);
+		status = dc_iostream_read (device->iostream, answer + nbytes, len, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive data.");
 			return status;
@@ -523,7 +521,7 @@ cochran_commander_read (cochran_commander_device_t *device, dc_event_progress_t
 		return DC_STATUS_UNSUPPORTED;
 	}
 
-	dc_serial_sleep(device->port, 550);
+	dc_iostream_sleep(device->iostream, 550);
 
 	// set back to 9600 baud
 	rc = cochran_commander_serial_setup(device);
@@ -623,18 +621,17 @@ cochran_commander_find_fingerprint(cochran_commander_device_t *device, cochran_d
 	// We track profile ringbuffer usage to determine which dives have profile data
 	int profile_capacity_remaining = device->layout->rb_profile_end - device->layout->rb_profile_begin;
 
-	int dive_count = -1;
-	data->fp_dive_num = -1;
+	unsigned int dive_count = 0;
+	data->fp_dive_num = UNDEFINED;
 
 	// Start at end of log
 	if (data->dive_count < device->layout->rb_logbook_entry_count)
 		dive_count = data->dive_count;
 	else
 		dive_count = device->layout->rb_logbook_entry_count;
-	dive_count--;
 
 	unsigned int sample_read_size = 0;
-	data->invalid_profile_dive_num = -1;
+	data->invalid_profile_dive_num = UNDEFINED;
 
 	// Remove the pre-dive events that occur after the last dive
 	unsigned int rb_head_ptr = 0;
@@ -678,7 +675,7 @@ cochran_commander_find_fingerprint(cochran_commander_device_t *device, cochran_d
 
 	// Loop through dives to find FP, Accumulate profile data size,
 	// and find the last dive with invalid profile
-	for (unsigned int i = 0; i <= dive_count; ++i) {
+	for (unsigned int i = 0; i < dive_count; ++i) {
 		unsigned int idx = (device->layout->rb_logbook_entry_count + head_dive - (i + 1)) % device->layout->rb_logbook_entry_count;
 
 		unsigned char *log_entry = data->logbook + idx * device->layout->rb_logbook_entry_size;
@@ -716,7 +713,7 @@ cochran_commander_find_fingerprint(cochran_commander_device_t *device, cochran_d
 
 
 dc_status_t
-cochran_commander_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+cochran_commander_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	cochran_commander_device_t *device = NULL;
@@ -732,26 +729,19 @@ cochran_commander_device_open (dc_device_t **out, dc_context_t *context, const c
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 	cochran_commander_device_set_fingerprint((dc_device_t *) device, NULL, 0);
 
-	// Open the device.
-	status = dc_serial_open (&device->port, device->base.context, name);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (device->base.context, "Failed to open the serial port.");
-		goto error_free;
-	}
-
 	status = cochran_commander_serial_setup(device);
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
 	// Read ID from the device
 	status = cochran_commander_read_id (device, device->id, sizeof(device->id));
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Device not responding.");
-		goto error_close;
+		goto error_free;
 	}
 
 	unsigned int model = cochran_commander_get_model(device);
@@ -777,36 +767,18 @@ cochran_commander_device_open (dc_device_t **out, dc_context_t *context, const c
 	default:
 		ERROR (context, "Unknown model");
 		status = DC_STATUS_UNSUPPORTED;
-		goto error_close;
+		goto error_free;
 	}
 
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
-static dc_status_t
-cochran_commander_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	cochran_commander_device_t *device = (cochran_commander_device_t *) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
 static dc_status_t
 cochran_commander_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
@@ -842,12 +814,6 @@ cochran_commander_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	unsigned int config_size = sizeof(config);
 	unsigned int size = device->layout->rb_profile_end - device->layout->rb_logbook_begin;
 
-	// Make sure buffer is good.
-	if (!dc_buffer_clear(buffer)) {
-		ERROR (abstract->context, "Uninitialized buffer.");
-		return DC_STATUS_INVALIDARGS;
-	}
-
 	// Reserve space
 	if (!dc_buffer_resize(buffer, size)) {
 		ERROR(abstract->context, "Insufficient buffer space available.");
@@ -986,7 +952,7 @@ cochran_commander_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 	}
 
 	// Change tail to dive following the fingerprint dive.
-	if (data.fp_dive_num > -1)
+	if (data.fp_dive_num != UNDEFINED)
 		tail_dive = (data.fp_dive_num + 1) % layout->rb_logbook_entry_count;
 
 	// Number of dives to read
