@@ -29,8 +29,6 @@
 #include "parser-private.h"
 #include "array.h"
 
-#define C_ARRAY_SIZE(array) (sizeof (array) / sizeof *(array))
-
 #define COCHRAN_MODEL_COMMANDER_TM 0
 #define COCHRAN_MODEL_COMMANDER_PRE21000 1
 #define COCHRAN_MODEL_COMMANDER_AIR_NITROX 2
@@ -110,6 +108,9 @@ static const dc_parser_vtable_t cochran_commander_parser_vtable = {
 	sizeof(cochran_commander_parser_t),
 	DC_FAMILY_COCHRAN_COMMANDER,
 	cochran_commander_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	cochran_commander_parser_get_datetime, /* datetime */
 	cochran_commander_parser_get_field, /* fields */
 	cochran_commander_parser_samples_foreach, /* samples_foreach */
@@ -474,19 +475,19 @@ cochran_commander_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 	if (value) {
 		switch (type) {
 		case DC_FIELD_TEMPERATURE_SURFACE:
-			*((unsigned int*) value) = (data[layout->start_temp] - 32.0) / 1.8;
+			*((double *) value) = (data[layout->start_temp] - 32.0) / 1.8;
 			break;
 		case DC_FIELD_TEMPERATURE_MINIMUM:
 			if (data[layout->min_temp] == 0xFF)
 				return DC_STATUS_UNSUPPORTED;
-			*((unsigned int*) value) = (data[layout->min_temp] / 2.0 + 20 - 32) / 1.8;
+			*((double *) value) = (data[layout->min_temp] / 2.0 + 20 - 32) / 1.8;
 			break;
 		case DC_FIELD_TEMPERATURE_MAXIMUM:
 			if (layout->max_temp == UNSUPPORTED)
 				return DC_STATUS_UNSUPPORTED;
 			if (data[layout->max_temp] == 0xFF)
 				return DC_STATUS_UNSUPPORTED;
-			*((unsigned int*) value) = (data[layout->max_temp] / 2.0 + 20 - 32) / 1.8;
+			*((double *) value) = (data[layout->max_temp] / 2.0 + 20 - 32) / 1.8;
 			break;
 		case DC_FIELD_DIVETIME:
 			minutes = array_uint16_le(data + layout->divetime);
