@@ -48,6 +48,9 @@ static const dc_parser_vtable_t cressi_leonardo_parser_vtable = {
 	sizeof(cressi_leonardo_parser_t),
 	DC_FAMILY_CRESSI_LEONARDO,
 	cressi_leonardo_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	cressi_leonardo_parser_get_datetime, /* datetime */
 	cressi_leonardo_parser_get_field, /* fields */
 	cressi_leonardo_parser_samples_foreach, /* samples_foreach */
@@ -178,6 +181,9 @@ cressi_leonardo_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callbac
 
 	unsigned int gasmix_previous = 0xFFFFFFFF;
 	unsigned int gasmix = 0;
+	if (parser->model == DRAKE) {
+		gasmix = gasmix_previous;
+	}
 
 	unsigned int offset = SZ_HEADER;
 	while (offset + 2 <= size) {
