@@ -33,9 +33,7 @@
 #include "reefnet_sensuspro.h"
 #include "reefnet_sensusultra.h"
 #include "uwatec_aladin.h"
-#include "uwatec_g2.h"
 #include "uwatec_memomouse.h"
-#include "uwatec_meridian.h"
 #include "uwatec_smart.h"
 #include "oceanic_atom2.h"
 #include "oceanic_veo250.h"
@@ -49,6 +47,7 @@
 #include "hw_ostc3.h"
 #include "cressi_edy.h"
 #include "cressi_leonardo.h"
+#include "cressi_goa.h"
 #include "zeagle_n2ition3.h"
 #include "atomics_cobalt.h"
 #include "shearwater_petrel.h"
@@ -57,6 +56,15 @@
 #include "citizen_aqualand.h"
 #include "divesystem_idive.h"
 #include "cochran_commander.h"
+#include "tecdiving_divecomputereu.h"
+#include "mclean_extreme.h"
+#include "liquivision_lynx.h"
+#include "sporasub_sp2.h"
+#include "deepsix_excursion.h"
+#include "seac_screen.h"
+#include "deepblu_cosmiq.h"
+#include "oceans_s1.h"
+#include "divesoft_freedom.h"
 
 #include "device-private.h"
 #include "context-private.h"
@@ -100,7 +108,7 @@ dc_device_deallocate (dc_device_t *device)
 }
 
 dc_status_t
-dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descriptor, const char *name)
+dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descriptor, dc_iostream_t *iostream)
 {
 	dc_status_t rc = DC_STATUS_SUCCESS;
 	dc_device_t *device = NULL;
@@ -110,106 +118,130 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 
 	switch (dc_descriptor_get_type (descriptor)) {
 	case DC_FAMILY_SUUNTO_SOLUTION:
-		rc = suunto_solution_device_open (&device, context, name);
+		rc = suunto_solution_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_SUUNTO_EON:
-		rc = suunto_eon_device_open (&device, context, name);
+		rc = suunto_eon_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_SUUNTO_VYPER:
-		rc = suunto_vyper_device_open (&device, context, name);
+		rc = suunto_vyper_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_SUUNTO_VYPER2:
-		rc = suunto_vyper2_device_open (&device, context, name);
+		rc = suunto_vyper2_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_SUUNTO_D9:
-		rc = suunto_d9_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		rc = suunto_d9_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_SUUNTO_EONSTEEL:
-		rc = suunto_eonsteel_device_open (&device, context, dc_descriptor_get_model (descriptor));
+		rc = suunto_eonsteel_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_UWATEC_ALADIN:
-		rc = uwatec_aladin_device_open (&device, context, name);
+		rc = uwatec_aladin_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_UWATEC_MEMOMOUSE:
-		rc = uwatec_memomouse_device_open (&device, context, name);
+		rc = uwatec_memomouse_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_UWATEC_SMART:
-		rc = uwatec_smart_device_open (&device, context);
-		break;
-	case DC_FAMILY_UWATEC_MERIDIAN:
-		rc = uwatec_meridian_device_open (&device, context, name);
-		break;
-	case DC_FAMILY_UWATEC_G2:
-		rc = uwatec_g2_device_open (&device, context, dc_descriptor_get_model (descriptor));
+		rc = uwatec_smart_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_REEFNET_SENSUS:
-		rc = reefnet_sensus_device_open (&device, context, name);
+		rc = reefnet_sensus_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_REEFNET_SENSUSPRO:
-		rc = reefnet_sensuspro_device_open (&device, context, name);
+		rc = reefnet_sensuspro_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_REEFNET_SENSUSULTRA:
-		rc = reefnet_sensusultra_device_open (&device, context, name);
+		rc = reefnet_sensusultra_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_OCEANIC_VTPRO:
-		rc = oceanic_vtpro_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		rc = oceanic_vtpro_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_OCEANIC_VEO250:
-		rc = oceanic_veo250_device_open (&device, context, name);
+		rc = oceanic_veo250_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_OCEANIC_ATOM2:
-		rc = oceanic_atom2_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		rc = oceanic_atom2_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_MARES_NEMO:
-		rc = mares_nemo_device_open (&device, context, name);
+		rc = mares_nemo_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_MARES_PUCK:
-		rc = mares_puck_device_open (&device, context, name);
+		rc = mares_puck_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_MARES_DARWIN:
-		rc = mares_darwin_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		rc = mares_darwin_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_MARES_ICONHD:
-		rc = mares_iconhd_device_open (&device, context, name);
+		rc = mares_iconhd_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_HW_OSTC:
-		rc = hw_ostc_device_open (&device, context, name);
+		rc = hw_ostc_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_HW_FROG:
-		rc = hw_frog_device_open (&device, context, name);
+		rc = hw_frog_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_HW_OSTC3:
-		rc = hw_ostc3_device_open (&device, context, name);
+		rc = hw_ostc3_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_CRESSI_EDY:
-		rc = cressi_edy_device_open (&device, context, name);
+		rc = cressi_edy_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_CRESSI_LEONARDO:
-		rc = cressi_leonardo_device_open (&device, context, name);
+		rc = cressi_leonardo_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_CRESSI_GOA:
+		rc = cressi_goa_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_ZEAGLE_N2ITION3:
-		rc = zeagle_n2ition3_device_open (&device, context, name);
+		rc = zeagle_n2ition3_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_ATOMICS_COBALT:
-		rc = atomics_cobalt_device_open (&device, context);
+		rc = atomics_cobalt_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_SHEARWATER_PREDATOR:
-		rc = shearwater_predator_device_open (&device, context, name);
+		rc = shearwater_predator_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_SHEARWATER_PETREL:
-		rc = shearwater_petrel_device_open (&device, context, name);
+		rc = shearwater_petrel_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_DIVERITE_NITEKQ:
-		rc = diverite_nitekq_device_open (&device, context, name);
+		rc = diverite_nitekq_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_CITIZEN_AQUALAND:
-		rc = citizen_aqualand_device_open (&device, context, name);
+		rc = citizen_aqualand_device_open (&device, context, iostream);
 		break;
 	case DC_FAMILY_DIVESYSTEM_IDIVE:
-		rc = divesystem_idive_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		rc = divesystem_idive_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_COCHRAN_COMMANDER:
-		rc = cochran_commander_device_open (&device, context, name);
+		rc = cochran_commander_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_TECDIVING_DIVECOMPUTEREU:
+		rc = tecdiving_divecomputereu_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_MCLEAN_EXTREME:
+		rc = mclean_extreme_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_LIQUIVISION_LYNX:
+		rc = liquivision_lynx_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_SPORASUB_SP2:
+		rc = sporasub_sp2_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_DEEPSIX_EXCURSION:
+		rc = deepsix_excursion_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_SEAC_SCREEN:
+		rc = seac_screen_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_DEEPBLU_COSMIQ:
+		rc = deepblu_cosmiq_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_OCEANS_S1:
+		rc = oceans_s1_device_open (&device, context, iostream);
+		break;
+	case DC_FAMILY_DIVESOFT_FREEDOM:
+		rc = divesoft_freedom_device_open (&device, context, iostream);
 		break;
 	default:
 		return DC_STATUS_INVALIDARGS;
@@ -316,12 +348,17 @@ dc_device_dump (dc_device_t *device, dc_buffer_t *buffer)
 	if (device->vtable->dump == NULL)
 		return DC_STATUS_UNSUPPORTED;
 
+	if (buffer == NULL)
+		return DC_STATUS_INVALIDARGS;
+
+	dc_buffer_clear (buffer);
+
 	return device->vtable->dump (device, buffer);
 }
 
 
 dc_status_t
-device_dump_read (dc_device_t *device, unsigned char data[], unsigned int size, unsigned int blocksize)
+device_dump_read (dc_device_t *device, unsigned int address, unsigned char data[], unsigned int size, unsigned int blocksize)
 {
 	if (device == NULL)
 		return DC_STATUS_UNSUPPORTED;
@@ -342,7 +379,7 @@ device_dump_read (dc_device_t *device, unsigned char data[], unsigned int size,
 			len = blocksize;
 
 		// Read the packet.
-		dc_status_t rc = device->vtable->read (device, nbytes, data + nbytes, len);
+		dc_status_t rc = device->vtable->read (device, address + nbytes, data + nbytes, len);
 		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
@@ -379,6 +416,9 @@ dc_device_timesync (dc_device_t *device, const dc_datetime_t *datetime)
 	if (device->vtable->timesync == NULL)
 		return DC_STATUS_UNSUPPORTED;
 
+	if (datetime == NULL)
+		return DC_STATUS_INVALIDARGS;
+
 	return device->vtable->timesync (device, datetime);
 }
 
