@@ -23,6 +23,7 @@
 #define DIVERITE_NITEKQ_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 
@@ -31,7 +32,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-diverite_nitekq_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+diverite_nitekq_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
 diverite_nitekq_parser_create (dc_parser_t **parser, dc_context_t *context);
