@@ -23,6 +23,7 @@
 #define HW_OSTC_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 #include <libdivecomputer/hw_ostc.h>
@@ -32,10 +33,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-hw_ostc_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+hw_ostc_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-hw_ostc_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int hwos);
+hw_ostc_parser_create (dc_parser_t **parser, dc_context_t *context);
 
 #ifdef __cplusplus
 }
