@@ -27,13 +27,17 @@
 #include "hw_ostc3.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "array.h"
 #include "aes.h"
 #include "platform.h"
+#include "packet.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &hw_ostc3_device_vtable)
 
+#define OSTC3FW(major,minor) ( \
+		(((major) & 0xFF) << 8) | \
+		((minor) & 0xFF))
+
 #define SZ_DISPLAY    16
 #define SZ_CUSTOMTEXT 60
 #define SZ_VERSION    (SZ_CUSTOMTEXT + 4)
@@ -44,6 +48,7 @@
 #define SZ_FWINFO     4
 #define SZ_FIRMWARE   0x01E000        // 120KB
 #define SZ_FIRMWARE_BLOCK    0x1000   //   4KB
+#define SZ_FIRMWARE_BLOCK2   0x0100   //  256B
 #define FIRMWARE_AREA      0x3E0000
 
 #define RB_LOGBOOK_SIZE_COMPACT  16
@@ -52,6 +57,7 @@
 
 #define S_BLOCK_READ 0x20
 #define S_BLOCK_WRITE 0x30
+#define S_BLOCK_WRITE2 0x31
 #define S_ERASE    0x42
 #define S_READY    0x4C
 #define READY      0x4D
@@ -70,6 +76,7 @@
 #define S_UPLOAD   0x73
 #define WRITE      0x77
 #define RESET      0x78
+#define S_INIT     0xAA
 #define INIT       0xBB
 #define EXIT       0xFF
 
@@ -81,6 +88,20 @@
 #define CR         0x05
 
 #define NODELAY 0
+#define TIMEOUT 400
+
+#define HDR_COMPACT_LENGTH   0 // 3 bytes
+#define HDR_COMPACT_SUMMARY  3 // 10 bytes
+#define HDR_COMPACT_NUMBER  13 // 2 bytes
+#define HDR_COMPACT_VERSION 15 // 1 byte
+
+#define HDR_FULL_LENGTH      9 // 3 bytes
+#define HDR_FULL_SUMMARY    12 // 10 bytes
+#define HDR_FULL_NUMBER     80 // 2 bytes
+#define HDR_FULL_VERSION     8 // 1 byte
+
+#define HDR_FULL_POINTERS    2 // 6 bytes
+#define HDR_FULL_FIRMWARE   48 // 2 bytes
 
 typedef enum hw_ostc3_state_t {
 	OPEN,
@@ -91,10 +112,12 @@ typedef enum hw_ostc3_state_t {
 
 typedef struct hw_ostc3_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned int hardware;
 	unsigned int feature;
 	unsigned int model;
+	unsigned int serial;
+	unsigned int firmware;
 	unsigned char fingerprint[5];
 	hw_ostc3_state_t state;
 } hw_ostc3_device_t;
@@ -104,6 +127,7 @@ typedef struct hw_ostc3_logbook_t {
 	unsigned int profile;
 	unsigned int fingerprint;
 	unsigned int number;
+	unsigned int version;
 } hw_ostc3_logbook_t;
 
 typedef struct hw_ostc3_firmware_t {
@@ -111,8 +135,8 @@ typedef struct hw_ostc3_firmware_t {
 	unsigned int checksum;
 } hw_ostc3_firmware_t;
 
-// This key is used both for the Ostc3 and its cousin,
-// the Ostc Sport.
+// This key is used both for the OSTC3 and its cousin,
+// the OSTC Sport.
 // The Frog uses a similar protocol, and with another key.
 static const unsigned char ostc3_key[16] = {
 	0xF1, 0xE9, 0xB0, 0x30,
@@ -143,16 +167,18 @@ static const dc_device_vtable_t hw_ostc3_device_vtable = {
 
 static const hw_ostc3_logbook_t hw_ostc3_logbook_compact = {
 	RB_LOGBOOK_SIZE_COMPACT, /* size */
-	0,  /* profile */
-	3,  /* fingerprint */
-	13, /* number */
+	HDR_COMPACT_LENGTH,      /* profile */
+	HDR_COMPACT_SUMMARY,     /* fingerprint */
+	HDR_COMPACT_NUMBER,      /* number */
+	HDR_COMPACT_VERSION,     /* version */
 };
 
 static const hw_ostc3_logbook_t hw_ostc3_logbook_full = {
 	RB_LOGBOOK_SIZE_FULL, /* size */
-	9,  /* profile */
-	12, /* fingerprint */
-	80, /* number */
+	HDR_FULL_LENGTH,      /* profile */
+	HDR_FULL_SUMMARY,     /* fingerprint */
+	HDR_FULL_NUMBER,      /* number */
+	HDR_FULL_VERSION,     /* version */
 };
 
 
@@ -175,6 +201,67 @@ hw_ostc3_strncpy (unsigned char *data, unsigned int size, const char *text)
 	return 0;
 }
 
+static dc_status_t
+hw_ostc3_read (hw_ostc3_device_t *device, dc_event_progress_t *progress, unsigned char data[], size_t size)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+
+	size_t nbytes = 0;
+	while (nbytes < size) {
+		// Set the minimum packet size.
+		size_t length = 1024;
+
+		// Limit the packet size to the total size.
+		if (nbytes + length > size)
+			length = size - nbytes;
+
+		// Read the packet.
+		rc = dc_iostream_read (device->iostream, data + nbytes, length, NULL);
+		if (rc != DC_STATUS_SUCCESS)
+			return rc;
+
+		// Update and emit a progress event.
+		if (progress) {
+			progress->current += length;
+			device_event_emit ((dc_device_t *) device, DC_EVENT_PROGRESS, progress);
+		}
+
+		nbytes += length;
+	}
+
+	return rc;
+}
+
+static dc_status_t
+hw_ostc3_write (hw_ostc3_device_t *device, dc_event_progress_t *progress, const unsigned char data[], size_t size)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+
+	size_t nbytes = 0;
+	while (nbytes < size) {
+		// Set the maximum packet size.
+		size_t length = 1024;
+
+		// Limit the packet size to the total size.
+		if (nbytes + length > size)
+			length = size - nbytes;
+
+		// Write the packet.
+		rc = dc_iostream_write (device->iostream, data + nbytes, length, NULL);
+		if (rc != DC_STATUS_SUCCESS)
+			return rc;
+
+		// Update and emit a progress event.
+		if (progress) {
+			progress->current += length;
+			device_event_emit ((dc_device_t *) device, DC_EVENT_PROGRESS, progress);
+		}
+
+		nbytes += length;
+	}
+
+	return rc;
+}
 
 static dc_status_t
 hw_ostc3_transfer (hw_ostc3_device_t *device,
@@ -184,10 +271,15 @@ hw_ostc3_transfer (hw_ostc3_device_t *device,
                   unsigned int isize,
                   unsigned char output[],
                   unsigned int osize,
+                  unsigned int *actual,
                   unsigned int delay)
 {
 	dc_device_t *abstract = (dc_device_t *) device;
 	dc_status_t status = DC_STATUS_SUCCESS;
+	unsigned int length = osize;
+
+	if (cmd == DIVE && length < RB_LOGBOOK_SIZE_FULL)
+		return DC_STATUS_INVALIDARGS;
 
 	if (device_is_cancelled (abstract))
 		return DC_STATUS_CANCELLED;
@@ -197,7 +289,7 @@ hw_ostc3_transfer (hw_ostc3_device_t *device,
 
 	// Send the command.
 	unsigned char command[1] = {cmd};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -205,7 +297,7 @@ hw_ostc3_transfer (hw_ostc3_device_t *device,
 
 	// Read the echo.
 	unsigned char echo[1] = {0};
-	status = dc_serial_read (device->port, echo, sizeof (echo), NULL);
+	status = dc_iostream_read (device->iostream, echo, sizeof (echo), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the echo.");
 		return status;
@@ -223,82 +315,82 @@ hw_ostc3_transfer (hw_ostc3_device_t *device,
 	}
 
 	if (input) {
-		// Send the input data packet.
-		unsigned int nbytes = 0;
-		while (nbytes < isize) {
-			// Set the minimum packet size.
-			unsigned int len = 64;
-
-			// Limit the packet size to the total size.
-			if (nbytes + len > isize)
-				len = isize - nbytes;
-
-			// Write the packet.
-			status = dc_serial_write (device->port, input + nbytes, len, NULL);
+		if (cmd == WRITE) {
+			// Send the first byte of the input data packet.
+			status = hw_ostc3_write (device, progress, input, 1);
 			if (status != DC_STATUS_SUCCESS) {
 				ERROR (abstract->context, "Failed to send the data packet.");
 				return status;
 			}
 
-			// Update and emit a progress event.
-			if (progress) {
-				progress->current += len;
-				device_event_emit ((dc_device_t *) device, DC_EVENT_PROGRESS, progress);
-			}
+			dc_iostream_sleep (device->iostream, 10);
 
-			nbytes += len;
+			// Send the reamainder of the input data packet.
+			status = hw_ostc3_write (device, progress, input + 1, isize - 1);
+			if (status != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to send the data packet.");
+				return status;
+			}
+		} else {
+			// Send the input data packet.
+			status = hw_ostc3_write (device, progress, input, isize);
+			if (status != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to send the data packet.");
+				return status;
+			}
 		}
 	}
 
 	if (output) {
-		unsigned int nbytes = 0;
-		while (nbytes < osize) {
-			// Set the minimum packet size.
-			unsigned int len = 1024;
-
-			// Increase the packet size if more data is immediately available.
-			size_t available = 0;
-			status = dc_serial_get_available (device->port, &available);
-			if (status == DC_STATUS_SUCCESS && available > len)
-				len = available;
-
-			// Limit the packet size to the total size.
-			if (nbytes + len > osize)
-				len = osize - nbytes;
-
-			// Read the packet.
-			status = dc_serial_read (device->port, output + nbytes, len, NULL);
+		if (cmd == DIVE) {
+			// Read the dive header.
+			status = hw_ostc3_read (device, progress, output, RB_LOGBOOK_SIZE_FULL);
 			if (status != DC_STATUS_SUCCESS) {
-				ERROR (abstract->context, "Failed to receive the answer.");
+				ERROR (abstract->context, "Failed to receive the dive header.");
+				return status;
+			}
+
+			// When the hwOS firmware detects the dive profile is no longer
+			// valid, it sends a modified dive header (with the begin/end
+			// pointer fields reset to zero, and the length field reduced to 8
+			// bytes), along with an empty dive profile. Detect this condition
+			// and adjust the expected length.
+			if (array_isequal (output + HDR_FULL_POINTERS, 6, 0x00) &&
+				array_uint24_le (output + HDR_FULL_LENGTH) == 8 &&
+				length > RB_LOGBOOK_SIZE_FULL + 5) {
+				length = RB_LOGBOOK_SIZE_FULL + 5;
+			}
+
+			// Read the dive profile.
+			status = hw_ostc3_read (device, progress, output + RB_LOGBOOK_SIZE_FULL, length - RB_LOGBOOK_SIZE_FULL);
+			if (status != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to receive the dive profile.");
 				return status;
 			}
 
 			// Update and emit a progress event.
-			if (progress) {
-				progress->current += len;
+			if (progress && osize > length) {
+				progress->current += osize - length;
 				device_event_emit ((dc_device_t *) device, DC_EVENT_PROGRESS, progress);
 			}
-
-			nbytes += len;
+		} else {
+			// Read the output data packet.
+			status = hw_ostc3_read (device, progress, output, length);
+			if (status != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to receive the answer.");
+				return status;
+			}
 		}
 	}
 
 	if (delay) {
-		unsigned int count = delay / 100;
-		for (unsigned int i = 0; i < count; ++i) {
-			size_t available = 0;
-			status = dc_serial_get_available (device->port, &available);
-			if (status == DC_STATUS_SUCCESS && available > 0)
-				break;
-
-			dc_serial_sleep (device->port, 100);
-		}
+		dc_iostream_poll (device->iostream, delay);
 	}
 
 	if (cmd != EXIT) {
 		// Read the ready byte.
 		unsigned char answer[1] = {0};
-		status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+		status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the ready byte.");
 			return status;
@@ -311,15 +403,19 @@ hw_ostc3_transfer (hw_ostc3_device_t *device,
 		}
 	}
 
+	if (actual)
+		*actual = length;
+
 	return DC_STATUS_SUCCESS;
 }
 
 
 dc_status_t
-hw_ostc3_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+hw_ostc3_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc3_device_t *device = NULL;
+	dc_transport_t transport = dc_iostream_get_transport (iostream);
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
@@ -332,36 +428,41 @@ hw_ostc3_device_open (dc_device_t **out, dc_context_t *context, const char *name
 	}
 
 	// Set the default values.
-	device->port = NULL;
 	device->hardware = INVALID;
 	device->feature = 0;
 	device->model = 0;
+	device->serial = 0;
+	device->firmware = 0;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
-		goto error_free;
+	// Create the packet stream.
+	if (transport == DC_TRANSPORT_BLE) {
+		status = dc_packet_open (&device->iostream, context, iostream, 244, 20);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (context, "Failed to create the packet stream.");
+			goto error_free;
+		}
+	} else {
+		device->iostream = iostream;
 	}
 
 	// Set the serial communication protocol (115200 8N1).
-	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
+		goto error_free_iostream;
 	}
 
 	// Set the timeout for receiving data (3000ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free_iostream;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_sleep (device->port, 300);
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_sleep (device->iostream, 300);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	device->state = OPEN;
 
@@ -369,8 +470,10 @@ hw_ostc3_device_open (dc_device_t **out, dc_context_t *context, const char *name
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
+error_free_iostream:
+	if (transport == DC_TRANSPORT_BLE) {
+		dc_iostream_close (device->iostream);
+	}
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -387,9 +490,9 @@ hw_ostc3_device_id (hw_ostc3_device_t *device, unsigned char data[], unsigned in
 
 	// Send the command.
 	unsigned char hardware[SZ_HARDWARE2] = {0};
-	status = hw_ostc3_transfer (device, NULL, HARDWARE2, NULL, 0, hardware, SZ_HARDWARE2, NODELAY);
+	status = hw_ostc3_transfer (device, NULL, HARDWARE2, NULL, 0, hardware, SZ_HARDWARE2, NULL, NODELAY);
 	if (status == DC_STATUS_UNSUPPORTED) {
-		status = hw_ostc3_transfer (device, NULL, HARDWARE, NULL, 0, hardware + 1, SZ_HARDWARE, NODELAY);
+		status = hw_ostc3_transfer (device, NULL, HARDWARE, NULL, 0, hardware + 1, SZ_HARDWARE, NULL, NODELAY);
 	}
 	if (status != DC_STATUS_SUCCESS)
 		return status;
@@ -411,7 +514,7 @@ hw_ostc3_device_init_download (hw_ostc3_device_t *device)
 	dc_context_t *context = (abstract ? abstract->context : NULL);
 
 	// Send the init command.
-	dc_status_t status = hw_ostc3_transfer (device, NULL, INIT, NULL, 0, NULL, 0, NODELAY);
+	dc_status_t status = hw_ostc3_transfer (device, NULL, INIT, NULL, 0, NULL, 0, NULL, NODELAY);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to send the command.");
 		return status;
@@ -428,33 +531,29 @@ hw_ostc3_device_init_service (hw_ostc3_device_t *device)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
-	dc_context_t *context = (abstract ? abstract->context : NULL);
 
-	unsigned char command[] = {0xAA, 0xAB, 0xCD, 0xEF};
-	unsigned char output[5];
+	const unsigned char command[] = {S_INIT, 0xAB, 0xCD, 0xEF};
+	unsigned char answer[5] = {0};
 
-	// We cant use hw_ostc3_transfer here, due to the different echos
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	// Send the command and service key.
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to send the command.");
+		ERROR (abstract->context, "Failed to send the command.");
 		return status;
 	}
 
-	// Give the device some time to enter service mode
-	dc_serial_sleep (device->port, 100);
-
-	// Read the response
-	status = dc_serial_read (device->port, output, sizeof (output), NULL);
+	// Read the response.
+	status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to receive the echo.");
+		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
 	}
 
-	// Verify the response to service mode
-	if (output[0] != 0x4B || output[1] != 0xAB ||
-			output[2] != 0xCD || output[3] != 0xEF ||
-			output[4] != S_READY) {
-		ERROR (context, "Failed to verify echo.");
+	// Verify the response to service mode.
+	if (answer[0] != 0x4B || answer[1] != 0xAB ||
+			answer[2] != 0xCD || answer[3] != 0xEF ||
+			answer[4] != S_READY) {
+		ERROR (abstract->context, "Failed to verify the answer.");
 		return DC_STATUS_PROTOCOL;
 	}
 
@@ -506,10 +605,24 @@ hw_ostc3_device_init (hw_ostc3_device_t *device, hw_ostc3_state_t state)
 		return rc;
 	}
 
+	// Read the version information.
+	unsigned char version[SZ_VERSION] = {0};
+	rc = hw_ostc3_transfer (device, NULL, IDENTITY, NULL, 0, version, sizeof(version), NULL, NODELAY);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the version information.");
+		return rc;
+	}
+
 	// Cache the descriptor.
 	device->hardware = array_uint16_be(hardware + 0);
 	device->feature = array_uint16_be(hardware + 2);
 	device->model = hardware[4];
+	device->serial = array_uint16_le (version + 0);
+	if (device->hardware == OSTC4) {
+		device->firmware = array_uint16_le (version + 2);
+	} else {
+		device->firmware = array_uint16_be (version + 2);
+	}
 
 	return DC_STATUS_SUCCESS;
 }
@@ -524,17 +637,20 @@ hw_ostc3_device_close (dc_device_t *abstract)
 
 	// Send the exit command
 	if (device->state == DOWNLOAD || device->state == SERVICE) {
-		rc = hw_ostc3_transfer (device, NULL, EXIT, NULL, 0, NULL, 0, NODELAY);
+		rc = hw_ostc3_transfer (device, NULL, EXIT, NULL, 0, NULL, 0, NULL, NODELAY);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to send the command.");
 			dc_status_set_error(&status, rc);
 		}
 	}
 
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
+	// Close the packet stream.
+	if (dc_iostream_get_transport (device->iostream) == DC_TRANSPORT_BLE) {
+		rc = dc_iostream_close (device->iostream);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to close the packet stream.");
+			dc_status_set_error(&status, rc);
+		}
 	}
 
 	return status;
@@ -574,7 +690,7 @@ hw_ostc3_device_version (dc_device_t *abstract, unsigned char data[], unsigned i
 		return rc;
 
 	// Send the command.
-	rc = hw_ostc3_transfer (device, NULL, IDENTITY, NULL, 0, data, size, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, IDENTITY, NULL, 0, data, size, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -620,22 +736,10 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	// Download the version data.
-	unsigned char id[SZ_VERSION] = {0};
-	rc = hw_ostc3_device_version (abstract, id, sizeof (id));
-	if (rc != DC_STATUS_SUCCESS) {
-		ERROR (abstract->context, "Failed to read the version.");
-		return rc;
-	}
-
 	// Emit a device info event.
 	dc_event_devinfo_t devinfo;
-	if (device->hardware == OSTC4) {
-		devinfo.firmware = array_uint16_le (id + 2);
-	} else {
-		devinfo.firmware = array_uint16_be (id + 2);
-	}
-	devinfo.serial = array_uint16_le (id + 0);
+	devinfo.firmware = device->firmware;
+	devinfo.serial = device->serial;
 	if (device->hardware != UNKNOWN) {
 		devinfo.model = device->hardware;
 	} else {
@@ -659,11 +763,11 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 	// This is slower, but also works for older firmware versions.
 	unsigned int compact = 1;
 	rc = hw_ostc3_transfer (device, &progress, COMPACT,
-              NULL, 0, header, RB_LOGBOOK_SIZE_COMPACT * RB_LOGBOOK_COUNT, NODELAY);
+              NULL, 0, header, RB_LOGBOOK_SIZE_COMPACT * RB_LOGBOOK_COUNT, NULL, NODELAY);
 	if (rc == DC_STATUS_UNSUPPORTED) {
 		compact = 0;
 		rc = hw_ostc3_transfer (device, &progress, HEADER,
-		          NULL, 0, header, RB_LOGBOOK_SIZE_FULL * RB_LOGBOOK_COUNT, NODELAY);
+		          NULL, 0, header, RB_LOGBOOK_SIZE_FULL * RB_LOGBOOK_COUNT, NULL, NODELAY);
 	}
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to read the header.");
@@ -683,7 +787,6 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 	// The device maintains an internal counter which is incremented for every
 	// dive, and the current value at the time of the dive is stored in the
 	// dive header. Thus the most recent dive will have the highest value.
-	unsigned int count = 0;
 	unsigned int latest = 0;
 	unsigned int maximum = 0;
 	for (unsigned int i = 0; i < RB_LOGBOOK_COUNT; ++i) {
@@ -695,38 +798,40 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 
 		// Get the internal dive number.
 		unsigned int current = array_uint16_le (header + offset + logbook->number);
-		if (current > maximum) {
+		if (current > maximum || device->hardware == OSTC4) {
 			maximum = current;
 			latest = i;
 		}
-
-		count++;
 	}
 
 	// Calculate the total and maximum size.
 	unsigned int ndives = 0;
 	unsigned int size = 0;
 	unsigned int maxsize = 0;
-	for (unsigned int i = 0; i < count; ++i) {
+	unsigned char dive[RB_LOGBOOK_COUNT] = {0};
+	for (unsigned int i = 0; i < RB_LOGBOOK_COUNT; ++i) {
 		unsigned int idx = (latest + RB_LOGBOOK_COUNT - i) % RB_LOGBOOK_COUNT;
 		unsigned int offset = idx * logbook->size;
 
-		// Uninitialized header entries should no longer be present at this
-		// stage, unless the dives are interleaved with empty entries. But
-		// that's something we don't support at all.
+		// Ignore uninitialized header entries.
 		if (array_isequal (header + offset, logbook->size, 0xFF)) {
 			WARNING (abstract->context, "Unexpected empty header found.");
-			break;
+			continue;
 		}
 
 		// Calculate the profile length.
 		unsigned int length = RB_LOGBOOK_SIZE_FULL + array_uint24_le (header + offset + logbook->profile) - 3;
 		if (!compact) {
 			// Workaround for a bug in older firmware versions.
-			unsigned int firmware = array_uint16_be (header + offset + 0x30);
-			if (firmware < 93)
+			unsigned int firmware = array_uint16_be (header + offset + HDR_FULL_FIRMWARE);
+			if (firmware < OSTC3FW(0,93))
 				length -= 3;
 		}
+		if (length < RB_LOGBOOK_SIZE_FULL) {
+			ERROR (abstract->context, "Invalid profile length (%u bytes).", length);
+			free (header);
+			return DC_STATUS_DATAFORMAT;
+		}
 
 		// Check the fingerprint data.
 		if (memcmp (header + offset + logbook->fingerprint, device->fingerprint, sizeof (device->fingerprint)) == 0)
@@ -735,6 +840,7 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 		if (length > maxsize)
 			maxsize = length;
 		size += length;
+		dive[ndives] = idx;
 		ndives++;
 	}
 
@@ -758,22 +864,22 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 
 	// Download the dives.
 	for (unsigned int i = 0; i < ndives; ++i) {
-		unsigned int idx = (latest + RB_LOGBOOK_COUNT - i) % RB_LOGBOOK_COUNT;
+		unsigned int idx = dive[i];
 		unsigned int offset = idx * logbook->size;
 
 		// Calculate the profile length.
 		unsigned int length = RB_LOGBOOK_SIZE_FULL + array_uint24_le (header + offset + logbook->profile) - 3;
 		if (!compact) {
 			// Workaround for a bug in older firmware versions.
-			unsigned int firmware = array_uint16_be (header + offset + 0x30);
-			if (firmware < 93)
+			unsigned int firmware = array_uint16_be (header + offset + HDR_FULL_FIRMWARE);
+			if (firmware < OSTC3FW(0,93))
 				length -= 3;
 		}
 
 		// Download the dive.
 		unsigned char number[1] = {idx};
 		rc = hw_ostc3_transfer (device, &progress, DIVE,
-			number, sizeof (number), profile, length, NODELAY);
+			number, sizeof (number), profile, length, &length, NODELAY);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to read the dive.");
 			free (profile);
@@ -782,14 +888,38 @@ hw_ostc3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, voi
 		}
 
 		// Verify the header in the logbook and profile are identical.
-		if (!compact && memcmp (profile, header + offset, logbook->size) != 0) {
+		if (memcmp (profile + HDR_FULL_VERSION, header + offset + logbook->version, 1) != 0 ||
+			compact ?
+			memcmp (profile + HDR_FULL_SUMMARY, header + offset + HDR_COMPACT_SUMMARY, 10) != 0 ||
+			memcmp (profile + HDR_FULL_NUMBER, header + offset + HDR_COMPACT_NUMBER, 2) != 0 :
+			memcmp (profile + HDR_FULL_SUMMARY, header + offset + HDR_FULL_SUMMARY, RB_LOGBOOK_SIZE_FULL - HDR_FULL_SUMMARY) != 0) {
 			ERROR (abstract->context, "Unexpected profile header.");
 			free (profile);
 			free (header);
-			return rc;
+			return DC_STATUS_DATAFORMAT;
 		}
 
-		if (callback && !callback (profile, length, profile + 12, sizeof (device->fingerprint), userdata))
+		// Detect invalid profile data.
+		unsigned int delta = device->hardware == OSTC4 ? 3 : 0;
+		if (length < RB_LOGBOOK_SIZE_FULL + 2 ||
+			profile[length - 2] != 0xFD || profile[length - 1] != 0xFD) {
+			// A valid profile should have at least a correct 2 byte
+			// end-of-profile marker.
+			WARNING (abstract->context, "Invalid profile end marker detected!");
+			length = RB_LOGBOOK_SIZE_FULL;
+		} else if (length == RB_LOGBOOK_SIZE_FULL + 2) {
+			// A profile containing only the 2 byte end-of-profile
+			// marker is considered a valid empty profile.
+		} else if (length < RB_LOGBOOK_SIZE_FULL + 5 ||
+			array_uint24_le (profile + RB_LOGBOOK_SIZE_FULL) + delta != array_uint24_le (profile + HDR_FULL_LENGTH)) {
+			// If there is more data available, then there should be a
+			// valid profile header containing a length matching the
+			// length in the dive header.
+			WARNING (abstract->context, "Invalid profile header detected.");
+			length = RB_LOGBOOK_SIZE_FULL;
+		}
+
+		if (callback && !callback (profile, length, profile + HDR_FULL_SUMMARY, sizeof (device->fingerprint), userdata))
 			break;
 	}
 
@@ -805,11 +935,6 @@ hw_ostc3_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime)
 {
 	hw_ostc3_device_t *device = (hw_ostc3_device_t *) abstract;
 
-	if (datetime == NULL) {
-		ERROR (abstract->context, "Invalid parameter specified.");
-		return DC_STATUS_INVALIDARGS;
-	}
-
 	dc_status_t rc = hw_ostc3_device_init (device, DOWNLOAD);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
@@ -818,7 +943,7 @@ hw_ostc3_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime)
 	unsigned char packet[6] = {
 		datetime->hour, datetime->minute, datetime->second,
 		datetime->month, datetime->day, datetime->year - 2000};
-	rc = hw_ostc3_transfer (device, NULL, CLOCK, packet, sizeof (packet), NULL, 0, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, CLOCK, packet, sizeof (packet), NULL, 0, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -846,7 +971,7 @@ hw_ostc3_device_display (dc_device_t *abstract, const char *text)
 		return rc;
 
 	// Send the command.
-	rc = hw_ostc3_transfer (device, NULL, DISPLAY, packet, sizeof (packet), NULL, 0, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, DISPLAY, packet, sizeof (packet), NULL, 0, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -874,7 +999,7 @@ hw_ostc3_device_customtext (dc_device_t *abstract, const char *text)
 		return rc;
 
 	// Send the command.
-	rc = hw_ostc3_transfer (device, NULL, CUSTOMTEXT, packet, sizeof (packet), NULL, 0, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, CUSTOMTEXT, packet, sizeof (packet), NULL, 0, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -900,7 +1025,7 @@ hw_ostc3_device_config_read (dc_device_t *abstract, unsigned int config, unsigne
 
 	// Send the command.
 	unsigned char command[1] = {config};
-	rc = hw_ostc3_transfer (device, NULL, READ, command, sizeof (command), data, size, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, READ, command, sizeof (command), data, size, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -927,7 +1052,7 @@ hw_ostc3_device_config_write (dc_device_t *abstract, unsigned int config, const
 	// Send the command.
 	unsigned char command[SZ_CONFIG + 1] = {config};
 	memcpy(command + 1, data, size);
-	rc = hw_ostc3_transfer (device, NULL, WRITE, command, size + 1, NULL, 0, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, WRITE, command, size + 1, NULL, 0, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -947,7 +1072,7 @@ hw_ostc3_device_config_reset (dc_device_t *abstract)
 		return rc;
 
 	// Send the command.
-	rc = hw_ostc3_transfer (device, NULL, RESET, NULL, 0, NULL, 0, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, RESET, NULL, 0, NULL, 0, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -974,7 +1099,7 @@ hw_ostc3_firmware_readline (FILE *fp, dc_context_t *context, unsigned int addr,
 	unsigned char ascii[39];
 	unsigned char faddr_byte[3];
 	unsigned int faddr = 0;
-	int n = 0;
+	size_t n = 0;
 
 	if (size > 16) {
 		ERROR (context, "Invalid arguments.");
@@ -1125,7 +1250,11 @@ hw_ostc3_firmware_readfile4 (dc_buffer_t *buffer, dc_context_t *context, const c
 	size_t n = 0;
 	unsigned char block[1024] = {0};
 	while ((n = fread (block, 1, sizeof (block), fp)) > 0) {
-		dc_buffer_append (buffer, block, n);
+		if (!dc_buffer_append (buffer, block, n)) {
+			ERROR (context, "Insufficient buffer space available.");
+			fclose (fp);
+			return DC_STATUS_NOMEMORY;
+		}
 	}
 
 	// Close the file.
@@ -1160,12 +1289,16 @@ hw_ostc3_firmware_erase (hw_ostc3_device_t *device, unsigned int addr, unsigned
 	// Convert size to number of pages, rounded up.
 	unsigned char blocks = ((size + SZ_FIRMWARE_BLOCK - 1) / SZ_FIRMWARE_BLOCK);
 
+	// Estimate the required delay. Erasing a 4K flash memory page
+	// takes around 25 milliseconds.
+	unsigned int delay = blocks * 25;
+
 	// Erase just the needed pages.
 	unsigned char buffer[4];
 	array_uint24_be_set (buffer, addr);
 	buffer[3] = blocks;
 
-	return hw_ostc3_transfer (device, NULL, S_ERASE, buffer, sizeof (buffer), NULL, 0, NODELAY);
+	return hw_ostc3_transfer (device, NULL, S_ERASE, buffer, sizeof (buffer), NULL, 0, NULL, delay);
 }
 
 static dc_status_t
@@ -1175,22 +1308,63 @@ hw_ostc3_firmware_block_read (hw_ostc3_device_t *device, unsigned int addr, unsi
 	array_uint24_be_set (buffer, addr);
 	array_uint24_be_set (buffer + 3, block_size);
 
-	return hw_ostc3_transfer (device, NULL, S_BLOCK_READ, buffer, sizeof (buffer), block, block_size, NODELAY);
+	return hw_ostc3_transfer (device, NULL, S_BLOCK_READ, buffer, sizeof (buffer), block, block_size, NULL, NODELAY);
 }
 
 static dc_status_t
-hw_ostc3_firmware_block_write (hw_ostc3_device_t *device, unsigned int addr, const unsigned char block[], unsigned int block_size)
+hw_ostc3_firmware_block_write1 (hw_ostc3_device_t *device, unsigned int addr, const unsigned char block[], unsigned int block_size)
 {
 	unsigned char buffer[3 + SZ_FIRMWARE_BLOCK];
 
-	// We currenty only support writing max SZ_FIRMWARE_BLOCK sized blocks.
+	// We currently only support writing max SZ_FIRMWARE_BLOCK sized blocks.
 	if (block_size > SZ_FIRMWARE_BLOCK)
 		return DC_STATUS_INVALIDARGS;
 
 	array_uint24_be_set (buffer, addr);
 	memcpy (buffer + 3, block, block_size);
 
-	return hw_ostc3_transfer (device, NULL, S_BLOCK_WRITE, buffer, 3 + block_size, NULL, 0, NODELAY);
+	return hw_ostc3_transfer (device, NULL, S_BLOCK_WRITE, buffer, 3 + block_size, NULL, 0, NULL, TIMEOUT);
+}
+
+static dc_status_t
+hw_ostc3_firmware_block_write2 (hw_ostc3_device_t *device, unsigned int address, const unsigned char data[], unsigned int size)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+
+	if ((address % SZ_FIRMWARE_BLOCK2 != 0) ||
+		(size % SZ_FIRMWARE_BLOCK2 != 0)) {
+		return DC_STATUS_INVALIDARGS;
+	}
+
+	unsigned int nbytes = 0;
+	while (nbytes < size) {
+		unsigned char buffer[3 + SZ_FIRMWARE_BLOCK2];
+		array_uint24_be_set (buffer, address);
+		memcpy (buffer + 3, data + nbytes, SZ_FIRMWARE_BLOCK2);
+
+		status = hw_ostc3_transfer (device, NULL, S_BLOCK_WRITE2, buffer, sizeof(buffer), NULL, 0, NULL, NODELAY);
+		if (status != DC_STATUS_SUCCESS) {
+			return status;
+		}
+
+		address += SZ_FIRMWARE_BLOCK2;
+		nbytes += SZ_FIRMWARE_BLOCK2;
+	}
+
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t
+hw_ostc3_firmware_block_write (hw_ostc3_device_t *device, unsigned int address, const unsigned char data[], unsigned int size)
+{
+	// Support for the S_BLOCK_WRITE2 command is only available since the
+	// hwOS Tech firmware v3.09 and the hwOS Sport firmware v10.64.
+	if ((device->firmware < OSTC3FW(3,9)) ||
+		(device->firmware >= OSTC3FW(10,0) && device->firmware < OSTC3FW(10,64))) {
+		return hw_ostc3_firmware_block_write1 (device, address, data, size);
+	} else {
+		return hw_ostc3_firmware_block_write2 (device, address, data, size);
+	}
 }
 
 static dc_status_t
@@ -1209,7 +1383,7 @@ hw_ostc3_firmware_upgrade (dc_device_t *abstract, unsigned int checksum)
 		buffer[4]  = (buffer[4]<<1 | buffer[4]>>7);
 	}
 
-	rc = hw_ostc3_transfer (device, NULL, S_UPGRADE, buffer, sizeof (buffer), NULL, 0, NODELAY);
+	rc = hw_ostc3_transfer (device, NULL, S_UPGRADE, buffer, sizeof (buffer), NULL, 0, NULL, NODELAY);
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to send flash firmware command");
 		return rc;
@@ -1270,7 +1444,7 @@ hw_ostc3_device_fwupdate3 (dc_device_t *abstract, const char *filename)
 
 	for (unsigned int len = 0; len < SZ_FIRMWARE; len += SZ_FIRMWARE_BLOCK) {
 		char status[SZ_DISPLAY + 1]; // Status message on the display
-		snprintf (status, sizeof(status), " Uploading %2d%%", (100 * len) / SZ_FIRMWARE);
+		dc_platform_snprintf (status, sizeof(status), " Uploading %2d%%", (100 * len) / SZ_FIRMWARE);
 		hw_ostc3_device_display (abstract, status);
 
 		rc = hw_ostc3_firmware_block_write (device, FIRMWARE_AREA + len, firmware->data + len, SZ_FIRMWARE_BLOCK);
@@ -1289,7 +1463,7 @@ hw_ostc3_device_fwupdate3 (dc_device_t *abstract, const char *filename)
 	for (unsigned int len = 0; len < SZ_FIRMWARE; len += SZ_FIRMWARE_BLOCK) {
 		unsigned char block[SZ_FIRMWARE_BLOCK];
 		char status[SZ_DISPLAY + 1]; // Status message on the display
-		snprintf (status, sizeof(status), " Verifying %2d%%", (100 * len) / SZ_FIRMWARE);
+		dc_platform_snprintf (status, sizeof(status), " Verifying %2d%%", (100 * len) / SZ_FIRMWARE);
 		hw_ostc3_device_display (abstract, status);
 
 		rc = hw_ostc3_firmware_block_read (device, FIRMWARE_AREA + len, block, sizeof (block));
@@ -1391,7 +1565,7 @@ hw_ostc3_device_fwupdate4 (dc_device_t *abstract, const char *filename)
 		// Read the firmware version info.
 		unsigned char fwinfo[SZ_FWINFO] = {0};
 		status = hw_ostc3_transfer (device, NULL, S_FWINFO,
-			data + offset + 4, 1, fwinfo, sizeof(fwinfo), NODELAY);
+			data + offset + 4, 1, fwinfo, sizeof(fwinfo), NULL, NODELAY);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to read the firmware info.");
 			goto error;
@@ -1404,7 +1578,7 @@ hw_ostc3_device_fwupdate4 (dc_device_t *abstract, const char *filename)
 			!array_isequal(fwinfo, sizeof(fwinfo), 0xFF))
 		{
 			status = hw_ostc3_transfer (device, &progress, S_UPLOAD,
-				data + offset, length, NULL, 0, usecs / 1000);
+				data + offset, length, NULL, 0, NULL, usecs / 1000);
 			if (status != DC_STATUS_SUCCESS) {
 				goto error;
 			}
@@ -1530,12 +1704,6 @@ hw_ostc3_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	hw_ostc3_device_t *device = (hw_ostc3_device_t *) abstract;
 
-	// Erase the current contents of the buffer.
-	if (!dc_buffer_clear (buffer)) {
-		ERROR (abstract->context, "Insufficient buffer space available.");
-		return DC_STATUS_NOMEMORY;
-	}
-
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	progress.maximum = SZ_MEMORY;
@@ -1547,6 +1715,21 @@ hw_ostc3_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		return rc;
 	}
 
+	// Emit a device info event.
+	dc_event_devinfo_t devinfo;
+	devinfo.firmware = device->firmware;
+	devinfo.serial = device->serial;
+	if (device->hardware != UNKNOWN) {
+		devinfo.model = device->hardware;
+	} else {
+		// Fallback to the serial number.
+		if (devinfo.serial > 10000)
+			devinfo.model = SPORT;
+		else
+			devinfo.model = OSTC3;
+	}
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
 	// Allocate the required amount of memory.
 	if (!dc_buffer_resize (buffer, SZ_MEMORY)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
@@ -1557,7 +1740,7 @@ hw_ostc3_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	unsigned int nbytes = 0;
 	while (nbytes < SZ_MEMORY) {
-		// packet size. Can be almost arbetary size.
+		// packet size. Can be almost arbitrary size.
 		unsigned int len = SZ_FIRMWARE_BLOCK;
 
 		// Read a block
