@@ -23,6 +23,7 @@
 #define HW_OSTC3_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 #include <libdivecomputer/hw_ostc3.h>
@@ -32,7 +33,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-hw_ostc3_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+hw_ostc3_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
 hw_ostc3_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model);
