@@ -20,6 +20,7 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "libdivecomputer/units.h"
 
@@ -43,6 +44,14 @@
 #define HEADER  1
 #define PROFILE 2
 
+#define TEMPERATURE 0
+#define DECO        1
+#define GF          2
+#define PPO2        3
+#define DECOPLAN    4
+#define CNS         5
+#define TANK        6
+
 #define OSTC_ZHL16_OC    0
 #define OSTC_GAUGE       1
 #define OSTC_ZHL16_CC    2
@@ -59,9 +68,24 @@
 #define OSTC3_CC    1
 #define OSTC3_GAUGE 2
 #define OSTC3_APNEA 3
+#define OSTC3_PSCR  4
+
+#define OSTC3_ZHL16    0
+#define OSTC3_ZHL16_GF 1
+#define OSTC4_VPM      2
 
 #define OSTC4      0x3B
 
+#define OSTC3FW(major,minor) ( \
+		(((major) & 0xFF) << 8) | \
+		((minor) & 0xFF))
+
+#define OSTC4FW(major,minor,micro,beta) ( \
+		(((major) & 0x1F) << 11) | \
+		(((minor) & 0x1F) >> 6) | \
+		(((micro) & 0x1F) << 1) | \
+		((beta) & 0x01))
+
 typedef struct hw_ostc_sample_info_t {
 	unsigned int type;
 	unsigned int divisor;
@@ -71,18 +95,24 @@ typedef struct hw_ostc_sample_info_t {
 typedef struct hw_ostc_layout_t {
 	unsigned int datetime;
 	unsigned int maxdepth;
-	unsigned int avgdepth;
 	unsigned int divetime;
-	unsigned int atmospheric;
-	unsigned int salinity;
-	unsigned int duration;
 	unsigned int temperature;
+	unsigned int atmospheric;
 	unsigned int firmware;
+	unsigned int salinity;
+	unsigned int avgdepth;
+	unsigned int duration;
+	unsigned int gf;
+	unsigned int decomodel;
+	unsigned int divemode;
 } hw_ostc_layout_t;
 
 typedef struct hw_ostc_gasmix_t {
 	unsigned int oxygen;
 	unsigned int helium;
+	unsigned int type;
+	unsigned int enabled;
+	unsigned int diluent;
 } hw_ostc_gasmix_t;
 
 typedef struct hw_ostc_parser_t {
@@ -111,6 +141,9 @@ static const dc_parser_vtable_t hw_ostc_parser_vtable = {
 	sizeof(hw_ostc_parser_t),
 	DC_FAMILY_HW_OSTC,
 	hw_ostc_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	hw_ostc_parser_get_datetime, /* datetime */
 	hw_ostc_parser_get_field, /* fields */
 	hw_ostc_parser_samples_foreach, /* samples_foreach */
@@ -120,41 +153,50 @@ static const dc_parser_vtable_t hw_ostc_parser_vtable = {
 static const hw_ostc_layout_t hw_ostc_layout_ostc = {
 	3,  /* datetime */
 	8,  /* maxdepth */
-	45, /* avgdepth */
 	10, /* divetime */
-	15, /* atmospheric */
-	43, /* salinity */
-	47, /* duration */
 	13, /* temperature */
+	15, /* atmospheric */
 	32, /* firmware */
+	43, /* salinity */
+	45, /* avgdepth */
+	47, /* duration */
+	49, /* gf */
+	UNDEFINED, /* decomodel */
+	51, /* divemode */
 };
 
 static const hw_ostc_layout_t hw_ostc_layout_frog = {
 	9,  /* datetime */
 	14, /* maxdepth */
-	45, /* avgdepth */
 	16, /* divetime */
-	21, /* atmospheric */
-	43, /* salinity */
-	47, /* duration */
 	19, /* temperature */
+	21, /* atmospheric */
 	32, /* firmware */
+	43, /* salinity */
+	45, /* avgdepth */
+	47, /* duration */
+	49, /* gf */
+	UNDEFINED, /* decomodel */
+	51, /* divemode */
 };
 
 static const hw_ostc_layout_t hw_ostc_layout_ostc3 = {
 	12, /* datetime */
 	17, /* maxdepth */
-	73, /* avgdepth */
 	19, /* divetime */
-	24, /* atmospheric */
-	70, /* salinity */
-	75, /* duration */
 	22, /* temperature */
+	24, /* atmospheric */
 	48, /* firmware */
+	70, /* salinity */
+	73, /* avgdepth */
+	75, /* duration */
+	77, /* gf */
+	79, /* decomodel */
+	82, /* divemode */
 };
 
 static unsigned int
-hw_ostc_find_gasmix (hw_ostc_parser_t *parser, unsigned int o2, unsigned int he, unsigned int type)
+hw_ostc_find_gasmix (hw_ostc_parser_t *parser, unsigned int o2, unsigned int he, unsigned int dil, unsigned int type)
 {
 	unsigned int offset = 0;
 	unsigned int count = parser->ngasmixes;
@@ -166,7 +208,7 @@ hw_ostc_find_gasmix (hw_ostc_parser_t *parser, unsigned int o2, unsigned int he,
 
 	unsigned int i = offset;
 	while (i < count) {
-		if (o2 == parser->gasmix[i].oxygen && he == parser->gasmix[i].helium)
+		if (o2 == parser->gasmix[i].oxygen && he == parser->gasmix[i].helium && dil == parser->gasmix[i].diluent)
 			break;
 		i++;
 	}
@@ -174,6 +216,18 @@ hw_ostc_find_gasmix (hw_ostc_parser_t *parser, unsigned int o2, unsigned int he,
 	return i;
 }
 
+static unsigned int
+hw_ostc_is_ccr (unsigned int divemode, unsigned int version)
+{
+	if (version == 0x21) {
+		return divemode == OSTC_ZHL16_CC || divemode == OSTC_ZHL16_CC_GF || divemode == OSTC_PSCR_GF;
+	} else if (version == 0x23 || version == 0x24) {
+		return divemode == OSTC3_CC || divemode == OSTC3_PSCR;
+	} else {
+		return 0;
+	}
+}
+
 static dc_status_t
 hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 {
@@ -222,6 +276,13 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 		return DC_STATUS_DATAFORMAT;
 	}
 
+	// Get the dive mode.
+	unsigned int divemode = layout->divemode < header ?
+		data[layout->divemode] : UNDEFINED;
+
+	// Get the CCR mode.
+	unsigned int ccr = hw_ostc_is_ccr (divemode, version);
+
 	// Get all the gas mixes, the index of the inital mix,
 	// the initial setpoint (used in the fixed setpoint CCR mode),
 	// and the initial CNS from the header
@@ -238,19 +299,25 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 		for (unsigned int i = 0; i < ngasmixes; ++i) {
 			gasmix[i].oxygen = data[25 + 2 * i];
 			gasmix[i].helium = 0;
+			gasmix[i].type = 0;
+			gasmix[i].enabled = 1;
+			gasmix[i].diluent = 0;
 		}
 	} else if (version == 0x23 || version == 0x24) {
 		ngasmixes = 5;
 		for (unsigned int i = 0; i < ngasmixes; ++i) {
 			gasmix[i].oxygen = data[28 + 4 * i + 0];
 			gasmix[i].helium = data[28 + 4 * i + 1];
+			gasmix[i].type   = data[28 + 4 * i + 3];
+			gasmix[i].enabled = gasmix[i].type != 0;
+			gasmix[i].diluent = ccr;
 			// Find the first gas marked as the initial gas.
 			if (initial == UNDEFINED && data[28 + 4 * i + 3] == 1) {
 				initial = i + 1; /* One based index! */
 			}
 		}
 		// The first fixed setpoint is the initial setpoint in CCR mode.
-		if (data[82] == OSTC3_CC) {
+		if (ccr) {
 			initial_setpoint = data[60];
 		}
 		// Initial CNS
@@ -263,6 +330,13 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 		for (unsigned int i = 0; i < ngasmixes; ++i) {
 			gasmix[i].oxygen = data[19 + 2 * i + 0];
 			gasmix[i].helium = data[19 + 2 * i + 1];
+			gasmix[i].type = 0;
+			if (version == 0x21) {
+				gasmix[i].enabled = data[53] & (1 << i);
+			} else {
+				gasmix[i].enabled = 1;
+			}
+			gasmix[i].diluent = ccr;
 		}
 	}
 	if (initial != UNDEFINED) {
@@ -322,6 +396,9 @@ hw_ostc_parser_create_internal (dc_parser_t **out, dc_context_t *context, unsign
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->gasmix[i].oxygen = 0;
 		parser->gasmix[i].helium = 0;
+		parser->gasmix[i].type = 0;
+		parser->gasmix[i].enabled = 0;
+		parser->gasmix[i].diluent = 0;
 	}
 
 	*out = (dc_parser_t *) parser;
@@ -331,9 +408,9 @@ hw_ostc_parser_create_internal (dc_parser_t **out, dc_context_t *context, unsign
 
 
 dc_status_t
-hw_ostc_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int hwos)
+hw_ostc_parser_create (dc_parser_t **out, dc_context_t *context)
 {
-	return hw_ostc_parser_create_internal (out, context, hwos, 0);
+	return hw_ostc_parser_create_internal (out, context, 0, 0);
 }
 
 dc_status_t
@@ -360,6 +437,9 @@ hw_ostc_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsig
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->gasmix[i].oxygen = 0;
 		parser->gasmix[i].helium = 0;
+		parser->gasmix[i].type = 0;
+		parser->gasmix[i].enabled = 0;
+		parser->gasmix[i].diluent = 0;
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -371,7 +451,6 @@ hw_ostc_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
 	hw_ostc_parser_t *parser = (hw_ostc_parser_t *) abstract;
 	const unsigned char *data = abstract->data;
-	unsigned int size = abstract->size;
 
 	// Cache the header data.
 	dc_status_t rc = hw_ostc_parser_cache (parser);
@@ -432,7 +511,6 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 {
 	hw_ostc_parser_t *parser = (hw_ostc_parser_t *) abstract;
 	const unsigned char *data = abstract->data;
-	unsigned int size = abstract->size;
 
 	// Cache the header data.
 	dc_status_t rc = hw_ostc_parser_cache (parser);
@@ -451,6 +529,8 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 	dc_salinity_t *water = (dc_salinity_t *) value;
+	dc_decomodel_t *decomodel = (dc_decomodel_t *) value;
+
 	unsigned int salinity = data[layout->salinity];
 	if (version == 0x23 || version == 0x24)
 		salinity += 100;
@@ -464,6 +544,8 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 			*((double *) value) = array_uint16_le (data + layout->maxdepth) / 100.0;
 			break;
 		case DC_FIELD_AVGDEPTH:
+			if (parser->version < 0x21)
+				return DC_STATUS_UNSUPPORTED;
 			*((double *) value) = array_uint16_le (data + layout->avgdepth) / 100.0;
 			break;
 		case DC_FIELD_GASMIX_COUNT:
@@ -492,7 +574,7 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 			break;
 		case DC_FIELD_DIVEMODE:
 			if (version == 0x21) {
-				switch (data[51]) {
+				switch (data[layout->divemode]) {
 				case OSTC_APNEA:
 					*((dc_divemode_t *) value) = DC_DIVEMODE_FREEDIVE;
 					break;
@@ -505,14 +587,16 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 					break;
 				case OSTC_ZHL16_CC:
 				case OSTC_ZHL16_CC_GF:
+					*((dc_divemode_t *) value) = DC_DIVEMODE_CCR;
+					break;
 				case OSTC_PSCR_GF:
-					*((dc_divemode_t *) value) = DC_DIVEMODE_CC;
+					*((dc_divemode_t *) value) = DC_DIVEMODE_SCR;
 					break;
 				default:
 					return DC_STATUS_DATAFORMAT;
 				}
 			} else if (version == 0x22) {
-				switch (data[51]) {
+				switch (data[layout->divemode]) {
 				case FROG_ZHL16:
 				case FROG_ZHL16_GF:
 					*((dc_divemode_t *) value) = DC_DIVEMODE_OC;
@@ -524,12 +608,12 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 					return DC_STATUS_DATAFORMAT;
 				}
 			} else if (version == 0x23 || version == 0x24) {
-				switch (data[82]) {
+				switch (data[layout->divemode]) {
 				case OSTC3_OC:
 					*((dc_divemode_t *) value) = DC_DIVEMODE_OC;
 					break;
 				case OSTC3_CC:
-					*((dc_divemode_t *) value) = DC_DIVEMODE_CC;
+					*((dc_divemode_t *) value) = DC_DIVEMODE_CCR;
 					break;
 				case OSTC3_GAUGE:
 					*((dc_divemode_t *) value) = DC_DIVEMODE_GAUGE;
@@ -537,6 +621,9 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 				case OSTC3_APNEA:
 					*((dc_divemode_t *) value) = DC_DIVEMODE_FREEDIVE;
 					break;
+				case OSTC3_PSCR:
+					*((dc_divemode_t *) value) = DC_DIVEMODE_SCR;
+					break;
 				default:
 					return DC_STATUS_DATAFORMAT;
 				}
@@ -544,6 +631,70 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 				return DC_STATUS_UNSUPPORTED;
 			}
 			break;
+		case DC_FIELD_DECOMODEL:
+			if (version == 0x21) {
+				switch (data[layout->divemode]) {
+				case OSTC_APNEA:
+				case OSTC_GAUGE:
+					decomodel->type = DC_DECOMODEL_NONE;
+					break;
+				case OSTC_ZHL16_OC:
+				case OSTC_ZHL16_CC:
+					decomodel->type = DC_DECOMODEL_BUHLMANN;
+					decomodel->params.gf.low  = 100;
+					decomodel->params.gf.high = 100;
+					break;
+				case OSTC_ZHL16_OC_GF:
+				case OSTC_ZHL16_CC_GF:
+				case OSTC_PSCR_GF:
+					decomodel->type = DC_DECOMODEL_BUHLMANN;
+					decomodel->params.gf.low  = data[layout->gf + 0];
+					decomodel->params.gf.high = data[layout->gf + 1];
+					break;
+				default:
+					return DC_STATUS_DATAFORMAT;
+				}
+			} else if (version == 0x22) {
+				switch (data[layout->divemode]) {
+				case FROG_ZHL16:
+					decomodel->type = DC_DECOMODEL_BUHLMANN;
+					decomodel->params.gf.low  = 100;
+					decomodel->params.gf.high = 100;
+					break;
+				case FROG_ZHL16_GF:
+					decomodel->type = DC_DECOMODEL_BUHLMANN;
+					decomodel->params.gf.low  = data[layout->gf + 0];
+					decomodel->params.gf.high = data[layout->gf + 1];
+					break;
+				case FROG_APNEA:
+					decomodel->type = DC_DECOMODEL_NONE;
+					break;
+				default:
+					return DC_STATUS_DATAFORMAT;
+				}
+			} else if (version == 0x23 || version == 0x24) {
+				switch (data[layout->decomodel]) {
+				case OSTC3_ZHL16:
+					decomodel->type = DC_DECOMODEL_BUHLMANN;
+					decomodel->params.gf.low  = 100;
+					decomodel->params.gf.high = 100;
+					break;
+				case OSTC3_ZHL16_GF:
+					decomodel->type = DC_DECOMODEL_BUHLMANN;
+					decomodel->params.gf.low  = data[layout->gf + 0];
+					decomodel->params.gf.high = data[layout->gf + 1];
+					break;
+				case OSTC4_VPM:
+					decomodel->type = DC_DECOMODEL_VPM;
+					break;
+				default:
+					return DC_STATUS_DATAFORMAT;
+				}
+			} else {
+				return DC_STATUS_UNSUPPORTED;
+			}
+			decomodel->conservatism = 0;
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -569,6 +720,15 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 	unsigned int header = parser->header;
 	const hw_ostc_layout_t *layout = parser->layout;
 
+	// Exit if no profile data available.
+	const unsigned char empty[] = {0x08, 0x00, 0x00, 0xFD, 0xFD};
+	if (size == header ||
+		(size == header + 2 && memcmp(data + header, empty + 3, 2) == 0) ||
+		(size == header + 5 && memcmp(data + header, empty, 5) == 0)) {
+		parser->cached = PROFILE;
+		return DC_STATUS_SUCCESS;
+	}
+
 	// Check the header length.
 	if (version == 0x23 || version == 0x24) {
 		if (size < header + 5) {
@@ -584,14 +744,6 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 	else
 		samplerate = data[36];
 
-	// Get the salinity factor.
-	unsigned int salinity = data[layout->salinity];
-	if (version == 0x23 || version == 0x24)
-		salinity += 100;
-	if (salinity < 100 || salinity > 104)
-		salinity = 100;
-	double hydrostatic = GRAVITY * salinity * 10.0;
-
 	// Get the number of sample descriptors.
 	unsigned int nconfig = 0;
 	if (version == 0x23 || version == 0x24)
@@ -626,20 +778,21 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 
 		if (info[i].divisor) {
 			switch (info[i].type) {
-			case 0: // Temperature
-			case 1: // Deco / NDL
+			case TEMPERATURE:
+			case DECO:
+			case TANK:
 				if (info[i].size != 2) {
 					ERROR(abstract->context, "Unexpected sample size.");
 					return DC_STATUS_DATAFORMAT;
 				}
 				break;
-			case 3: // ppO2
+			case PPO2:
 				if (info[i].size != 3 && info[i].size != 9) {
 					ERROR(abstract->context, "Unexpected sample size.");
 					return DC_STATUS_DATAFORMAT;
 				}
 				break;
-			case 5: // CNS
+			case CNS:
 				if (info[i].size != 1 && info[i].size != 2) {
 					ERROR(abstract->context, "Unexpected sample size.");
 					return DC_STATUS_DATAFORMAT;
@@ -659,8 +812,16 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 		firmware = array_uint16_be (data + layout->firmware);
 	}
 
+	// Get the dive mode.
+	unsigned int divemode = layout->divemode < header ?
+		data[layout->divemode] : UNDEFINED;
+
+	// Get the CCR mode.
+	unsigned int ccr = hw_ostc_is_ccr (divemode, version);
+
 	unsigned int time = 0;
 	unsigned int nsamples = 0;
+	unsigned int tank = parser->initial != UNDEFINED ? parser->initial : 0;
 
 	unsigned int offset = header;
 	if (version == 0x23 || version == 0x24)
@@ -693,9 +854,9 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 			if (callback) callback (DC_SAMPLE_CNS, sample, userdata);
 		}
 
-		// Depth (mbar).
+		// Depth (1/100 m).
 		unsigned int depth = array_uint16_le (data + offset);
-		sample.depth = (depth * BAR / 1000.0) / hydrostatic;
+		sample.depth = depth / 100.0;
 		if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 		offset += 2;
 
@@ -765,7 +926,7 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 			}
 			unsigned int o2 = data[offset];
 			unsigned int he = data[offset + 1];
-			unsigned int idx = hw_ostc_find_gasmix (parser, o2, he, MANUAL);
+			unsigned int idx = hw_ostc_find_gasmix (parser, o2, he, ccr, MANUAL);
 			if (idx >= parser->ngasmixes) {
 				if (idx >= NGASMIXES) {
 					ERROR (abstract->context, "Maximum number of gas mixes reached.");
@@ -773,6 +934,9 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 				}
 				parser->gasmix[idx].oxygen = o2;
 				parser->gasmix[idx].helium = he;
+				parser->gasmix[idx].type = 0;
+				parser->gasmix[idx].enabled = 1;
+				parser->gasmix[idx].diluent = ccr;
 				parser->ngasmixes = idx + 1;
 			}
 
@@ -789,13 +953,18 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 				return DC_STATUS_DATAFORMAT;
 			}
 			unsigned int idx = data[offset];
-			if (idx < 1 || idx > parser->ngasmixes) {
-				ERROR(abstract->context, "Invalid gas mix.");
+			if (parser->model == OSTC4 && ccr && idx > parser->nfixed) {
+				// Fix the OSTC4 diluent index.
+				idx -= parser->nfixed;
+			}
+			if (idx < 1 || idx > parser->nfixed) {
+				ERROR(abstract->context, "Invalid gas mix (%u).", idx);
 				return DC_STATUS_DATAFORMAT;
 			}
 			idx--; /* Convert to a zero based index. */
 			sample.gasmix = idx;
 			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+			tank = idx;
 			offset++;
 			length--;
 		}
@@ -822,7 +991,7 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 
 				unsigned int o2 = data[offset];
 				unsigned int he = data[offset + 1];
-				unsigned int idx = hw_ostc_find_gasmix (parser, o2, he, MANUAL);
+				unsigned int idx = hw_ostc_find_gasmix (parser, o2, he, 0, MANUAL);
 				if (idx >= parser->ngasmixes) {
 					if (idx >= NGASMIXES) {
 						ERROR (abstract->context, "Maximum number of gas mixes reached.");
@@ -830,6 +999,9 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 					}
 					parser->gasmix[idx].oxygen = o2;
 					parser->gasmix[idx].helium = he;
+					parser->gasmix[idx].type = 0;
+					parser->gasmix[idx].enabled = 1;
+					parser->gasmix[idx].diluent = 0;
 					parser->ngasmixes = idx + 1;
 				}
 
@@ -844,6 +1016,17 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 		for (unsigned int i = 0; i < nconfig; ++i) {
 			if (info[i].divisor && (nsamples % info[i].divisor) == 0) {
 				if (length < info[i].size) {
+					// Due to a bug in the hwOS Tech firmware v3.03 to v3.08, and
+					// the hwOS Sport firmware v10.57 to v10.63, the ppO2 divisor
+					// is sometimes not correctly reset to zero when no ppO2
+					// samples are being recorded.
+					if (info[i].type == PPO2 && parser->hwos && parser->model != OSTC4 &&
+						((firmware >= OSTC3FW(3,3) && firmware <= OSTC3FW(3,8)) ||
+						(firmware >= OSTC3FW(10,57) && firmware <= OSTC3FW(10,63)))) {
+						WARNING (abstract->context, "Reset invalid ppO2 divisor to zero.");
+						info[i].divisor = 0;
+						continue;
+					}
 					ERROR (abstract->context, "Buffer overflow detected!");
 					return DC_STATUS_DATAFORMAT;
 				}
@@ -852,15 +1035,15 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 				unsigned int count = 0;
 				unsigned int value = 0;
 				switch (info[i].type) {
-				case 0: // Temperature (0.1 °C).
+				case TEMPERATURE:
 					value = array_uint16_le (data + offset);
 					sample.temperature = value / 10.0;
 					if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 					break;
-				case 1: // Deco / NDL
+				case DECO:
 					// Due to a firmware bug, the deco/ndl info is incorrect for
 					// all OSTC4 dives with a firmware older than version 1.0.8.
-					if (parser->model == OSTC4 && firmware < 0x0810)
+					if (parser->model == OSTC4 && firmware < OSTC4FW(1,0,8,0))
 						break;
 					if (data[offset]) {
 						sample.deco.type = DC_DECO_DECOSTOP;
@@ -872,7 +1055,7 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 					sample.deco.time = data[offset + 1] * 60;
 					if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
 					break;
-				case 3: // ppO2 (0.01 bar).
+				case PPO2:
 					for (unsigned int j = 0; j < 3; ++j) {
 						if (info[i].size == 3) {
 							ppo2[j] = data[offset + j];
@@ -889,13 +1072,27 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 						}
 					}
 					break;
-				case 5: // CNS
+				case CNS:
 					if (info[i].size == 2)
 						sample.cns = array_uint16_le (data + offset) / 100.0;
 					else
 						sample.cns = data[offset] / 100.0;
 					if (callback) callback (DC_SAMPLE_CNS, sample, userdata);
 					break;
+				case TANK:
+					value = array_uint16_le (data + offset);
+					if (value != 0) {
+						sample.pressure.tank = tank;
+						sample.pressure.value = value;
+						// The hwOS Sport firmware used a resolution of
+						// 0.1 bar between versions 10.40 and 10.50.
+						if (parser->hwos && parser->model != OSTC4 &&
+							(firmware >= OSTC3FW(10,40) && firmware <= OSTC3FW(10,50))) {
+							sample.pressure.value /= 10.0;
+						}
+						if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+					}
+					break;
 				default: // Not yet used.
 					break;
 				}
@@ -927,7 +1124,7 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 
 				unsigned int o2 = data[offset];
 				unsigned int he = data[offset + 1];
-				unsigned int idx = hw_ostc_find_gasmix (parser, o2, he, MANUAL);
+				unsigned int idx = hw_ostc_find_gasmix (parser, o2, he, 0, MANUAL);
 				if (idx >= parser->ngasmixes) {
 					if (idx >= NGASMIXES) {
 						ERROR (abstract->context, "Maximum number of gas mixes reached.");
@@ -935,6 +1132,9 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 					}
 					parser->gasmix[idx].oxygen = o2;
 					parser->gasmix[idx].helium = he;
+					parser->gasmix[idx].type = 0;
+					parser->gasmix[idx].enabled = 1;
+					parser->gasmix[idx].diluent = 0;
 					parser->ngasmixes = idx + 1;
 				}
 
