@@ -23,6 +23,7 @@
 #define MARES_ICONHD_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 
@@ -31,7 +32,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-mares_iconhd_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+mares_iconhd_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
 mares_iconhd_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
