@@ -68,6 +68,9 @@ static const dc_parser_vtable_t mares_nemo_parser_vtable = {
 	sizeof(mares_nemo_parser_t),
 	DC_FAMILY_MARES_NEMO,
 	mares_nemo_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	mares_nemo_parser_get_datetime, /* datetime */
 	mares_nemo_parser_get_field, /* fields */
 	mares_nemo_parser_samples_foreach, /* samples_foreach */
