@@ -31,61 +31,6 @@
 
 #define ISINSTANCE(parser) dc_parser_isinstance((parser), &oceanic_atom2_parser_vtable)
 
-#define ATOM1       0x4250
-#define EPICA       0x4257
-#define VT3         0x4258
-#define T3A         0x4259
-#define ATOM2       0x4342
-#define GEO         0x4344
-#define MANTA       0x4345
-#define DATAMASK    0x4347
-#define COMPUMASK   0x4348
-#define OC1A        0x434E
-#define F10A        0x434D
-#define WISDOM2     0x4350
-#define INSIGHT2    0x4353
-#define ELEMENT2    0x4357
-#define VEO20       0x4359
-#define VEO30       0x435A
-#define ZEN         0x4441
-#define ZENAIR      0x4442
-#define ATMOSAI2    0x4443
-#define PROPLUS21   0x4444
-#define GEO20       0x4446
-#define VT4         0x4447
-#define OC1B        0x4449
-#define VOYAGER2G   0x444B
-#define ATOM3       0x444C
-#define DG03        0x444D
-#define OCS         0x4450
-#define OC1C        0x4451
-#define VT41        0x4452
-#define EPICB       0x4453
-#define T3B         0x4455
-#define ATOM31      0x4456
-#define A300AI      0x4457
-#define WISDOM3     0x4458
-#define A300        0x445A
-#define TX1         0x4542
-#define MUNDIAL2    0x4543
-#define AMPHOS      0x4545
-#define AMPHOSAIR   0x4546
-#define PROPLUS3    0x4548
-#define F11A        0x4549
-#define OCI         0x454B
-#define A300CS      0x454C
-#define MUNDIAL3    0x4550
-#define F10B        0x4553
-#define F11B        0x4554
-#define XPAIR       0x4555
-#define VISION      0x4556
-#define VTX         0x4557
-#define I300        0x4559
-#define I750TC      0x455A
-#define I450T       0x4641
-#define I550        0x4642
-#define I200        0x4646
-
 #define NORMAL   0
 #define GAUGE    1
 #define FREEDIVE 2
@@ -123,6 +68,9 @@ static const dc_parser_vtable_t oceanic_atom2_parser_vtable = {
 	sizeof(oceanic_atom2_parser_t),
 	DC_FAMILY_OCEANIC_ATOM2,
 	oceanic_atom2_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	oceanic_atom2_parser_get_datetime, /* datetime */
 	oceanic_atom2_parser_get_field, /* fields */
 	oceanic_atom2_parser_samples_foreach, /* samples_foreach */
@@ -156,13 +104,17 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		model == A300 || model == MANTA ||
 		model == INSIGHT2 || model == ZEN ||
 		model == I300 || model == I550 ||
-		model == I200) {
+		model == I200 || model == I200C ||
+		model == I300C || model == GEO40 ||
+		model == VEO40 || model == I470TC ||
+		model == GEOAIR) {
 		parser->headersize -= PAGESIZE;
 	} else if (model == VT4 || model == VT41) {
 		parser->headersize += PAGESIZE;
 	} else if (model == TX1) {
 		parser->headersize += 2 * PAGESIZE;
-	} else if (model == ATOM1) {
+	} else if (model == ATOM1 || model == I100 ||
+		model == PROPLUS4) {
 		parser->headersize -= 2 * PAGESIZE;
 	} else if (model == F10A || model == F10B ||
 		model == MUNDIAL2 || model == MUNDIAL3) {
@@ -172,8 +124,15 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		parser->headersize = 5 * PAGESIZE;
 		parser->footersize = 0;
 	} else if (model == A300CS || model == VTX ||
-		model == I450T || model == I750TC) {
+		model == I450T || model == I750TC ||
+		model == I770R || model == SAGE ||
+		model == BEACON) {
 		parser->headersize = 5 * PAGESIZE;
+	} else if (model == PROPLUSX) {
+		parser->headersize = 3 * PAGESIZE;
+	} else if (model == I550C || model == WISDOM4 ||
+		model == I200CV2) {
+		parser->headersize = 5 * PAGESIZE / 2;
 	}
 
 	parser->cached = 0;
@@ -248,8 +207,13 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case A300AI:
 		case OCI:
 		case I550:
+		case I550C:
 		case VISION:
 		case XPAIR:
+		case WISDOM4:
+		case I470TC:
+		case I200CV2:
+		case GEOAIR:
 			datetime->year   = ((p[5] & 0xE0) >> 5) + ((p[7] & 0xE0) >> 2) + 2000;
 			datetime->month  = (p[3] & 0x0F);
 			datetime->day    = ((p[0] & 0x80) >> 3) + ((p[3] & 0xF0) >> 4);
@@ -269,6 +233,12 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case INSIGHT2:
 		case I300:
 		case I200:
+		case I200C:
+		case I100:
+		case I300C:
+		case GEO40:
+		case VEO40:
+		case PROPLUS4:
 			datetime->year   = ((p[3] & 0xE0) >> 1) + (p[4] & 0x0F) + 2000;
 			datetime->month  = (p[4] & 0xF0) >> 4;
 			datetime->day    = p[3] & 0x1F;
@@ -279,6 +249,9 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case AMPHOS:
 		case AMPHOSAIR:
 		case VOYAGER2G:
+		case TALIS:
+		case AMPHOS2:
+		case AMPHOSAIR2:
 			datetime->year   = (p[3] & 0x1F) + 2000;
 			datetime->month  = (p[7] & 0xF0) >> 4;
 			datetime->day    = ((p[3] & 0x80) >> 3) + ((p[5] & 0xF0) >> 4);
@@ -309,6 +282,10 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case VTX:
 		case I450T:
 		case I750TC:
+		case PROPLUSX:
+		case I770R:
+		case SAGE:
+		case BEACON:
 			datetime->year   = (p[10]) + 2000;
 			datetime->month  = (p[8]);
 			datetime->day    = (p[9]);
@@ -410,6 +387,7 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	unsigned int ngasmixes = 0;
 	unsigned int o2_offset = 0;
 	unsigned int he_offset = 0;
+	unsigned int o2_step = 1;
 	if (mode == FREEDIVE) {
 		ngasmixes = 0;
 	} else if (parser->model == DATAMASK || parser->model == COMPUMASK) {
@@ -428,7 +406,8 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 		he_offset = 0x48;
 		ngasmixes = 6;
 	} else if (parser->model == A300CS || parser->model == VTX ||
-		parser->model == I750TC) {
+		parser->model == I750TC || parser->model == SAGE ||
+		parser->model == BEACON) {
 		o2_offset = 0x2A;
 		if (data[0x39] & 0x04) {
 			ngasmixes = 1;
@@ -445,6 +424,20 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	} else if (parser->model == ZEN) {
 		o2_offset = header + 4;
 		ngasmixes = 2;
+	} else if (parser->model == PROPLUSX) {
+		o2_offset = 0x24;
+		ngasmixes = 4;
+	} else if (parser->model == I770R) {
+		o2_offset = 0x30;
+		ngasmixes = 4;
+		o2_step = 2;
+	} else if (parser->model == I470TC) {
+		o2_offset = 0x28;
+		ngasmixes = 3;
+		o2_step = 2;
+	} else if (parser->model == WISDOM4) {
+		o2_offset = header + 4;
+		ngasmixes = 1;
 	} else {
 		o2_offset = header + 4;
 		ngasmixes = 3;
@@ -456,8 +449,8 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	parser->mode = mode;
 	parser->ngasmixes = ngasmixes;
 	for (unsigned int i = 0; i < ngasmixes; ++i) {
-		if (data[o2_offset + i]) {
-			parser->oxygen[i] = data[o2_offset + i];
+		if (data[o2_offset + i * o2_step]) {
+			parser->oxygen[i] = data[o2_offset + i * o2_step];
 		} else {
 			parser->oxygen[i] = 21;
 		}
@@ -478,9 +471,7 @@ oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
-
 	const unsigned char *data = abstract->data;
-	unsigned int size = abstract->size;
 
 	// Cache the header data.
 	status = oceanic_atom2_parser_cache (parser);
@@ -620,42 +611,21 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	unsigned int interval = 1;
 	unsigned int samplerate = 1;
 	if (parser->mode != FREEDIVE) {
-		unsigned int idx = 0x17;
+		unsigned int offset = 0x17;
 		if (parser->model == A300CS || parser->model == VTX ||
-			parser->model == I450T || parser->model == I750TC)
-			idx = 0x1f;
-		switch (data[idx] & 0x03) {
-		case 0:
-			interval = 2;
-			break;
-		case 1:
-			interval = 15;
-			break;
-		case 2:
-			interval = 30;
-			break;
-		case 3:
-			interval = 60;
-			break;
-		}
+			parser->model == I450T || parser->model == I750TC ||
+			parser->model == PROPLUSX || parser->model == I770R ||
+			parser->model == SAGE || parser->model == BEACON)
+			offset = 0x1f;
+		const unsigned int intervals[] = {2, 15, 30, 60};
+		unsigned int idx = data[offset] & 0x03;
+		interval = intervals[idx];
 	} else if (parser->model == F11A || parser->model == F11B) {
-		unsigned int idx = 0x29;
-		switch (data[idx] & 0x03) {
-		case 0:
-			interval = 1;
-			samplerate = 4;
-			break;
-		case 1:
-			interval = 1;
-			samplerate = 2;
-			break;
-		case 2:
-			interval = 1;
-			break;
-		case 3:
-			interval = 2;
-			break;
-		}
+		const unsigned int intervals[] = {1, 1, 1, 2};
+		const unsigned int samplerates[] = {4, 2, 1, 1};
+		unsigned int idx = data[0x29] & 0x03;
+		interval = intervals[idx];
+		samplerate = samplerates[idx];
 		if (samplerate > 1) {
 			// Some models supports multiple samples per second.
 			// Since our smallest unit of time is one second, we can't
@@ -677,7 +647,10 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == OC1C || parser->model == OCI ||
 		parser->model == TX1 || parser->model == A300CS ||
 		parser->model == VTX || parser->model == I450T ||
-		parser->model == I750TC) {
+		parser->model == I750TC || parser->model == PROPLUSX ||
+		parser->model == I770R || parser->model == I470TC ||
+		parser->model == SAGE || parser->model == BEACON ||
+		parser->model == GEOAIR) {
 		samplesize = PAGESIZE;
 	}
 
@@ -690,7 +663,10 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == A300 || parser->model == ZEN ||
 		parser->model == GEO || parser->model == GEO20 ||
 		parser->model == MANTA || parser->model == I300 ||
-		parser->model == I200) {
+		parser->model == I200 || parser->model == I100 ||
+		parser->model == I300C || parser->model == TALIS ||
+		parser->model == I200C || parser->model == I200CV2 ||
+		parser->model == GEO40 || parser->model == VEO40) {
 		have_pressure = 0;
 	}
 
@@ -759,7 +735,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				tank = 0;
 				pressure = (((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF);
 			} else if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I750TC) {
+				parser->model == I750TC || parser->model == SAGE ||
+				parser->model == BEACON) {
 				// Tank pressure (1 psi) and number (one based index)
 				tank = (data[offset + 1] & 0x03) - 1;
 				pressure = ((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF;
@@ -808,7 +785,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			}
 
 			// Time.
-			if (parser->model == I450T) {
+			if (parser->model == I450T || parser->model == I470TC) {
 				unsigned int minute = bcd2dec(data[offset + 0]);
 				unsigned int hour   = bcd2dec(data[offset + 1] & 0x0F);
 				unsigned int second = bcd2dec(data[offset + 2]);
@@ -840,12 +817,18 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == ELEMENT2 || parser->model == MANTA ||
 					parser->model == ZEN) {
 					temperature = data[offset + 6];
+				} else if (parser->model == TALIS) {
+					temperature = data[offset + 7];
 				} else if (parser->model == GEO20 || parser->model == VEO20 ||
 					parser->model == VEO30 || parser->model == OC1A ||
 					parser->model == OC1B || parser->model == OC1C ||
 					parser->model == OCI || parser->model == A300 ||
 					parser->model == I450T || parser->model == I300 ||
-					parser->model == I200) {
+					parser->model == I200 || parser->model == I100 ||
+					parser->model == I300C || parser->model == I200C ||
+					parser->model == GEO40 || parser->model == VEO40 ||
+					parser->model == I470TC || parser->model == I200CV2 ||
+					parser->model == GEOAIR) {
 					temperature = data[offset + 3];
 				} else if (parser->model == OCS || parser->model == TX1) {
 					temperature = data[offset + 1];
@@ -855,15 +838,19 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == XPAIR) {
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else if (parser->model == A300CS || parser->model == VTX ||
-					parser->model == I750TC) {
+					parser->model == I750TC || parser->model == PROPLUSX ||
+					parser->model == I770R|| parser->model == SAGE ||
+					parser->model == BEACON) {
 					temperature = data[offset + 11];
 				} else {
 					unsigned int sign;
 					if (parser->model == DG03 || parser->model == PROPLUS3 ||
-						parser->model == I550)
+						parser->model == I550 || parser->model == I550C ||
+						parser->model == PROPLUS4 || parser->model == WISDOM4)
 						sign = (~data[offset + 5] & 0x04) >> 2;
 					else if (parser->model == VOYAGER2G || parser->model == AMPHOS ||
-						parser->model == AMPHOSAIR || parser->model == ZENAIR)
+						parser->model == AMPHOSAIR || parser->model == ZENAIR ||
+						parser->model == AMPHOS2 || parser->model == AMPHOSAIR2)
 						sign = (data[offset + 5] & 0x04) >> 2;
 					else if (parser->model == ATOM2 || parser->model == PROPLUS21 ||
 						parser->model == EPICA || parser->model == EPICB ||
@@ -885,17 +872,22 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			if (have_pressure) {
 				if (parser->model == OC1A || parser->model == OC1B ||
 					parser->model == OC1C || parser->model == OCI ||
-					parser->model == I450T)
+					parser->model == I450T || parser->model == I470TC ||
+					parser->model == GEOAIR)
 					pressure = (data[offset + 10] + (data[offset + 11] << 8)) & 0x0FFF;
 				else if (parser->model == VT4 || parser->model == VT41||
 					parser->model == ATOM3 || parser->model == ATOM31 ||
 					parser->model == ZENAIR ||parser->model == A300AI ||
 					parser->model == DG03 || parser->model == PROPLUS3 ||
 					parser->model == AMPHOSAIR || parser->model == I550 ||
-					parser->model == VISION || parser->model == XPAIR)
+					parser->model == VISION || parser->model == XPAIR ||
+					parser->model == I550C || parser->model == PROPLUS4 ||
+					parser->model == WISDOM4 || parser->model == AMPHOSAIR2)
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else if (parser->model == TX1 || parser->model == A300CS ||
-					parser->model == VTX || parser->model == I750TC)
+					parser->model == VTX || parser->model == I750TC ||
+					parser->model == PROPLUSX || parser->model == I770R ||
+					parser->model == SAGE || parser->model == BEACON)
 					pressure = array_uint16_le (data + offset + 4);
 				else
 					pressure -= data[offset + 1];
@@ -913,7 +905,11 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				parser->model == OC1B || parser->model == OC1C ||
 				parser->model == OCI || parser->model == A300 ||
 				parser->model == I450T || parser->model == I300 ||
-				parser->model == I200)
+				parser->model == I200 || parser->model == I100 ||
+				parser->model == I300C || parser->model == I200C ||
+				parser->model == GEO40 || parser->model == VEO40 ||
+				parser->model == I470TC || parser->model == I200CV2 ||
+				parser->model == GEOAIR)
 				depth = (data[offset + 4] + (data[offset + 5] << 8)) & 0x0FFF;
 			else if (parser->model == ATOM1)
 				depth = data[offset + 3] * 16;
@@ -943,7 +939,9 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			unsigned int have_deco = 0;
 			unsigned int decostop = 0, decotime = 0;
 			if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I450T || parser->model == I750TC) {
+				parser->model == I750TC || parser->model == SAGE ||
+				parser->model == PROPLUSX || parser->model == I770R ||
+				parser->model == BEACON) {
 				decostop = (data[offset + 15] & 0x70) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_deco = 1;
@@ -956,13 +954,19 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				decotime = array_uint16_le(data + offset + 6);
 				have_deco = 1;
 			} else if (parser->model == ATOM31 || parser->model == VISION ||
-				parser->model == XPAIR || parser->model == I550) {
+				parser->model == XPAIR || parser->model == I550 ||
+				parser->model == I550C || parser->model == WISDOM4) {
 				decostop = (data[offset + 5] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 4) & 0x03FF;
 				have_deco = 1;
 			} else if (parser->model == I200 || parser->model == I300 ||
 				parser->model == OC1A || parser->model == OC1B ||
-				parser->model == OC1C || parser->model == OCI) {
+				parser->model == OC1C || parser->model == OCI ||
+				parser->model == I100 || parser->model == I300C ||
+				parser->model == I450T || parser->model == I200C ||
+				parser->model == GEO40 || parser->model == VEO40 ||
+				parser->model == I470TC || parser->model == I200CV2 ||
+				parser->model == GEOAIR) {
 				decostop = (data[offset + 7] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x0FFF;
 				have_deco = 1;
@@ -986,11 +990,14 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				have_rbt = 1;
 			} else if (parser->model == I450T || parser->model == OC1A ||
 				parser->model == OC1B || parser->model == OC1C ||
-				parser->model == OCI) {
+				parser->model == OCI || parser->model == PROPLUSX ||
+				parser->model == I770R || parser->model == I470TC ||
+				parser->model == GEOAIR) {
 				rbt = array_uint16_le(data + offset + 8) & 0x01FF;
 				have_rbt = 1;
 			} else if (parser->model == VISION || parser->model == XPAIR ||
-				parser->model == I550) {
+				parser->model == I550 || parser->model == I550C ||
+				parser->model == WISDOM4) {
 				rbt = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_rbt = 1;
 			}
@@ -1002,7 +1009,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			// Bookmarks
 			unsigned int have_bookmark = 0;
 			if (parser->model == OC1A || parser->model == OC1B ||
-				parser->model == OC1C || parser->model == OCI) {
+				parser->model == OC1C || parser->model == OCI ||
+				parser->model == GEOAIR) {
 				have_bookmark = data[offset + 12] & 0x80;
 			}
 			if (have_bookmark) {
