@@ -32,7 +32,7 @@
 
 #define VTABLE(abstract)	((const oceanic_common_device_vtable_t *) abstract->vtable)
 
-#define RB_LOGBOOK_DISTANCE(a,b,l)	ringbuffer_distance (a, b, 0, l->rb_logbook_begin, l->rb_logbook_end)
+#define RB_LOGBOOK_DISTANCE(a,b,l)	ringbuffer_distance (a, b, 1, l->rb_logbook_begin, l->rb_logbook_end)
 #define RB_LOGBOOK_INCR(a,b,l)		ringbuffer_increment (a, b, l->rb_logbook_begin, l->rb_logbook_end)
 
 #define RB_PROFILE_DISTANCE(a,b,l)	ringbuffer_distance (a, b, 0, l->rb_profile_begin, l->rb_profile_end)
@@ -41,7 +41,7 @@
 #define INVALID 0
 
 static unsigned int
-get_profile_first (const unsigned char data[], const oceanic_common_layout_t *layout)
+get_profile_first (const unsigned char data[], const oceanic_common_layout_t *layout, unsigned int pagesize)
 {
 	unsigned int value;
 
@@ -55,17 +55,23 @@ get_profile_first (const unsigned char data[], const oceanic_common_layout_t *la
 		return array_uint16_le (data + 16);
 	}
 
-	if (layout->memsize > 0x20000)
-		return (value & 0x3FFF) * PAGESIZE;
-	else if (layout->memsize > 0x10000)
-		return (value & 0x1FFF) * PAGESIZE;
-	else
-		return (value & 0x0FFF) * PAGESIZE;
+	unsigned int npages = (layout->memsize - layout->highmem) / pagesize;
+	if (npages > 0x4000) {
+		value &= 0x7FFF;
+	} else if (npages > 0x2000) {
+		value &= 0x3FFF;
+	}  else if (npages > 0x1000) {
+		value &= 0x1FFF;
+	} else {
+		value &= 0x0FFF;
+	}
+
+	return layout->highmem + value * pagesize;
 }
 
 
 static unsigned int
-get_profile_last (const unsigned char data[], const oceanic_common_layout_t *layout)
+get_profile_last (const unsigned char data[], const oceanic_common_layout_t *layout, unsigned int pagesize)
 {
 	unsigned int value;
 
@@ -79,36 +85,69 @@ get_profile_last (const unsigned char data[], const oceanic_common_layout_t *lay
 		return array_uint16_le(data + 18);
 	}
 
-	if (layout->memsize > 0x20000)
-		return (value & 0x3FFF) * PAGESIZE;
-	else if (layout->memsize > 0x10000)
-		return (value & 0x1FFF) * PAGESIZE;
-	else
-		return (value & 0x0FFF) * PAGESIZE;
+	unsigned int npages = (layout->memsize - layout->highmem) / pagesize;
+
+	if (npages > 0x4000) {
+		value &= 0x7FFF;
+	} else if (npages > 0x2000) {
+		value &= 0x3FFF;
+	} else if (npages > 0x1000) {
+		value &= 0x1FFF;
+	} else {
+		value &= 0x0FFF;
+	}
+
+	return layout->highmem + value * pagesize;
 }
 
 
 static int
-oceanic_common_match_pattern (const unsigned char *string, const unsigned char *pattern)
+oceanic_common_match_pattern (const unsigned char *string, const unsigned char *pattern, unsigned int *firmware)
 {
+	unsigned int value = 0;
+	unsigned int count = 0;
+
 	for (unsigned int i = 0; i < PAGESIZE; ++i, ++pattern, ++string) {
-		if (*pattern != '\0' && *pattern != *string)
-			return 0;
+		if (*pattern != '\0') {
+			// Compare the pattern.
+			if (*pattern != *string)
+				return 0;
+		} else {
+			// Extract the firmware version.
+			// This is based on the assumption that (only) the first block of
+			// zeros in the pattern contains the firmware version.
+			if (i == 0 || *(pattern - 1) != '\0')
+				count++;
+			if (count == 1) {
+				value <<= 8;
+				value |= *string;
+			}
+		}
+	}
+
+	if (firmware) {
+		*firmware = value;
 	}
 
 	return 1;
 }
 
-
-int
-oceanic_common_match (const unsigned char *version, const oceanic_common_version_t patterns[], unsigned int n)
+const oceanic_common_version_t *
+oceanic_common_match (const unsigned char *version, const oceanic_common_version_t patterns[], size_t n, unsigned int *firmware)
 {
-	for (unsigned int i = 0; i < n; ++i) {
-		if (oceanic_common_match_pattern (version, patterns[i]))
-			return 1;
+	for (size_t i = 0; i < n; ++i) {
+		unsigned int fw = 0;
+		if (oceanic_common_match_pattern (version, patterns[i].pattern, &fw) &&
+			fw >= patterns[i].firmware)
+		{
+			if (firmware) {
+				*firmware = fw;
+			}
+			return patterns + i;
+		}
 	}
 
-	return 0;
+	return NULL;
 }
 
 
@@ -118,8 +157,10 @@ oceanic_common_device_init (oceanic_common_device_t *device)
 	assert (device != NULL);
 
 	// Set the default values.
+	device->firmware = 0;
 	memset (device->version, 0, sizeof (device->version));
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
+	device->model = 0;
 	device->layout = NULL;
 	device->multipage = 1;
 }
@@ -151,14 +192,16 @@ oceanic_common_device_set_fingerprint (dc_device_t *abstract, const unsigned cha
 dc_status_t
 oceanic_common_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_common_device_t *device = (oceanic_common_device_t *) abstract;
 
 	assert (device != NULL);
 	assert (device->layout != NULL);
 
-	// Erase the current contents of the buffer and
-	// allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, device->layout->memsize)) {
+	const oceanic_common_layout_t *layout = device->layout;
+
+	// Allocate the required amount of memory.
+	if (!dc_buffer_resize (buffer, layout->memsize)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
@@ -169,8 +212,30 @@ oceanic_common_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	vendor.size = sizeof (device->version);
 	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
 
-	return device_dump_read (abstract, dc_buffer_get_data (buffer),
+	// Download the memory dump.
+	status = device_dump_read (abstract, 0, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), PAGESIZE * device->multipage);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
+	}
+
+	// Emit a device info event.
+	unsigned char *id = dc_buffer_get_data (buffer) + layout->cf_devinfo;
+	dc_event_devinfo_t devinfo;
+	devinfo.model = array_uint16_be (id + 8);
+	devinfo.firmware = device->firmware;
+	if (layout->pt_mode_serial == 0)
+		devinfo.serial = array_convert_bcd2dec (id + 10, 3);
+	else if (layout->pt_mode_serial == 1)
+		devinfo.serial = array_convert_bin2dec (id + 11, 3);
+	else
+		devinfo.serial =
+			(id[11] & 0x0F) * 100000 + ((id[11] & 0xF0) >> 4) * 10000 +
+			(id[12] & 0x0F) * 1000   + ((id[12] & 0xF0) >> 4) * 100 +
+			(id[13] & 0x0F) * 10     + ((id[13] & 0xF0) >> 4) * 1;
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
+	return status;
 }
 
 
@@ -209,31 +274,37 @@ oceanic_common_device_logbook (dc_device_t *abstract, dc_event_progress_t *progr
 	// Get the logbook pointers.
 	unsigned int rb_logbook_first = array_uint16_le (pointers + 4);
 	unsigned int rb_logbook_last  = array_uint16_le (pointers + 6);
-	if (rb_logbook_first < layout->rb_logbook_begin ||
-		rb_logbook_first >= layout->rb_logbook_end ||
-		rb_logbook_last < layout->rb_logbook_begin ||
+	if (rb_logbook_last < layout->rb_logbook_begin ||
 		rb_logbook_last >= layout->rb_logbook_end)
 	{
-		ERROR (abstract->context, "Invalid logbook pointer detected (0x%04x 0x%04x).",
-			rb_logbook_first, rb_logbook_last);
+		ERROR (abstract->context, "Invalid logbook end pointer detected (0x%04x).", rb_logbook_last);
 		return DC_STATUS_DATAFORMAT;
 	}
 
-	// Calculate the end pointer and the number of bytes.
-	unsigned int rb_logbook_end, rb_logbook_size;
+	// Calculate the end pointer.
+	unsigned int rb_logbook_end = 0;
 	if (layout->pt_mode_global == 0) {
 		rb_logbook_end  = RB_LOGBOOK_INCR (rb_logbook_last, layout->rb_logbook_entry_size, layout);
-		rb_logbook_size = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_last, layout) + layout->rb_logbook_entry_size;
 	} else {
 		rb_logbook_end  = rb_logbook_last;
-		rb_logbook_size = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_last, layout);
-		// In a typical ringbuffer implementation with only two begin/end
-		// pointers, there is no distinction possible between an empty and
-		// a full ringbuffer. We always consider the ringbuffer full in
-		// that case, because an empty ringbuffer can be detected by
-		// inspecting the logbook entries once they are downloaded.
-		if (rb_logbook_first == rb_logbook_last)
-			rb_logbook_size = layout->rb_logbook_end - layout->rb_logbook_begin;
+	}
+
+	// Calculate the number of bytes.
+	// In a typical ringbuffer implementation with only two begin/end
+	// pointers, there is no distinction possible between an empty and a
+	// full ringbuffer. We always consider the ringbuffer full in that
+	// case, because an empty ringbuffer can be detected by inspecting
+	// the logbook entries once they are downloaded.
+	unsigned int rb_logbook_size = 0;
+	if (rb_logbook_first < layout->rb_logbook_begin ||
+		rb_logbook_first >= layout->rb_logbook_end)
+	{
+		// Fall back to downloading the entire logbook ringbuffer as
+		// workaround for an invalid logbook begin pointer!
+		ERROR (abstract->context, "Invalid logbook begin pointer detected (0x%04x).", rb_logbook_first);
+		rb_logbook_size = layout->rb_logbook_end - layout->rb_logbook_begin;
+	} else {
+		rb_logbook_size = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_end, layout);
 	}
 
 	// Update and emit a progress event.
@@ -266,6 +337,7 @@ oceanic_common_device_logbook (dc_device_t *abstract, dc_event_progress_t *progr
 	// entries first. If an already downloaded entry is identified (by means
 	// of its fingerprint), the transfer is aborted immediately to reduce
 	// the transfer time.
+	unsigned int count = 0;
 	unsigned int nbytes = 0;
 	unsigned int offset = rb_logbook_size;
 	while (nbytes < rb_logbook_size) {
@@ -289,8 +361,7 @@ oceanic_common_device_logbook (dc_device_t *abstract, dc_event_progress_t *progr
 		// fix this here.
 		if (array_isequal (logbooks + offset, layout->rb_logbook_entry_size, 0xFF)) {
 			WARNING (abstract->context, "Uninitialized logbook entries detected!");
-			offset += layout->rb_logbook_entry_size;
-			break;
+			continue;
 		}
 
 		// Compare the fingerprint to identify previously downloaded entries.
@@ -298,13 +369,19 @@ oceanic_common_device_logbook (dc_device_t *abstract, dc_event_progress_t *progr
 			offset += layout->rb_logbook_entry_size;
 			break;
 		}
+
+		count++;
 	}
 
 	// Update and emit a progress event.
 	progress->maximum -= rb_logbook_size - nbytes;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
 
-	dc_buffer_slice (logbook, offset, rb_logbook_size - offset);
+	if (count) {
+		dc_buffer_slice (logbook, offset, rb_logbook_size - offset);
+	} else {
+		dc_buffer_clear (logbook);
+	}
 
 	dc_rbstream_free (rbstream);
 
@@ -316,6 +393,7 @@ dc_status_t
 oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progress, dc_buffer_t *logbook, dc_dive_callback_t callback, void *userdata)
 {
 	oceanic_common_device_t *device = (oceanic_common_device_t *) abstract;
+	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	assert (device != NULL);
@@ -325,6 +403,9 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 
 	const oceanic_common_layout_t *layout = device->layout;
 
+	// Get the pagesize
+	unsigned int pagesize = layout->highmem ? 16 * PAGESIZE : PAGESIZE;
+
 	// Cache the logbook pointer and size.
 	const unsigned char *logbooks = dc_buffer_get_data (logbook);
 	unsigned int rb_logbook_size = dc_buffer_get_size (logbook);
@@ -346,9 +427,15 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		// Move to the start of the current entry.
 		entry -= layout->rb_logbook_entry_size;
 
+		// Skip uninitialized entries.
+		if (array_isequal (logbooks + entry, layout->rb_logbook_entry_size, 0xFF)) {
+			WARNING (abstract->context, "Skipping uninitialized logbook entry!");
+			continue;
+		}
+
 		// Get the profile pointers.
-		unsigned int rb_entry_first = get_profile_first (logbooks + entry, layout);
-		unsigned int rb_entry_last  = get_profile_last (logbooks + entry, layout);
+		unsigned int rb_entry_first = get_profile_first (logbooks + entry, layout, pagesize);
+		unsigned int rb_entry_last  = get_profile_last (logbooks + entry, layout, pagesize);
 		if (rb_entry_first < layout->rb_profile_begin ||
 			rb_entry_first >= layout->rb_profile_end ||
 			rb_entry_last < layout->rb_profile_begin ||
@@ -356,12 +443,13 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		{
 			ERROR (abstract->context, "Invalid ringbuffer pointer detected (0x%06x 0x%06x).",
 				rb_entry_first, rb_entry_last);
-			break;
+			status = DC_STATUS_DATAFORMAT;
+			continue;
 		}
 
 		// Calculate the end pointer and the number of bytes.
-		unsigned int rb_entry_end   = RB_PROFILE_INCR (rb_entry_last, PAGESIZE, layout);
-		unsigned int rb_entry_size  = RB_PROFILE_DISTANCE (rb_entry_first, rb_entry_last, layout) + PAGESIZE;
+		unsigned int rb_entry_end   = RB_PROFILE_INCR (rb_entry_last, pagesize, layout);
+		unsigned int rb_entry_size  = RB_PROFILE_DISTANCE (rb_entry_first, rb_entry_last, layout) + pagesize;
 
 		// Take the end pointer of the most recent logbook entry as the
 		// end of profile pointer.
@@ -394,6 +482,11 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 	progress->maximum -= (layout->rb_profile_end - layout->rb_profile_begin) - rb_profile_size;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
 
+	// Exit if there are no dives.
+	if (rb_profile_size == 0) {
+		return status;
+	}
+
 	// Create the ringbuffer stream.
 	dc_rbstream_t *rbstream = NULL;
 	rc = dc_rbstream_new (&rbstream, abstract, PAGESIZE, PAGESIZE * device->multipage, layout->rb_profile_begin, layout->rb_profile_end, rb_profile_end);
@@ -424,9 +517,15 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		// Move to the start of the current entry.
 		entry -= layout->rb_logbook_entry_size;
 
+		// Skip uninitialized entries.
+		if (array_isequal (logbooks + entry, layout->rb_logbook_entry_size, 0xFF)) {
+			WARNING (abstract->context, "Skipping uninitialized logbook entry!");
+			continue;
+		}
+
 		// Get the profile pointers.
-		unsigned int rb_entry_first = get_profile_first (logbooks + entry, layout);
-		unsigned int rb_entry_last  = get_profile_last (logbooks + entry, layout);
+		unsigned int rb_entry_first = get_profile_first (logbooks + entry, layout, pagesize);
+		unsigned int rb_entry_last  = get_profile_last (logbooks + entry, layout, pagesize);
 		if (rb_entry_first < layout->rb_profile_begin ||
 			rb_entry_first >= layout->rb_profile_end ||
 			rb_entry_last < layout->rb_profile_begin ||
@@ -434,14 +533,13 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		{
 			ERROR (abstract->context, "Invalid ringbuffer pointer detected (0x%06x 0x%06x).",
 				rb_entry_first, rb_entry_last);
-			dc_rbstream_free (rbstream);
-			free (profiles);
-			return DC_STATUS_DATAFORMAT;
+			status = DC_STATUS_DATAFORMAT;
+			continue;
 		}
 
 		// Calculate the end pointer and the number of bytes.
-		unsigned int rb_entry_end   = RB_PROFILE_INCR (rb_entry_last, PAGESIZE, layout);
-		unsigned int rb_entry_size  = RB_PROFILE_DISTANCE (rb_entry_first, rb_entry_last, layout) + PAGESIZE;
+		unsigned int rb_entry_end   = RB_PROFILE_INCR (rb_entry_last, pagesize, layout);
+		unsigned int rb_entry_size  = RB_PROFILE_DISTANCE (rb_entry_first, rb_entry_last, layout) + pagesize;
 
 		// Skip gaps between the profiles.
 		unsigned int gap = 0;
@@ -463,9 +561,8 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		rc = dc_rbstream_read (rbstream, progress, profiles + offset, rb_entry_size + gap);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to read the dive.");
-			dc_rbstream_free (rbstream);
-			free (profiles);
-			return rc;
+			status = rc;
+			break;
 		}
 
 		remaining -= rb_entry_size + gap;
@@ -476,6 +573,21 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		offset -= layout->rb_logbook_entry_size;
 		memcpy (profiles + offset, logbooks + entry, layout->rb_logbook_entry_size);
 
+		// Remove padding from the profile.
+		if (layout->highmem) {
+			// The logbook entry contains the total number of pages containing
+			// profile data, excluding the footer page. Limit the profile size
+			// to this size.
+			unsigned int value = array_uint16_le (profiles + offset + 12);
+			unsigned int value_hi = value & 0xE000;
+			unsigned int value_lo = value & 0x0FFF;
+			unsigned int npages = ((value_hi >> 1) | value_lo) + 1;
+			unsigned int length = npages * PAGESIZE;
+			if (rb_entry_size > length) {
+				rb_entry_size = length;
+			}
+		}
+
 		unsigned char *p = profiles + offset;
 		if (callback && !callback (p, rb_entry_size + layout->rb_logbook_entry_size, p, layout->rb_logbook_entry_size, userdata)) {
 			break;
@@ -485,7 +597,7 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 	dc_rbstream_free (rbstream);
 	free (profiles);
 
-	return DC_STATUS_SUCCESS;
+	return status;
 }
 
 
@@ -527,11 +639,11 @@ oceanic_common_device_foreach (dc_device_t *abstract, dc_dive_callback_t callbac
 	// Emit a device info event.
 	dc_event_devinfo_t devinfo;
 	devinfo.model = array_uint16_be (id + 8);
-	devinfo.firmware = 0;
+	devinfo.firmware = device->firmware;
 	if (layout->pt_mode_serial == 0)
-		devinfo.serial = bcd2dec (id[10]) * 10000 + bcd2dec (id[11]) * 100 + bcd2dec (id[12]);
+		devinfo.serial = array_convert_bcd2dec (id + 10, 3);
 	else if (layout->pt_mode_serial == 1)
-		devinfo.serial = id[11] * 10000 + id[12] * 100 + id[13];
+		devinfo.serial = array_convert_bin2dec (id + 11, 3);
 	else
 		devinfo.serial =
 			(id[11] & 0x0F) * 100000 + ((id[11] & 0xF0) >> 4) * 10000 +
