@@ -26,7 +26,6 @@
 #include "oceanic_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "ringbuffer.h"
 #include "checksum.h"
 
@@ -40,7 +39,7 @@
 
 typedef struct oceanic_veo250_device_t {
 	oceanic_common_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned int last;
 } oceanic_veo250_device_t;
 
@@ -63,19 +62,9 @@ static const oceanic_common_device_vtable_t oceanic_veo250_device_vtable = {
 	oceanic_common_device_profile,
 };
 
-static const oceanic_common_version_t oceanic_veo250_version[] = {
-	{"GENREACT \0\0 256K"},
-	{"VEO 200 R\0\0 256K"},
-	{"VEO 250 R\0\0 256K"},
-	{"SEEMANN R\0\0 256K"},
-	{"VEO 180 R\0\0 256K"},
-	{"AERISXR2 \0\0 256K"},
-	{"INSIGHT R\0\0 256K"},
-	{"HO DGO2 R\0\0 256K"},
-};
-
 static const oceanic_common_layout_t oceanic_veo250_layout = {
 	0x8000, /* memsize */
+	0, /* highmem */
 	0x0000, /* cf_devinfo */
 	0x0040, /* cf_pointers */
 	0x0400, /* rb_logbook_begin */
@@ -88,6 +77,16 @@ static const oceanic_common_layout_t oceanic_veo250_layout = {
 	1, /* pt_mode_serial */
 };
 
+static const oceanic_common_version_t versions[] = {
+	{"GENREACT \0\0 256K", 0, REACTPRO,      &oceanic_veo250_layout},
+	{"VEO 200 R\0\0 256K", 0, VEO200,        &oceanic_veo250_layout},
+	{"VEO 250 R\0\0 256K", 0, VEO250,        &oceanic_veo250_layout},
+	{"SEEMANN R\0\0 256K", 0, XP5,           &oceanic_veo250_layout},
+	{"VEO 180 R\0\0 256K", 0, VEO180,        &oceanic_veo250_layout},
+	{"AERISXR2 \0\0 256K", 0, XR2,           &oceanic_veo250_layout},
+	{"INSIGHT R\0\0 256K", 0, INSIGHT,       &oceanic_veo250_layout},
+	{"HO DGO2 R\0\0 256K", 0, DG02,          &oceanic_veo250_layout},
+};
 
 static dc_status_t
 oceanic_veo250_send (oceanic_veo250_device_t *device, const unsigned char command[], unsigned int csize)
@@ -99,10 +98,10 @@ oceanic_veo250_send (oceanic_veo250_device_t *device, const unsigned char comman
 		return DC_STATUS_CANCELLED;
 
 	// Discard garbage bytes.
-	dc_serial_purge (device->port, DC_DIRECTION_INPUT);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
 
 	// Send the command to the dive computer.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -110,7 +109,7 @@ oceanic_veo250_send (oceanic_veo250_device_t *device, const unsigned char comman
 
 	// Receive the response (ACK/NAK) of the dive computer.
 	unsigned char response = NAK;
-	status = dc_serial_read (device->port, &response, 1, NULL);
+	status = dc_iostream_read (device->iostream, &response, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -149,11 +148,11 @@ oceanic_veo250_transfer (oceanic_veo250_device_t *device, const unsigned char co
 			return rc;
 
 		// Delay the next attempt.
-		dc_serial_sleep (device->port, 100);
+		dc_iostream_sleep (device->iostream, 100);
 	}
 
 	// Receive the answer of the dive computer.
-	status = dc_serial_read (device->port, answer, asize, NULL);
+	status = dc_iostream_read (device->iostream, answer, asize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -177,7 +176,7 @@ oceanic_veo250_init (oceanic_veo250_device_t *device)
 
 	// Send the command to the dive computer.
 	unsigned char command[2] = {0x55, 0x00};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -186,7 +185,7 @@ oceanic_veo250_init (oceanic_veo250_device_t *device)
 	// Receive the answer of the dive computer.
 	size_t n = 0;
 	unsigned char answer[13] = {0};
-	status = dc_serial_read (device->port, answer, sizeof (answer), &n);
+	status = dc_iostream_read (device->iostream, answer, sizeof (answer), &n);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		if (n == 0)
@@ -215,7 +214,7 @@ oceanic_veo250_quit (oceanic_veo250_device_t *device)
 
 	// Send the command to the dive computer.
 	unsigned char command[2] = {0x98, 0x00};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -226,7 +225,7 @@ oceanic_veo250_quit (oceanic_veo250_device_t *device)
 
 
 dc_status_t
-oceanic_veo250_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+oceanic_veo250_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_veo250_device_t *device = NULL;
@@ -248,81 +247,89 @@ oceanic_veo250_device_open (dc_device_t **out, dc_context_t *context, const char
 	device->base.multipage = MULTIPAGE;
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 	device->last = 0;
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (9600 8N1).
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (3000 ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		goto error_close;
+		goto error_free;
 	}
 
+	// Clear the RTS line to reset the PIC inside the data cable as it
+	// may not have have been previously cleared. This ensures that the
+	// PIC will always start in a known state once RTS is set. Starting
+	// in a known default state is very important as the PIC won't
+	// respond to init commands unless it is in a default state.
+	status = dc_iostream_set_rts (device->iostream, 0);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to clear the RTS line.");
+		goto error_free;
+	}
+
+	// Hold RTS clear for a bit to allow PIC to reset.
+	dc_iostream_sleep (device->iostream, 100);
+
 	// Set the RTS line.
-	status = dc_serial_set_rts (device->port, 1);
+	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	dc_serial_sleep (device->port, 100);
+	dc_iostream_sleep (device->iostream, 100);
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Initialize the data cable (PPS mode).
 	status = oceanic_veo250_init (device);
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
 	// Delay the sending of the version command.
-	dc_serial_sleep (device->port, 100);
+	dc_iostream_sleep (device->iostream, 100);
 
 	// Switch the device from surface mode into download mode. Before sending
 	// this command, the device needs to be in PC mode (manually activated by
 	// the user), or already in download mode.
 	status = oceanic_veo250_device_version ((dc_device_t *) device, device->base.version, sizeof (device->base.version));
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
-	// Override the base class values.
-	if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_veo250_version)) {
-		device->base.layout = &oceanic_veo250_layout;
-	} else {
+	// Detect the memory layout.
+	const oceanic_common_version_t *version = OCEANIC_COMMON_MATCH(device->base.version, versions, &device->base.firmware);
+	if (version == NULL) {
 		WARNING (context, "Unsupported device detected!");
 		device->base.layout = &oceanic_veo250_layout;
+		device->base.model = 0;
+	} else {
+		device->base.layout = version->layout;
+		device->base.model = version->model;
 	}
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -342,12 +349,6 @@ oceanic_veo250_device_close (dc_device_t *abstract)
 		dc_status_set_error(&status, rc);
 	}
 
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
 	return status;
 }
 
