@@ -27,7 +27,6 @@
 #include "oceanic_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "ringbuffer.h"
 #include "checksum.h"
 #include "array.h"
@@ -41,8 +40,6 @@
 #define NAK 0xA5
 #define END 0x51
 
-#define AERIS500AI 0x4151
-
 typedef enum oceanic_vtpro_protocol_t {
 	MOD,
 	INTR,
@@ -50,8 +47,7 @@ typedef enum oceanic_vtpro_protocol_t {
 
 typedef struct oceanic_vtpro_device_t {
 	oceanic_common_device_t base;
-	dc_serial_t *port;
-	unsigned int model;
+	dc_iostream_t *iostream;
 	oceanic_vtpro_protocol_t protocol;
 } oceanic_vtpro_device_t;
 
@@ -75,21 +71,9 @@ static const oceanic_common_device_vtable_t oceanic_vtpro_device_vtable = {
 	oceanic_common_device_profile,
 };
 
-static const oceanic_common_version_t oceanic_vtpro_version[] = {
-	{"VERSAPRO \0\0 256K"},
-	{"ATMOSTWO \0\0 256K"},
-	{"PROPLUS2 \0\0 256K"},
-	{"ATMOSAIR \0\0 256K"},
-	{"VTPRO  r\0\0  256K"},
-	{"ELITE  r\0\0  256K"},
-};
-
-static const oceanic_common_version_t oceanic_wisdom_version[] = {
-	{"WISDOM r\0\0  256K"},
-};
-
 static const oceanic_common_layout_t oceanic_vtpro_layout = {
 	0x8000, /* memsize */
+	0, /* highmem */
 	0x0000, /* cf_devinfo */
 	0x0040, /* cf_pointers */
 	0x0240, /* rb_logbook_begin */
@@ -104,6 +88,7 @@ static const oceanic_common_layout_t oceanic_vtpro_layout = {
 
 static const oceanic_common_layout_t oceanic_wisdom_layout = {
 	0x8000, /* memsize */
+	0, /* highmem */
 	0x0000, /* cf_devinfo */
 	0x0040, /* cf_pointers */
 	0x03D0, /* rb_logbook_begin */
@@ -118,6 +103,7 @@ static const oceanic_common_layout_t oceanic_wisdom_layout = {
 
 static const oceanic_common_layout_t aeris_500ai_layout = {
 	0x20000, /* memsize */
+	0, /* highmem */
 	0x0000, /* cf_devinfo */
 	0x0110, /* cf_pointers */
 	0x0200, /* rb_logbook_begin */
@@ -130,6 +116,17 @@ static const oceanic_common_layout_t aeris_500ai_layout = {
 	2, /* pt_mode_serial */
 };
 
+static const oceanic_common_version_t versions[] = {
+	{"VERSAPRO \0\0 256K", 0, VERSAPRO,      &oceanic_vtpro_layout},
+	{"ATMOSTWO \0\0 256K", 0, ATMOS2,        &oceanic_vtpro_layout},
+	{"PROPLUS2 \0\0 256K", 0, PROPLUS2,      &oceanic_vtpro_layout},
+	{"ATMOSAIR \0\0 256K", 0, ATMOSAI,       &oceanic_vtpro_layout},
+	{"VTPRO  r\0\0  256K", 0, VTPRO,         &oceanic_vtpro_layout},
+	{"ELITE  r\0\0  256K", 0, ELITE,         &oceanic_vtpro_layout},
+
+	{"WISDOM r\0\0  256K", 0, WISDOM,        &oceanic_wisdom_layout},
+};
+
 static dc_status_t
 oceanic_vtpro_send (oceanic_vtpro_device_t *device, const unsigned char command[], unsigned int csize)
 {
@@ -140,7 +137,7 @@ oceanic_vtpro_send (oceanic_vtpro_device_t *device, const unsigned char command[
 		return DC_STATUS_CANCELLED;
 
 	// Send the command to the dive computer.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -148,7 +145,7 @@ oceanic_vtpro_send (oceanic_vtpro_device_t *device, const unsigned char command[
 
 	// Receive the response (ACK/NAK) of the dive computer.
 	unsigned char response = NAK;
-	status = dc_serial_read (device->port, &response, 1, NULL);
+	status = dc_iostream_read (device->iostream, &response, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -189,7 +186,7 @@ oceanic_vtpro_transfer (oceanic_vtpro_device_t *device, const unsigned char comm
 
 	if (asize) {
 		// Receive the answer of the dive computer.
-		status = dc_serial_read (device->port, answer, asize, NULL);
+		status = dc_iostream_read (device->iostream, answer, asize, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -210,7 +207,7 @@ oceanic_vtpro_init (oceanic_vtpro_device_t *device)
 	unsigned char command[2][2] = {
 		{0xAA, 0x00},
 		{0x20, 0x00}};
-	status = dc_serial_write (device->port, command[device->protocol], sizeof (command[device->protocol]), NULL);
+	status = dc_iostream_write (device->iostream, command[device->protocol], sizeof (command[device->protocol]), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -218,7 +215,7 @@ oceanic_vtpro_init (oceanic_vtpro_device_t *device)
 
 	// Receive the answer of the dive computer.
 	unsigned char answer[13] = {0};
-	status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+	status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -269,9 +266,13 @@ oceanic_vtpro_calibrate (oceanic_vtpro_device_t *device)
 	// device needs approximately 6 seconds to respond.
 	unsigned char answer[2] = {0};
 	unsigned char command[2] = {0x18, 0x00};
-	dc_serial_set_timeout (device->port, 9000);
-	dc_status_t rc = oceanic_vtpro_transfer (device, command, sizeof (command), answer, sizeof (answer));
-	dc_serial_set_timeout (device->port, 3000);
+	dc_status_t rc = dc_iostream_set_timeout (device->iostream, 9000);
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
+	rc = oceanic_vtpro_transfer (device, command, sizeof (command), answer, sizeof (answer));
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
+	rc = dc_iostream_set_timeout (device->iostream, 3000);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -337,7 +338,7 @@ oceanic_aeris500ai_device_logbook (dc_device_t *abstract, dc_event_progress_t *p
 	for (unsigned int i = 0; i < last + 1; ++i) {
 		// Receive the answer of the dive computer.
 		unsigned char answer[PAGESIZE / 2 + 1] = {0};
-		rc = dc_serial_read (device->port, answer, sizeof(answer), NULL);
+		rc = dc_iostream_read (device->iostream, answer, sizeof(answer), NULL);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return rc;
@@ -377,7 +378,7 @@ oceanic_vtpro_device_logbook (dc_device_t *abstract, dc_event_progress_t *progre
 {
 	oceanic_vtpro_device_t *device = (oceanic_vtpro_device_t *) abstract;
 
-	if (device->model == AERIS500AI) {
+	if (device->base.model == AERIS500AI) {
 		return oceanic_aeris500ai_device_logbook (abstract, progress, logbook);
 	} else {
 		return oceanic_common_device_logbook (abstract, progress, logbook);
@@ -385,7 +386,7 @@ oceanic_vtpro_device_logbook (dc_device_t *abstract, dc_event_progress_t *progre
 }
 
 dc_status_t
-oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
+oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream, unsigned int model)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_vtpro_device_t *device = NULL;
@@ -407,59 +408,65 @@ oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, const char
 	device->base.multipage = MULTIPAGE;
 
 	// Set the default values.
-	device->port = NULL;
-	device->model = model;
+	device->iostream = iostream;
 	if (model == AERIS500AI) {
 		device->protocol = INTR;
 	} else {
 		device->protocol = MOD;
 	}
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (9600 8N1).
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (3000 ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		goto error_close;
+		goto error_free;
 	}
 
+	// Clear the RTS line to reset the PIC inside the data cable as it
+	// may not have have been previously cleared. This ensures that the
+	// PIC will always start in a known state once RTS is set. Starting
+	// in a known default state is very important as the PIC won't
+	// respond to init commands unless it is in a default state.
+	status = dc_iostream_set_rts (device->iostream, 0);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to clear the RTS line.");
+		goto error_free;
+	}
+
+	// Hold RTS clear for a bit to allow PIC to reset.
+	dc_iostream_sleep (device->iostream, 100);
+
 	// Set the RTS line.
-	status = dc_serial_set_rts (device->port, 1);
+	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	dc_serial_sleep (device->port, device->protocol == MOD ? 100 : 1000);
+	dc_iostream_sleep (device->iostream, device->protocol == MOD ? 100 : 1000);
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Initialize the data cable (MOD mode).
 	status = oceanic_vtpro_init (device);
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
 	// Switch the device from surface mode into download mode. Before sending
@@ -467,7 +474,7 @@ oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, const char
 	// the user), or already in download mode.
 	status = oceanic_vtpro_device_version ((dc_device_t *) device, device->base.version, sizeof (device->base.version));
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
 	// Calibrate the device. Although calibration is optional, it's highly
@@ -475,27 +482,29 @@ oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, const char
 	// when processing the command itself is quite slow.
 	status = oceanic_vtpro_calibrate (device);
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
-	// Override the base class values.
+	// Detect the memory layout.
 	if (model == AERIS500AI) {
 		device->base.layout = &aeris_500ai_layout;
-	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_wisdom_version)) {
-		device->base.layout = &oceanic_wisdom_layout;
-	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_vtpro_version)) {
-		device->base.layout = &oceanic_vtpro_layout;
+		device->base.model = AERIS500AI;
 	} else {
-		WARNING (context, "Unsupported device detected!");
-		device->base.layout = &oceanic_vtpro_layout;
+		const oceanic_common_version_t * version = OCEANIC_COMMON_MATCH(device->base.version, versions, &device->base.firmware);
+		if (version == NULL) {
+			WARNING (context, "Unsupported device detected!");
+			device->base.layout = &oceanic_vtpro_layout;
+			device->base.model = 0;
+		} else {
+			device->base.layout = version->layout;
+			device->base.model = version->model;
+		}
 	}
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -515,12 +524,6 @@ oceanic_vtpro_device_close (dc_device_t *abstract)
 		dc_status_set_error(&status, rc);
 	}
 
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
 	return status;
 }
 
@@ -590,8 +593,8 @@ oceanic_vtpro_device_version (dc_device_t *abstract, unsigned char data[], unsig
 				return rc;
 
 			// Verify the checksum of the answer.
-			unsigned char crc = answer[PAGESIZE / 2];
-			unsigned char ccrc = checksum_add_uint4 (answer, PAGESIZE / 2, 0x00);
+			crc = answer[PAGESIZE / 2];
+			ccrc = checksum_add_uint4 (answer, PAGESIZE / 2, 0x00);
 			if (crc != ccrc) {
 				ERROR (abstract->context, "Unexpected answer checksum.");
 				return DC_STATUS_PROTOCOL;
