@@ -31,8 +31,6 @@
 
 #define ISINSTANCE(parser) dc_parser_isinstance((parser), &oceanic_vtpro_parser_vtable)
 
-#define AERIS500AI 0x4151
-
 typedef struct oceanic_vtpro_parser_t oceanic_vtpro_parser_t;
 
 struct oceanic_vtpro_parser_t {
@@ -53,6 +51,9 @@ static const dc_parser_vtable_t oceanic_vtpro_parser_vtable = {
 	sizeof(oceanic_vtpro_parser_t),
 	DC_FAMILY_OCEANIC_VTPRO,
 	oceanic_vtpro_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	oceanic_vtpro_parser_get_datetime, /* datetime */
 	oceanic_vtpro_parser_get_field, /* fields */
 	oceanic_vtpro_parser_samples_foreach, /* samples_foreach */
@@ -129,7 +130,7 @@ oceanic_vtpro_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 			else
 				datetime->year = bcd2dec (((p[32 + 3] & 0xC0) >> 2) + ((p[32 + 2] & 0xF0) >> 4)) + 2000;
 			datetime->month  = (p[4] & 0xF0) >> 4;
-			datetime->day    = bcd2dec (p[3]);
+			datetime->day    = bcd2dec (p[3] & 0x7F);
 			datetime->hour   = bcd2dec (p[1] & 0x7F);
 			pm = p[1] & 0x80;
 		}
@@ -181,7 +182,7 @@ oceanic_vtpro_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 		maxdepth = data[footer + 1];
 	} else {
 		oxygen = data[footer + 3];
-		maxdepth = array_uint16_le(data + footer + 0) & 0x0FFF;
+		maxdepth = array_uint16_le(data + footer + 0) & 0x01FF;
 	}
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
@@ -359,6 +360,21 @@ oceanic_vtpro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
 		if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 
+		// NDL / Deco
+		if (parser->model != AERIS500AI) {
+			unsigned int decostop = (data[offset + 5] & 0xF0) >> 4;
+			unsigned int decotime = array_uint16_le(data + offset + 4) & 0x0FFF;
+			if (decostop) {
+				sample.deco.type = DC_DECO_DECOSTOP;
+				sample.deco.depth = decostop * 10 * FEET;
+			} else {
+				sample.deco.type = DC_DECO_NDL;
+				sample.deco.depth = 0.0;
+			}
+			sample.deco.time = decotime * 60;
+			if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+		}
+
 		offset += PAGESIZE / 2;
 	}
 
