@@ -33,10 +33,8 @@
 #include "reefnet_sensusultra.h"
 #include "uwatec_aladin.h"
 #include "uwatec_memomouse.h"
-#include "uwatec_meridian.h"
 #include "uwatec_smart.h"
 #include "oceanic_atom2.h"
-#include "oceanic_atom2.h"
 #include "oceanic_veo250.h"
 #include "oceanic_vtpro.h"
 #include "mares_darwin.h"
@@ -48,6 +46,7 @@
 #include "hw_ostc3.h"
 #include "cressi_edy.h"
 #include "cressi_leonardo.h"
+#include "cressi_goa.h"
 #include "zeagle_n2ition3.h"
 #include "atomics_cobalt.h"
 #include "shearwater_petrel.h"
@@ -56,6 +55,15 @@
 #include "citizen_aqualand.h"
 #include "divesystem_idive.h"
 #include "cochran_commander.h"
+#include "tecdiving_divecomputereu.h"
+#include "mclean_extreme.h"
+#include "liquivision_lynx.h"
+#include "sporasub_sp2.h"
+#include "deepsix_excursion.h"
+#include "seac_screen.h"
+#include "deepblu_cosmiq.h"
+#include "oceans_s1.h"
+#include "divesoft_freedom.h"
 
 #include "context-private.h"
 #include "parser-private.h"
@@ -97,9 +105,7 @@ dc_parser_new_internal (dc_parser_t **out, dc_context_t *context, dc_family_t fa
 		rc = uwatec_memomouse_parser_create (&parser, context, devtime, systime);
 		break;
 	case DC_FAMILY_UWATEC_SMART:
-	case DC_FAMILY_UWATEC_MERIDIAN:
-	case DC_FAMILY_UWATEC_G2:
-		rc = uwatec_smart_parser_create (&parser, context, model, devtime, systime);
+		rc = uwatec_smart_parser_create (&parser, context, model);
 		break;
 	case DC_FAMILY_REEFNET_SENSUS:
 		rc = reefnet_sensus_parser_create (&parser, context, devtime, systime);
@@ -133,7 +139,7 @@ dc_parser_new_internal (dc_parser_t **out, dc_context_t *context, dc_family_t fa
 		rc = mares_iconhd_parser_create (&parser, context, model);
 		break;
 	case DC_FAMILY_HW_OSTC:
-		rc = hw_ostc_parser_create (&parser, context, 0);
+		rc = hw_ostc_parser_create (&parser, context);
 		break;
 	case DC_FAMILY_HW_FROG:
 	case DC_FAMILY_HW_OSTC3:
@@ -146,6 +152,9 @@ dc_parser_new_internal (dc_parser_t **out, dc_context_t *context, dc_family_t fa
 	case DC_FAMILY_CRESSI_LEONARDO:
 		rc = cressi_leonardo_parser_create (&parser, context, model);
 		break;
+	case DC_FAMILY_CRESSI_GOA:
+		rc = cressi_goa_parser_create (&parser, context, model);
+		break;
 	case DC_FAMILY_ATOMICS_COBALT:
 		rc = atomics_cobalt_parser_create (&parser, context);
 		break;
@@ -167,6 +176,33 @@ dc_parser_new_internal (dc_parser_t **out, dc_context_t *context, dc_family_t fa
 	case DC_FAMILY_COCHRAN_COMMANDER:
 		rc = cochran_commander_parser_create (&parser, context, model);
 		break;
+	case DC_FAMILY_TECDIVING_DIVECOMPUTEREU:
+		rc = tecdiving_divecomputereu_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_MCLEAN_EXTREME:
+		rc = mclean_extreme_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_LIQUIVISION_LYNX:
+		rc = liquivision_lynx_parser_create (&parser, context, model);
+		break;
+	case DC_FAMILY_SPORASUB_SP2:
+		rc = sporasub_sp2_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_DEEPSIX_EXCURSION:
+		rc = deepsix_excursion_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_SEAC_SCREEN:
+		rc = seac_screen_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_DEEPBLU_COSMIQ:
+		rc = deepblu_cosmiq_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_OCEANS_S1:
+		rc = oceans_s1_parser_create (&parser, context);
+		break;
+	case DC_FAMILY_DIVESOFT_FREEDOM:
+		rc = divesoft_freedom_parser_create (&parser, context);
+		break;
 	default:
 		return DC_STATUS_INVALIDARGS;
 	}
@@ -245,6 +281,45 @@ dc_parser_get_type (dc_parser_t *parser)
 }
 
 
+dc_status_t
+dc_parser_set_clock (dc_parser_t *parser, unsigned int devtime, dc_ticks_t systime)
+{
+	if (parser == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	if (parser->vtable->set_clock == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	return parser->vtable->set_clock (parser, devtime, systime);
+}
+
+
+dc_status_t
+dc_parser_set_atmospheric (dc_parser_t *parser, double atmospheric)
+{
+	if (parser == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	if (parser->vtable->set_atmospheric == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	return parser->vtable->set_atmospheric (parser, atmospheric);
+}
+
+
+dc_status_t
+dc_parser_set_density (dc_parser_t *parser, double density)
+{
+	if (parser == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	if (parser->vtable->set_density == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	return parser->vtable->set_density (parser, density);
+}
+
+
 dc_status_t
 dc_parser_set_data (dc_parser_t *parser, const unsigned char *data, unsigned int size)
 {
