@@ -49,6 +49,9 @@ struct reefnet_sensus_parser_t {
 };
 
 static dc_status_t reefnet_sensus_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
+static dc_status_t reefnet_sensus_parser_set_clock (dc_parser_t *abstract, unsigned int devtime, dc_ticks_t systime);
+static dc_status_t reefnet_sensus_parser_set_atmospheric (dc_parser_t *abstract, double atmospheric);
+static dc_status_t reefnet_sensus_parser_set_density (dc_parser_t *abstract, double density);
 static dc_status_t reefnet_sensus_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t reefnet_sensus_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t reefnet_sensus_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
@@ -57,6 +60,9 @@ static const dc_parser_vtable_t reefnet_sensus_parser_vtable = {
 	sizeof(reefnet_sensus_parser_t),
 	DC_FAMILY_REEFNET_SENSUS,
 	reefnet_sensus_parser_set_data, /* set_data */
+	reefnet_sensus_parser_set_clock, /* set_clock */
+	reefnet_sensus_parser_set_atmospheric, /* set_atmospheric */
+	reefnet_sensus_parser_set_density, /* set_density */
 	reefnet_sensus_parser_get_datetime, /* datetime */
 	reefnet_sensus_parser_get_field, /* fields */
 	reefnet_sensus_parser_samples_foreach, /* samples_foreach */
@@ -80,8 +86,8 @@ reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 	}
 
 	// Set the default values.
-	parser->atmospheric = ATM;
-	parser->hydrostatic = 1025.0 * GRAVITY;
+	parser->atmospheric = DEF_ATMOSPHERIC;
+	parser->hydrostatic = DEF_DENSITY_SALT * GRAVITY;
 	parser->devtime = devtime;
 	parser->systime = systime;
 	parser->cached = 0;
@@ -123,6 +129,40 @@ reefnet_sensus_parser_set_calibration (dc_parser_t *abstract, double atmospheric
 }
 
 
+static dc_status_t
+reefnet_sensus_parser_set_clock (dc_parser_t *abstract, unsigned int devtime, dc_ticks_t systime)
+{
+	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t *) abstract;
+
+	parser->devtime = devtime;
+	parser->systime = systime;
+
+	return DC_STATUS_SUCCESS;
+}
+
+
+static dc_status_t
+reefnet_sensus_parser_set_atmospheric (dc_parser_t *abstract, double atmospheric)
+{
+	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t *) abstract;
+
+	parser->atmospheric = atmospheric;
+
+	return DC_STATUS_SUCCESS;
+}
+
+
+static dc_status_t
+reefnet_sensus_parser_set_density (dc_parser_t *abstract, double density)
+{
+	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t *) abstract;
+
+	parser->hydrostatic = density * GRAVITY;
+
+	return DC_STATUS_SUCCESS;
+}
+
+
 static dc_status_t
 reefnet_sensus_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
@@ -133,7 +173,12 @@ reefnet_sensus_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *dateti
 
 	unsigned int timestamp = array_uint32_le (abstract->data + 2);
 
-	dc_ticks_t ticks = parser->systime - (parser->devtime - timestamp);
+	dc_ticks_t ticks = parser->systime;
+	if (timestamp < parser->devtime) {
+		ticks -= parser->devtime - timestamp;
+	} else {
+		ticks += timestamp - parser->devtime;
+	}
 
 	if (!dc_datetime_localtime (datetime, ticks))
 		return DC_STATUS_DATAFORMAT;
