@@ -26,7 +26,6 @@
 #include "reefnet_sensusultra.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "checksum.h"
 #include "array.h"
 
@@ -45,7 +44,7 @@
 
 typedef struct reefnet_sensusultra_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned char handshake[SZ_HANDSHAKE];
 	unsigned int timestamp;
 	unsigned int devtime;
@@ -55,7 +54,6 @@ typedef struct reefnet_sensusultra_device_t {
 static dc_status_t reefnet_sensusultra_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t reefnet_sensusultra_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t reefnet_sensusultra_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t reefnet_sensusultra_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t reefnet_sensusultra_device_vtable = {
 	sizeof(reefnet_sensusultra_device_t),
@@ -66,12 +64,12 @@ static const dc_device_vtable_t reefnet_sensusultra_device_vtable = {
 	reefnet_sensusultra_device_dump, /* dump */
 	reefnet_sensusultra_device_foreach, /* foreach */
 	NULL, /* timesync */
-	reefnet_sensusultra_device_close /* close */
+	NULL /* close */
 };
 
 
 dc_status_t
-reefnet_sensusultra_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+reefnet_sensusultra_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	reefnet_sensusultra_device_t *device = NULL;
@@ -87,65 +85,39 @@ reefnet_sensusultra_device_open (dc_device_t **out, dc_context_t *context, const
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 	device->timestamp = 0;
 	device->systime = (dc_ticks_t) -1;
 	device->devtime = 0;
 	memset (device->handshake, 0, sizeof (device->handshake));
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (115200 8N1).
+	status = dc_iostream_configure (device->iostream, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (115200 8N1).
-	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (3000ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
 
-static dc_status_t
-reefnet_sensusultra_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
-
 dc_status_t
 reefnet_sensusultra_device_get_handshake (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
@@ -190,7 +162,7 @@ reefnet_sensusultra_send_uchar (reefnet_sensusultra_device_t *device, unsigned c
 
 	// Wait for the prompt byte.
 	unsigned char prompt = 0;
-	status = dc_serial_read (device->port, &prompt, 1, NULL);
+	status = dc_iostream_read (device->iostream, &prompt, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the prompt byte");
 		return status;
@@ -203,7 +175,7 @@ reefnet_sensusultra_send_uchar (reefnet_sensusultra_device_t *device, unsigned c
 	}
 
 	// Send the value to the device.
-	status = dc_serial_write (device->port, &value, 1, NULL);
+	status = dc_iostream_write (device->iostream, &value, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the value.");
 		return status;
@@ -244,7 +216,7 @@ reefnet_sensusultra_packet (reefnet_sensusultra_device_t *device, unsigned char
 		return DC_STATUS_CANCELLED;
 
 	// Receive the data packet.
-	status = dc_serial_read (device->port, data, size, NULL);
+	status = dc_iostream_read (device->iostream, data, size, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the packet.");
 		return status;
@@ -252,7 +224,7 @@ reefnet_sensusultra_packet (reefnet_sensusultra_device_t *device, unsigned char
 
 	// Verify the checksum of the packet.
 	unsigned short crc = array_uint16_le (data + size - 2);
-	unsigned short ccrc = checksum_crc_ccitt_uint16 (data + header, size - header - 2);
+	unsigned short ccrc = checksum_crc16_ccitt (data + header, size - header - 2, 0xffff, 0x0000);
 	if (crc != ccrc) {
 		ERROR (abstract->context, "Unexpected answer checksum.");
 		return DC_STATUS_PROTOCOL;
@@ -346,7 +318,7 @@ static dc_status_t
 reefnet_sensusultra_send (reefnet_sensusultra_device_t *device, unsigned short command)
 {
 	// Flush the input and output buffers.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Wake-up the device and send the instruction code.
 	unsigned int nretries = 0;
@@ -366,8 +338,8 @@ reefnet_sensusultra_send (reefnet_sensusultra_device_t *device, unsigned short c
 		// not accidentally buffered by the host and (mis)interpreted as part
 		// of the next packet.
 
-		dc_serial_sleep (device->port, 250);
-		dc_serial_purge (device->port, DC_DIRECTION_ALL);
+		dc_iostream_sleep (device->iostream, 250);
+		dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -379,9 +351,8 @@ reefnet_sensusultra_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
-	// Erase the current contents of the buffer and
-	// pre-allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, SZ_MEMORY)) {
+	// Pre-allocate the required amount of memory.
+	if (!dc_buffer_reserve (buffer, SZ_MEMORY)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
@@ -506,7 +477,7 @@ reefnet_sensusultra_device_write_user (dc_device_t *abstract, const unsigned cha
 	}
 
 	// Send the checksum to the device.
-	unsigned short crc = checksum_crc_ccitt_uint16 (data, SZ_USER);
+	unsigned short crc = checksum_crc16_ccitt (data, SZ_USER, 0xffff, 0x0000);
 	rc = reefnet_sensusultra_send_ushort (device, crc);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
@@ -716,6 +687,7 @@ reefnet_sensusultra_device_foreach (dc_device_t *abstract, dc_dive_callback_t ca
 
 		// Prepend the packet to the buffer.
 		if (!dc_buffer_prepend (buffer, packet + 2, SZ_PACKET)) {
+			dc_buffer_free (buffer);
 			ERROR (abstract->context, "Insufficient buffer space available.");
 			return DC_STATUS_NOMEMORY;
 		}
