@@ -30,15 +30,13 @@
 #include <fcntl.h>	// fcntl
 #include <termios.h>	// tcgetattr, tcsetattr, cfsetispeed, cfsetospeed, tcflush, tcsendbreak
 #include <sys/ioctl.h>	// ioctl
-#include <sys/time.h>	// gettimeofday
-#include <time.h>	// nanosleep
+#include <sys/select.h>	// select
 #ifdef HAVE_LINUX_SERIAL_H
 #include <linux/serial.h>
 #endif
 #ifdef HAVE_IOKIT_SERIAL_IOSS_H
 #include <IOKit/serial/ioss.h>
 #endif
-#include <stdio.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <fnmatch.h>
@@ -53,28 +51,86 @@
 #define NOPTY 1
 #endif
 
-#include "serial.h"
+#include <libdivecomputer/serial.h>
+
 #include "common-private.h"
 #include "context-private.h"
+#include "iostream-private.h"
+#include "iterator-private.h"
+#include "descriptor-private.h"
+#include "platform.h"
+#include "timer.h"
 
-struct dc_serial_t {
-	/* Library context. */
-	dc_context_t *context;
+#define DIRNAME "/dev"
+
+static dc_status_t dc_serial_iterator_next (dc_iterator_t *iterator, void *item);
+static dc_status_t dc_serial_iterator_free (dc_iterator_t *iterator);
+
+static dc_status_t dc_serial_set_timeout (dc_iostream_t *iostream, int timeout);
+static dc_status_t dc_serial_set_break (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_dtr (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_rts (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_get_lines (dc_iostream_t *iostream, unsigned int *value);
+static dc_status_t dc_serial_get_available (dc_iostream_t *iostream, size_t *value);
+static dc_status_t dc_serial_configure (dc_iostream_t *iostream, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol);
+static dc_status_t dc_serial_poll (dc_iostream_t *iostream, int timeout);
+static dc_status_t dc_serial_read (dc_iostream_t *iostream, void *data, size_t size, size_t *actual);
+static dc_status_t dc_serial_write (dc_iostream_t *iostream, const void *data, size_t size, size_t *actual);
+static dc_status_t dc_serial_ioctl (dc_iostream_t *iostream, unsigned int request, void *data, size_t size);
+static dc_status_t dc_serial_flush (dc_iostream_t *iostream);
+static dc_status_t dc_serial_purge (dc_iostream_t *iostream, dc_direction_t direction);
+static dc_status_t dc_serial_sleep (dc_iostream_t *iostream, unsigned int milliseconds);
+static dc_status_t dc_serial_close (dc_iostream_t *iostream);
+
+struct dc_serial_device_t {
+	char name[256];
+};
+
+typedef struct dc_serial_iterator_t {
+	dc_iterator_t base;
+	dc_descriptor_t *descriptor;
+	DIR *dp;
+} dc_serial_iterator_t;
+
+typedef struct dc_serial_t {
+	dc_iostream_t base;
 	/*
 	 * The file descriptor corresponding to the serial port.
 	 */
 	int fd;
 	int timeout;
+	dc_timer_t *timer;
 	/*
 	 * Serial port settings are saved into this variable immediately
 	 * after the port is opened. These settings are restored when the
 	 * serial port is closed.
 	 */
 	struct termios tty;
-	/* Half-duplex settings */
-	int halfduplex;
-	unsigned int baudrate;
-	unsigned int nbits;
+} dc_serial_t;
+
+static const dc_iterator_vtable_t dc_serial_iterator_vtable = {
+	sizeof(dc_serial_iterator_t),
+	dc_serial_iterator_next,
+	dc_serial_iterator_free,
+};
+
+static const dc_iostream_vtable_t dc_serial_vtable = {
+	sizeof(dc_serial_t),
+	dc_serial_set_timeout, /* set_timeout */
+	dc_serial_set_break, /* set_break */
+	dc_serial_set_dtr, /* set_dtr */
+	dc_serial_set_rts, /* set_rts */
+	dc_serial_get_lines, /* get_lines */
+	dc_serial_get_available, /* get_available */
+	dc_serial_configure, /* configure */
+	dc_serial_poll, /* poll */
+	dc_serial_read, /* read */
+	dc_serial_write, /* write */
+	dc_serial_ioctl, /* ioctl */
+	dc_serial_flush, /* flush */
+	dc_serial_purge, /* purge */
+	dc_serial_sleep, /* sleep */
+	dc_serial_close, /* close */
 };
 
 static dc_status_t
@@ -95,12 +151,62 @@ syserror(int errcode)
 	}
 }
 
-dc_status_t
-dc_serial_enumerate (dc_serial_callback_t callback, void *userdata)
+const char *
+dc_serial_device_get_name (dc_serial_device_t *device)
 {
-	DIR *dp = NULL;
+	if (device == NULL || device->name[0] == '\0')
+		return NULL;
+
+	return device->name;
+}
+
+void
+dc_serial_device_free (dc_serial_device_t *device)
+{
+	free (device);
+}
+
+dc_status_t
+dc_serial_iterator_new (dc_iterator_t **out, dc_context_t *context, dc_descriptor_t *descriptor)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_iterator_t *iterator = NULL;
+
+	if (out == NULL)
+		return DC_STATUS_INVALIDARGS;
+
+	iterator = (dc_serial_iterator_t *) dc_iterator_allocate (context, &dc_serial_iterator_vtable);
+	if (iterator == NULL) {
+		SYSERROR (context, ENOMEM);
+		return DC_STATUS_NOMEMORY;
+	}
+
+	iterator->dp = opendir (DIRNAME);
+	if (iterator->dp == NULL) {
+		int errcode = errno;
+		SYSERROR (context, errcode);
+		status = syserror (errcode);
+		goto error_free;
+	}
+
+	iterator->descriptor = descriptor;
+
+	*out = (dc_iterator_t *) iterator;
+
+	return DC_STATUS_SUCCESS;
+
+error_free:
+	dc_iterator_deallocate ((dc_iterator_t *) iterator);
+	return status;
+}
+
+static dc_status_t
+dc_serial_iterator_next (dc_iterator_t *abstract, void *out)
+{
+	dc_serial_iterator_t *iterator = (dc_serial_iterator_t *) abstract;
+	dc_serial_device_t *device = NULL;
+
 	struct dirent *ep = NULL;
-	const char *dirname = "/dev";
 	const char *patterns[] = {
 #if defined (__APPLE__)
 		"tty.*",
@@ -113,59 +219,75 @@ dc_serial_enumerate (dc_serial_callback_t callback, void *userdata)
 		NULL
 	};
 
-	dp = opendir (dirname);
-	if (dp == NULL) {
-		return DC_STATUS_IO;
-	}
-
-	while ((ep = readdir (dp)) != NULL) {
+	while ((ep = readdir (iterator->dp)) != NULL) {
 		for (size_t i = 0; patterns[i] != NULL; ++i) {
-			if (fnmatch (patterns[i], ep->d_name, 0) == 0) {
-				char filename[1024];
-				int n = snprintf (filename, sizeof (filename), "%s/%s", dirname, ep->d_name);
-				if (n >= sizeof (filename)) {
-					closedir (dp);
-					return DC_STATUS_NOMEMORY;
-				}
+			if (fnmatch (patterns[i], ep->d_name, 0) != 0)
+				continue;
 
-				callback (filename, userdata);
-				break;
+			char filename[sizeof(device->name)];
+			int n = dc_platform_snprintf (filename, sizeof (filename), "%s/%s", DIRNAME, ep->d_name);
+			if (n < 0 || (size_t) n >= sizeof (filename)) {
+				return DC_STATUS_NOMEMORY;
 			}
+
+			if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_SERIAL, filename, NULL)) {
+				continue;
+			}
+
+			device = (dc_serial_device_t *) malloc (sizeof(dc_serial_device_t));
+			if (device == NULL) {
+				SYSERROR (abstract->context, ENOMEM);
+				return DC_STATUS_NOMEMORY;
+			}
+
+			strncpy(device->name, filename, sizeof(device->name));
+
+			*(dc_serial_device_t **) out = device;
+
+			return DC_STATUS_SUCCESS;
 		}
 	}
 
-	closedir (dp);
+	return DC_STATUS_DONE;
+}
+
+static dc_status_t
+dc_serial_iterator_free (dc_iterator_t *abstract)
+{
+	dc_serial_iterator_t *iterator = (dc_serial_iterator_t *) abstract;
+
+	closedir (iterator->dp);
 
 	return DC_STATUS_SUCCESS;
 }
 
 dc_status_t
-dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
+dc_serial_open (dc_iostream_t **out, dc_context_t *context, const char *name)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = NULL;
 
-	if (out == NULL)
+	if (out == NULL || name == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	INFO (context, "Open: name=%s", name ? name : "");
+	INFO (context, "Open: name=%s", name);
 
 	// Allocate memory.
-	dc_serial_t *device = (dc_serial_t *) malloc (sizeof (dc_serial_t));
+	device = (dc_serial_t *) dc_iostream_allocate (context, &dc_serial_vtable, DC_TRANSPORT_SERIAL);
 	if (device == NULL) {
 		SYSERROR (context, ENOMEM);
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Library context.
-	device->context = context;
-
 	// Default to blocking reads.
 	device->timeout = -1;
 
-	// Default to full-duplex.
-	device->halfduplex = 0;
-	device->baudrate = 0;
-	device->nbits = 0;
+	// Create a high resolution timer.
+	status = dc_timer_new (&device->timer);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to create a high resolution timer.");
+		goto error_free;
+	}
 
 	// Open the device in non-blocking mode, to return immediately
 	// without waiting for the modem connection to complete.
@@ -174,7 +296,7 @@ dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
 		int errcode = errno;
 		SYSERROR (context, errcode);
 		status = syserror (errcode);
-		goto error_free;
+		goto error_timer_free;
 	}
 
 #ifndef ENABLE_PTY
@@ -198,65 +320,64 @@ dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
 		goto error_close;
 	}
 
-	*out = device;
+	*out = (dc_iostream_t *) device;
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
 	close (device->fd);
+error_timer_free:
+	dc_timer_free (device->timer);
 error_free:
-	free (device);
+	dc_iostream_deallocate ((dc_iostream_t *) device);
 	return status;
 }
 
-dc_status_t
-dc_serial_close (dc_serial_t *device)
+static dc_status_t
+dc_serial_close (dc_iostream_t *abstract)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
-
-	if (device == NULL)
-		return DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Restore the initial terminal attributes.
 	if (tcsetattr (device->fd, TCSANOW, &device->tty) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		dc_status_set_error(&status, syserror (errcode));
 	}
 
 #ifndef ENABLE_PTY
 	// Disable exclusive access mode.
-	ioctl (device->fd, TIOCNXCL, NULL);
+	if (ioctl (device->fd, TIOCNXCL, NULL)) {
+		int errcode = errno;
+		SYSERROR (abstract->context, errcode);
+		dc_status_set_error(&status, syserror (errcode));
+	}
 #endif
 
 	// Close the device.
 	if (close (device->fd) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		dc_status_set_error(&status, syserror (errcode));
 	}
 
-	// Free memory.
-	free (device);
+	dc_timer_free (device->timer);
 
 	return status;
 }
 
-dc_status_t
-dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol)
+static dc_status_t
+dc_serial_configure (dc_iostream_t *abstract, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Configure: baudrate=%i, databits=%i, parity=%i, stopbits=%i, flowcontrol=%i",
-		baudrate, databits, parity, stopbits, flowcontrol);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Retrieve the current settings.
 	struct termios tty;
 	memset (&tty, 0, sizeof (tty));
 	if (tcgetattr (device->fd, &tty) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -353,7 +474,7 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 	if (cfsetispeed (&tty, baud) != 0 ||
 		cfsetospeed (&tty, baud) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -450,7 +571,7 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 	// Apply the new settings.
 	if (tcsetattr (device->fd, TCSANOW, &tty) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -461,7 +582,7 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 		struct serial_struct ss;
 		if (ioctl (device->fd, TIOCGSERIAL, &ss) != 0 && NOPTY) {
 			int errcode = errno;
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 
@@ -473,14 +594,14 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 		// Apply the new settings.
 		if (ioctl (device->fd, TIOCSSERIAL, &ss) != 0 && NOPTY) {
 			int errcode = errno;
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 #elif defined(IOSSIOSPEED)
 		speed_t speed = baudrate;
 		if (ioctl (device->fd, IOSSIOSPEED, &speed) != 0 && NOPTY) {
 			int errcode = errno;
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 #else
@@ -489,48 +610,30 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 #endif
 	}
 
-	device->baudrate = baudrate;
-	device->nbits = 1 + databits + stopbits + (parity ? 1 : 0);
-
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_timeout (dc_serial_t *device, int timeout)
+static dc_status_t
+dc_serial_set_timeout (dc_iostream_t *abstract, int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Timeout: value=%i", timeout);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	device->timeout = timeout;
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_halfduplex (dc_serial_t *device, unsigned int value)
+static dc_status_t
+dc_serial_set_latency (dc_iostream_t *abstract, unsigned int milliseconds)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	device->halfduplex = value;
-
-	return DC_STATUS_SUCCESS;
-}
-
-dc_status_t
-dc_serial_set_latency (dc_serial_t *device, unsigned int milliseconds)
-{
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 #if defined(TIOCGSERIAL) && defined(TIOCSSERIAL) && !defined(__ANDROID__)
 	// Get the current settings.
 	struct serial_struct ss;
 	if (ioctl (device->fd, TIOCGSERIAL, &ss) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -544,7 +647,7 @@ dc_serial_set_latency (dc_serial_t *device, unsigned int milliseconds)
 	// Apply the new settings.
 	if (ioctl (device->fd, TIOCSSERIAL, &ss) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 #elif defined(IOSSDATALAT)
@@ -554,7 +657,7 @@ dc_serial_set_latency (dc_serial_t *device, unsigned int milliseconds)
 	unsigned long usec = (milliseconds == 0 ? 1 : milliseconds * 1000);
 	if (ioctl (device->fd, IOSSDATALAT, &usec) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 #endif
@@ -562,22 +665,51 @@ dc_serial_set_latency (dc_serial_t *device, unsigned int milliseconds)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
+static dc_status_t
+dc_serial_poll (dc_iostream_t *abstract, int timeout)
+{
+	dc_serial_t *device = (dc_serial_t *) abstract;
+	int rc = 0;
+
+	do {
+		fd_set fds;
+		FD_ZERO (&fds);
+		FD_SET (device->fd, &fds);
+
+		struct timeval tv, *ptv = NULL;
+		if (timeout > 0) {
+			tv.tv_sec  = (timeout / 1000);
+			tv.tv_usec = (timeout % 1000) * 1000;
+			ptv = &tv;
+		} else if (timeout == 0) {
+			tv.tv_sec  = 0;
+			tv.tv_usec = 0;
+			ptv = &tv;
+		}
+
+		rc = select (device->fd + 1, &fds, NULL, NULL, ptv);
+	} while (rc < 0 && errno == EINTR);
+
+	if (rc < 0) {
+		int errcode = errno;
+		SYSERROR (abstract->context, errcode);
+		return syserror (errcode);
+	} else if (rc == 0) {
+		return DC_STATUS_TIMEOUT;
+	} else {
+		return DC_STATUS_SUCCESS;
+	}
+}
+
+static dc_status_t
+dc_serial_read (dc_iostream_t *abstract, void *data, size_t size, size_t *actual)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	size_t nbytes = 0;
 
-	if (device == NULL) {
-		status = DC_STATUS_INVALIDARGS;
-		goto out_invalidargs;
-	}
-
-	// The total timeout.
-	int timeout = device->timeout;
-
 	// The absolute target time.
-	struct timeval tve;
+	dc_usecs_t target = 0;
 
 	int init = 1;
 	while (nbytes < size) {
@@ -585,40 +717,45 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 		FD_ZERO (&fds);
 		FD_SET (device->fd, &fds);
 
-		struct timeval tvt;
-		if (timeout > 0) {
-			struct timeval now;
-			if (gettimeofday (&now, NULL) != 0) {
-				int errcode = errno;
-				SYSERROR (device->context, errcode);
-				status = syserror (errcode);
+		struct timeval tv, *ptv = NULL;
+		if (device->timeout > 0) {
+			dc_usecs_t timeout = 0;
+
+			dc_usecs_t now = 0;
+			status = dc_timer_now (device->timer, &now);
+			if (status != DC_STATUS_SUCCESS) {
 				goto out;
 			}
 
 			if (init) {
 				// Calculate the initial timeout.
-				tvt.tv_sec  = (timeout / 1000);
-				tvt.tv_usec = (timeout % 1000) * 1000;
+				timeout = (dc_usecs_t) device->timeout * 1000;
 				// Calculate the target time.
-				timeradd (&now, &tvt, &tve);
+				target = now + timeout;
+				init = 0;
 			} else {
 				// Calculate the remaining timeout.
-				if (timercmp (&now, &tve, <))
-					timersub (&tve, &now, &tvt);
-				else
-					timerclear (&tvt);
+				if (now < target) {
+					timeout = target - now;
+				} else {
+					timeout = 0;
+				}
 			}
-			init = 0;
-		} else if (timeout == 0) {
-			timerclear (&tvt);
+			tv.tv_sec  = timeout / 1000000;
+			tv.tv_usec = timeout % 1000000;
+			ptv = &tv;
+		} else if (device->timeout == 0) {
+			tv.tv_sec  = 0;
+			tv.tv_usec = 0;
+			ptv = &tv;
 		}
 
-		int rc = select (device->fd + 1, &fds, NULL, NULL, timeout >= 0 ? &tvt : NULL);
+		int rc = select (device->fd + 1, &fds, NULL, NULL, ptv);
 		if (rc < 0) {
 			int errcode = errno;
 			if (errcode == EINTR)
 				continue; // Retry.
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		} else if (rc == 0) {
@@ -630,7 +767,7 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 			int errcode = errno;
 			if (errcode == EINTR || errcode == EAGAIN)
 				continue; // Retry.
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		} else if (n == 0) {
@@ -645,37 +782,19 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 	}
 
 out:
-	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Read", (unsigned char *) data, nbytes);
-
-out_invalidargs:
 	if (actual)
 		*actual = nbytes;
 
 	return status;
 }
 
-dc_status_t
-dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *actual)
+static dc_status_t
+dc_serial_write (dc_iostream_t *abstract, const void *data, size_t size, size_t *actual)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	size_t nbytes = 0;
 
-	if (device == NULL) {
-		status = DC_STATUS_INVALIDARGS;
-		goto out_invalidargs;
-	}
-
-	struct timeval tve, tvb;
-	if (device->halfduplex) {
-		// Get the current time.
-		if (gettimeofday (&tvb, NULL) != 0) {
-			int errcode = errno;
-			SYSERROR (device->context, errcode);
-			status = syserror (errcode);
-			goto out;
-		}
-	}
-
 	while (nbytes < size) {
 		fd_set fds;
 		FD_ZERO (&fds);
@@ -686,7 +805,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 			int errcode = errno;
 			if (errcode == EINTR)
 				continue; // Retry.
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		} else if (rc == 0) {
@@ -698,7 +817,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 			int errcode = errno;
 			if (errcode == EINTR || errcode == EAGAIN)
 				continue; // Retry.
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		} else if (n == 0) {
@@ -717,58 +836,34 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 #endif
 		int errcode = errno;
 		if (errcode != EINTR ) {
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		}
 	}
 
-	if (device->halfduplex) {
-		// Get the current time.
-		if (gettimeofday (&tve, NULL) != 0) {
-			int errcode = errno;
-			SYSERROR (device->context, errcode);
-			status = syserror (errcode);
-			goto out;
-		}
-
-		// Calculate the elapsed time (microseconds).
-		struct timeval tvt;
-		timersub (&tve, &tvb, &tvt);
-		unsigned long elapsed = tvt.tv_sec * 1000000 + tvt.tv_usec;
-
-		// Calculate the expected duration (microseconds). A 2 millisecond fudge
-		// factor is added because it improves the success rate significantly.
-		unsigned long expected = 1000000.0 * device->nbits / device->baudrate * size + 0.5 + 2000;
-
-		// Wait for the remaining time.
-		if (elapsed < expected) {
-			unsigned long remaining = expected - elapsed;
-
-			// The remaining time is rounded up to the nearest millisecond to
-			// match the Windows implementation. The higher resolution is
-			// pointless anyway, since we already added a fudge factor above.
-			dc_serial_sleep (device, (remaining + 999) / 1000);
-		}
-	}
-
 out:
-	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Write", (const unsigned char *) data, nbytes);
-
-out_invalidargs:
 	if (actual)
 		*actual = nbytes;
 
 	return status;
 }
 
-dc_status_t
-dc_serial_purge (dc_serial_t *device, dc_direction_t direction)
+static dc_status_t
+dc_serial_ioctl (dc_iostream_t *abstract, unsigned int request, void *data, size_t size)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+	switch (request) {
+	case DC_IOCTL_SERIAL_SET_LATENCY:
+		return dc_serial_set_latency (abstract, *(unsigned int *) data);
+	default:
+		return DC_STATUS_UNSUPPORTED;
+	}
+}
 
-	INFO (device->context, "Purge: direction=%u", direction);
+static dc_status_t
+dc_serial_purge (dc_iostream_t *abstract, dc_direction_t direction)
+{
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	int flags = 0;
 
@@ -788,93 +883,78 @@ dc_serial_purge (dc_serial_t *device, dc_direction_t direction)
 
 	if (tcflush (device->fd, flags) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_flush (dc_serial_t *device)
+static dc_status_t
+dc_serial_flush (dc_iostream_t *abstract)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Flush: none");
-
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_break (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_break (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Break: value=%i", level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	unsigned long action = (level ? TIOCSBRK : TIOCCBRK);
 
 	if (ioctl (device->fd, action, NULL) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_dtr (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_dtr (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "DTR: value=%i", level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	unsigned long action = (level ? TIOCMBIS : TIOCMBIC);
 
 	int value = TIOCM_DTR;
 	if (ioctl (device->fd, action, &value) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_rts (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_rts (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "RTS: value=%i", level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	unsigned long action = (level ? TIOCMBIS : TIOCMBIC);
 
 	int value = TIOCM_RTS;
 	if (ioctl (device->fd, action, &value) != 0 && NOPTY) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_get_available (dc_serial_t *device, size_t *value)
+static dc_status_t
+dc_serial_get_available (dc_iostream_t *abstract, size_t *value)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	int bytes = 0;
 	if (ioctl (device->fd, TIOCINQ, &bytes) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -884,18 +964,16 @@ dc_serial_get_available (dc_serial_t *device, size_t *value)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_get_lines (dc_serial_t *device, unsigned int *value)
+static dc_status_t
+dc_serial_get_lines (dc_iostream_t *abstract, unsigned int *value)
 {
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	unsigned int lines = 0;
 
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
 	int status = 0;
 	if (ioctl (device->fd, TIOCMGET, &status) != 0) {
 		int errcode = errno;
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -914,24 +992,13 @@ dc_serial_get_lines (dc_serial_t *device, unsigned int *value)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_sleep (dc_serial_t *device, unsigned int timeout)
+static dc_status_t
+dc_serial_sleep (dc_iostream_t *abstract, unsigned int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Sleep: value=%u", timeout);
-
-	struct timespec ts;
-	ts.tv_sec  = (timeout / 1000);
-	ts.tv_nsec = (timeout % 1000) * 1000000;
-
-	while (nanosleep (&ts, &ts) != 0) {
+	if (dc_platform_sleep (timeout) != 0) {
 		int errcode = errno;
-		if (errcode != EINTR ) {
-			SYSERROR (device->context, errcode);
-			return syserror (errcode);
-		}
+		SYSERROR (abstract->context, errcode);
+		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
