@@ -21,16 +21,52 @@
 
 #include <stdlib.h>
 
+#define WIN32_LEAN_AND_MEAN
 #define NOGDI
 #include <windows.h>
 
-#include "serial.h"
+#include <libdivecomputer/serial.h>
+
 #include "common-private.h"
 #include "context-private.h"
+#include "iostream-private.h"
+#include "iterator-private.h"
+#include "descriptor-private.h"
+#include "platform.h"
 
-struct dc_serial_t {
-	/* Library context. */
-	dc_context_t *context;
+static dc_status_t dc_serial_iterator_next (dc_iterator_t *iterator, void *item);
+static dc_status_t dc_serial_iterator_free (dc_iterator_t *iterator);
+
+static dc_status_t dc_serial_set_timeout (dc_iostream_t *iostream, int timeout);
+static dc_status_t dc_serial_set_break (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_dtr (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_rts (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_get_lines (dc_iostream_t *iostream, unsigned int *value);
+static dc_status_t dc_serial_get_available (dc_iostream_t *iostream, size_t *value);
+static dc_status_t dc_serial_configure (dc_iostream_t *iostream, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol);
+static dc_status_t dc_serial_poll (dc_iostream_t *iostream, int timeout);
+static dc_status_t dc_serial_read (dc_iostream_t *iostream, void *data, size_t size, size_t *actual);
+static dc_status_t dc_serial_write (dc_iostream_t *iostream, const void *data, size_t size, size_t *actual);
+static dc_status_t dc_serial_ioctl (dc_iostream_t *iostream, unsigned int request, void *data, size_t size);
+static dc_status_t dc_serial_flush (dc_iostream_t *iostream);
+static dc_status_t dc_serial_purge (dc_iostream_t *iostream, dc_direction_t direction);
+static dc_status_t dc_serial_sleep (dc_iostream_t *iostream, unsigned int milliseconds);
+static dc_status_t dc_serial_close (dc_iostream_t *iostream);
+
+struct dc_serial_device_t {
+	char name[256];
+};
+
+typedef struct dc_serial_iterator_t {
+	dc_iterator_t base;
+	dc_descriptor_t *descriptor;
+	HKEY hKey;
+	DWORD count;
+	DWORD current;
+} dc_serial_iterator_t;
+
+typedef struct dc_serial_t {
+	dc_iostream_t base;
 	/*
 	 * The file descriptor corresponding to the serial port.
 	 */
@@ -42,10 +78,36 @@ struct dc_serial_t {
 	 */
 	DCB dcb;
 	COMMTIMEOUTS timeouts;
-	/* Half-duplex settings */
-	int halfduplex;
-	unsigned int baudrate;
-	unsigned int nbits;
+
+	HANDLE hReadWrite, hPoll;
+	OVERLAPPED overlapped;
+	DWORD events;
+	BOOL pending;
+} dc_serial_t;
+
+static const dc_iterator_vtable_t dc_serial_iterator_vtable = {
+	sizeof(dc_serial_iterator_t),
+	dc_serial_iterator_next,
+	dc_serial_iterator_free,
+};
+
+static const dc_iostream_vtable_t dc_serial_vtable = {
+	sizeof(dc_serial_t),
+	dc_serial_set_timeout, /* set_timeout */
+	dc_serial_set_break, /* set_break */
+	dc_serial_set_dtr, /* set_dtr */
+	dc_serial_set_rts, /* set_rts */
+	dc_serial_get_lines, /* get_lines */
+	dc_serial_get_available, /* get_available */
+	dc_serial_configure, /* configure */
+	dc_serial_poll, /* poll */
+	dc_serial_read, /* read */
+	dc_serial_write, /* write */
+	dc_serial_ioctl, /* ioctl */
+	dc_serial_flush, /* flush */
+	dc_serial_purge, /* purge */
+	dc_serial_sleep, /* sleep */
+	dc_serial_close, /* close */
 };
 
 static dc_status_t
@@ -65,37 +127,93 @@ syserror(DWORD errcode)
 	}
 }
 
-dc_status_t
-dc_serial_enumerate (dc_serial_callback_t callback, void *userdata)
+const char *
+dc_serial_device_get_name (dc_serial_device_t *device)
 {
+	if (device == NULL || device->name[0] == '\0')
+		return NULL;
+
+	return device->name;
+}
+
+void
+dc_serial_device_free (dc_serial_device_t *device)
+{
+	free (device);
+}
+
+dc_status_t
+dc_serial_iterator_new (dc_iterator_t **out, dc_context_t *context, dc_descriptor_t *descriptor)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_iterator_t *iterator = NULL;
+	HKEY hKey = NULL;
+	DWORD count = 0;
+	LONG rc = 0;
+
+	if (out == NULL)
+		return DC_STATUS_INVALIDARGS;
+
+	iterator = (dc_serial_iterator_t *) dc_iterator_allocate (context, &dc_serial_iterator_vtable);
+	if (iterator == NULL) {
+		SYSERROR (context, ERROR_OUTOFMEMORY);
+		return DC_STATUS_NOMEMORY;
+	}
+
 	// Open the registry key.
-	HKEY hKey;
-	LONG rc = RegOpenKeyExA (HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\SERIALCOMM", 0, KEY_QUERY_VALUE, &hKey);
+	rc = RegOpenKeyExA (HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\SERIALCOMM", 0, KEY_QUERY_VALUE, &hKey);
 	if (rc != ERROR_SUCCESS) {
-		if (rc == ERROR_FILE_NOT_FOUND)
-			return DC_STATUS_SUCCESS;
-		else
-			return DC_STATUS_IO;
+		if (rc == ERROR_FILE_NOT_FOUND) {
+			hKey = NULL;
+		} else {
+			SYSERROR (context, rc);
+			status = syserror (rc);
+			goto error_free;
+		}
 	}
 
 	// Get the number of values.
-	DWORD count = 0;
-	rc = RegQueryInfoKey (hKey, NULL, NULL, NULL, NULL, NULL, NULL, &count, NULL, NULL, NULL, NULL);
-	if (rc != ERROR_SUCCESS) {
-		RegCloseKey(hKey);
-		return DC_STATUS_IO;
+	if (hKey) {
+		rc = RegQueryInfoKey (hKey, NULL, NULL, NULL, NULL, NULL, NULL, &count, NULL, NULL, NULL, NULL);
+		if (rc != ERROR_SUCCESS) {
+			SYSERROR (context, rc);
+			status = syserror (rc);
+			goto error_close;
+		}
 	}
 
-	for (DWORD i = 0; i < count; ++i) {
+	iterator->descriptor = descriptor;
+	iterator->hKey = hKey;
+	iterator->count = count;
+	iterator->current = 0;
+
+	*out = (dc_iterator_t *) iterator;
+
+	return DC_STATUS_SUCCESS;
+
+error_close:
+	RegCloseKey (hKey);
+error_free:
+	dc_iterator_deallocate ((dc_iterator_t *) iterator);
+	return status;
+}
+
+static dc_status_t
+dc_serial_iterator_next (dc_iterator_t *abstract, void *out)
+{
+	dc_serial_iterator_t *iterator = (dc_serial_iterator_t *) abstract;
+	dc_serial_device_t *device = NULL;
+
+	while (iterator->current < iterator->count) {
 		// Get the value name, data and type.
-		char name[512], data[512];
+		char name[256], data[sizeof(device->name)];
 		DWORD name_len = sizeof (name);
 		DWORD data_len = sizeof (data);
 		DWORD type = 0;
-		rc = RegEnumValueA (hKey, i, name, &name_len, NULL, &type, (LPBYTE) data, &data_len);
+		LONG rc = RegEnumValueA (iterator->hKey, iterator->current++, name, &name_len, NULL, &type, (LPBYTE) data, &data_len);
 		if (rc != ERROR_SUCCESS) {
-			RegCloseKey(hKey);
-			return DC_STATUS_IO;
+			SYSERROR (abstract->context, rc);
+			return syserror (rc);
 		}
 
 		// Ignore non-string values.
@@ -104,35 +222,59 @@ dc_serial_enumerate (dc_serial_callback_t callback, void *userdata)
 
 		// Prevent a possible buffer overflow.
 		if (data_len >= sizeof (data)) {
-			RegCloseKey(hKey);
 			return DC_STATUS_NOMEMORY;
 		}
 
 		// Null terminate the string.
 		data[data_len] = 0;
 
-		callback (data, userdata);
+		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_SERIAL, data, NULL)) {
+			continue;
+		}
+
+		device = (dc_serial_device_t *) malloc (sizeof(dc_serial_device_t));
+		if (device == NULL) {
+			SYSERROR (abstract->context, ERROR_OUTOFMEMORY);
+			return DC_STATUS_NOMEMORY;
+		}
+
+		strncpy(device->name, data, sizeof(device->name));
+
+		*(dc_serial_device_t **) out = device;
+
+		return DC_STATUS_SUCCESS;
 	}
 
-	RegCloseKey(hKey);
+	return DC_STATUS_DONE;
+}
+
+static dc_status_t
+dc_serial_iterator_free (dc_iterator_t *abstract)
+{
+	dc_serial_iterator_t *iterator = (dc_serial_iterator_t *) abstract;
+
+	if (iterator->hKey) {
+		RegCloseKey (iterator->hKey);
+	}
 
 	return DC_STATUS_SUCCESS;
 }
 
 dc_status_t
-dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
+dc_serial_open (dc_iostream_t **out, dc_context_t *context, const char *name)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = NULL;
 
-	if (out == NULL)
+	if (out == NULL || name == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	INFO (context, "Open: name=%s", name ? name : "");
+	INFO (context, "Open: name=%s", name);
 
 	// Build the device name.
 	const char *devname = NULL;
 	char buffer[MAX_PATH] = "\\\\.\\";
-	if (name && strncmp (name, buffer, 4) != 0) {
+	if (strncmp (name, buffer, 4) != 0) {
 		size_t length = strlen (name) + 1;
 		if (length + 4 > sizeof (buffer))
 			return DC_STATUS_NOMEMORY;
@@ -143,32 +285,47 @@ dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
 	}
 
 	// Allocate memory.
-	dc_serial_t *device = (dc_serial_t *) malloc (sizeof (dc_serial_t));
+	device = (dc_serial_t *) dc_iostream_allocate (context, &dc_serial_vtable, DC_TRANSPORT_SERIAL);
 	if (device == NULL) {
 		SYSERROR (context, ERROR_OUTOFMEMORY);
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Library context.
-	device->context = context;
+	// Default values.
+	memset(&device->overlapped, 0, sizeof(device->overlapped));
+	device->events = 0;
+	device->pending = FALSE;
 
-	// Default to full-duplex.
-	device->halfduplex = 0;
-	device->baudrate = 0;
-	device->nbits = 0;
+	// Create a manual reset event for I/O.
+	device->hReadWrite = CreateEvent (NULL, TRUE, FALSE, NULL);
+	if (device->hReadWrite == INVALID_HANDLE_VALUE) {
+		DWORD errcode = GetLastError ();
+		SYSERROR (context, errcode);
+		status = syserror (errcode);
+		goto error_free;
+	}
+
+	// Create a manual reset event for polling.
+	device->hPoll = CreateEvent (NULL, TRUE, FALSE, NULL);
+	if (device->hPoll == INVALID_HANDLE_VALUE) {
+		DWORD errcode = GetLastError ();
+		SYSERROR (context, errcode);
+		status = syserror (errcode);
+		goto error_free_readwrite;
+	}
 
 	// Open the device.
 	device->hFile = CreateFileA (devname,
 			GENERIC_READ | GENERIC_WRITE, 0,
 			NULL, // No security attributes.
 			OPEN_EXISTING,
-			0, // Non-overlapped I/O.
+			FILE_FLAG_OVERLAPPED,
 			NULL);
 	if (device->hFile == INVALID_HANDLE_VALUE) {
 		DWORD errcode = GetLastError ();
 		SYSERROR (context, errcode);
 		status = syserror (errcode);
-		goto error_free;
+		goto error_free_poll;
 	}
 
 	// Retrieve the current communication settings and timeouts,
@@ -183,60 +340,69 @@ dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
 		goto error_close;
 	}
 
-	*out = device;
+	// Enable event monitoring.
+	if (!SetCommMask (device->hFile, EV_RXCHAR)) {
+		DWORD errcode = GetLastError ();
+		SYSERROR (context, errcode);
+		status = syserror (errcode);
+		goto error_close;
+	}
+
+	*out = (dc_iostream_t *) device;
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
 	CloseHandle (device->hFile);
+error_free_poll:
+	CloseHandle (device->hPoll);
+error_free_readwrite:
+	CloseHandle (device->hReadWrite);
 error_free:
-	free (device);
+	dc_iostream_deallocate ((dc_iostream_t *) device);
 	return status;
 }
 
-dc_status_t
-dc_serial_close (dc_serial_t *device)
+static dc_status_t
+dc_serial_close (dc_iostream_t *abstract)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
-	if (device == NULL)
-		return DC_STATUS_SUCCESS;
+	// Disable event monitoring.
+	SetCommMask (device->hFile, 0);
 
 	// Restore the initial communication settings and timeouts.
 	if (!SetCommState (device->hFile, &device->dcb) ||
 		!SetCommTimeouts (device->hFile, &device->timeouts)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		dc_status_set_error(&status, syserror (errcode));
 	}
 
 	// Close the device.
 	if (!CloseHandle (device->hFile)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		dc_status_set_error(&status, syserror (errcode));
 	}
 
-	// Free memory.
-	free (device);
+	CloseHandle (device->hPoll);
+	CloseHandle (device->hReadWrite);
 
 	return status;
 }
 
-dc_status_t
-dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol)
+static dc_status_t
+dc_serial_configure (dc_iostream_t *abstract, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Configure: baudrate=%i, databits=%i, parity=%i, stopbits=%i, flowcontrol=%i",
-		baudrate, databits, parity, stopbits, flowcontrol);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Retrieve the current settings.
 	DCB dcb;
 	if (!GetCommState (device->hFile, &dcb)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -326,29 +492,23 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 	// Apply the new settings.
 	if (!SetCommState (device->hFile, &dcb)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
-	device->baudrate = baudrate;
-	device->nbits = 1 + databits + stopbits + (parity ? 1 : 0);
-
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_timeout (dc_serial_t *device, int timeout)
+static dc_status_t
+dc_serial_set_timeout (dc_iostream_t *abstract, int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Timeout: value=%i", timeout);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Retrieve the current timeouts.
 	COMMTIMEOUTS timeouts;
 	if (!GetCommTimeouts (device->hFile, &timeouts)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -379,47 +539,93 @@ dc_serial_set_timeout (dc_serial_t *device, int timeout)
 	// Activate the new timeouts.
 	if (!SetCommTimeouts (device->hFile, &timeouts)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_halfduplex (dc_serial_t *device, unsigned int value)
+static dc_status_t
+dc_serial_poll (dc_iostream_t *abstract, int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
-	device->halfduplex = value;
+	while (1) {
+		COMSTAT stats;
+		if (!ClearCommError (device->hFile, NULL, &stats)) {
+			DWORD errcode = GetLastError ();
+			SYSERROR (abstract->context, errcode);
+			return syserror (errcode);
+		}
 
-	return DC_STATUS_SUCCESS;
-}
+		if (stats.cbInQue)
+			break;
 
-dc_status_t
-dc_serial_set_latency (dc_serial_t *device, unsigned int value)
-{
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+		if (!device->pending) {
+			memset(&device->overlapped, 0, sizeof(device->overlapped));
+			device->overlapped.hEvent = device->hPoll;
+			device->events = 0;
+			if (!WaitCommEvent (device->hFile, &device->events, &device->overlapped)) {
+				DWORD errcode = GetLastError ();
+				if (errcode != ERROR_IO_PENDING) {
+					SYSERROR (abstract->context, errcode);
+					return syserror (errcode);
+				}
+				device->pending = TRUE;
+			}
+		}
 
-	return DC_STATUS_SUCCESS;
-}
+		if (device->pending) {
+			DWORD errcode = 0;
+			DWORD rc = WaitForSingleObject (device->hPoll, timeout >= 0 ? (DWORD) timeout : INFINITE);
+			switch (rc) {
+			case WAIT_OBJECT_0:
+				break;
+			case WAIT_TIMEOUT:
+				return DC_STATUS_TIMEOUT;
+			default:
+				errcode = GetLastError ();
+				SYSERROR (abstract->context, errcode);
+				return syserror (errcode);
+			}
+		}
 
-dc_status_t
-dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	DWORD dwRead = 0;
+		DWORD dummy = 0;
+		if (!GetOverlappedResult (device->hFile, &device->overlapped, &dummy, TRUE)) {
+			DWORD errcode = GetLastError ();
+			SYSERROR (abstract->context, errcode);
+			return syserror (errcode);
+		}
 
-	if (device == NULL) {
-		status = DC_STATUS_INVALIDARGS;
-		goto out_invalidargs;
+		device->pending = FALSE;
 	}
 
-	if (!ReadFile (device->hFile, data, size, &dwRead, NULL)) {
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t
+dc_serial_read (dc_iostream_t *abstract, void *data, size_t size, size_t *actual)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
+	DWORD dwRead = 0;
+
+	OVERLAPPED overlapped = {0};
+	overlapped.hEvent = device->hReadWrite;
+
+	if (!ReadFile (device->hFile, data, size, NULL, &overlapped)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		if (errcode != ERROR_IO_PENDING) {
+			SYSERROR (abstract->context, errcode);
+			status = syserror (errcode);
+			goto out;
+		}
+	}
+
+	if (!GetOverlappedResult (device->hFile, &overlapped, &dwRead, TRUE)) {
+		DWORD errcode = GetLastError ();
+		SYSERROR (abstract->context, errcode);
 		status = syserror (errcode);
 		goto out;
 	}
@@ -429,93 +635,64 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 	}
 
 out:
-	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Read", (unsigned char *) data, dwRead);
-
-out_invalidargs:
 	if (actual)
 		*actual = dwRead;
 
 	return status;
 }
 
-dc_status_t
-dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *actual)
+static dc_status_t
+dc_serial_write (dc_iostream_t *abstract, const void *data, size_t size, size_t *actual)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	DWORD dwWritten = 0;
 
-	if (device == NULL) {
-		status = DC_STATUS_INVALIDARGS;
-		goto out_invalidargs;
-	}
+	OVERLAPPED overlapped = {0};
+	overlapped.hEvent = device->hReadWrite;
 
-	LARGE_INTEGER begin, end, freq;
-	if (device->halfduplex) {
-		// Get the current time.
-		if (!QueryPerformanceFrequency(&freq) ||
-			!QueryPerformanceCounter(&begin)) {
-			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+	if (!WriteFile (device->hFile, data, size, NULL, &overlapped)) {
+		DWORD errcode = GetLastError ();
+		if (errcode != ERROR_IO_PENDING) {
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		}
 	}
 
-	if (!WriteFile (device->hFile, data, size, &dwWritten, NULL)) {
+	if (!GetOverlappedResult (device->hFile, &overlapped, &dwWritten, TRUE)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		status = syserror (errcode);
 		goto out;
 	}
 
-	if (device->halfduplex) {
-		// Get the current time.
-		if (!QueryPerformanceCounter(&end))  {
-			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
-			status = syserror (errcode);
-			goto out;
-		}
-
-		// Calculate the elapsed time (microseconds).
-		unsigned long elapsed = 1000000.0 * (end.QuadPart - begin.QuadPart) / freq.QuadPart + 0.5;
-
-		// Calculate the expected duration (microseconds). A 2 millisecond fudge
-		// factor is added because it improves the success rate significantly.
-		unsigned long expected = 1000000.0 * device->nbits / device->baudrate * size + 0.5 + 2000;
-
-		// Wait for the remaining time.
-		if (elapsed < expected) {
-			unsigned long remaining = expected - elapsed;
-
-			// The remaining time is rounded up to the nearest millisecond
-			// because the Windows Sleep() function doesn't have a higher
-			// resolution.
-			dc_serial_sleep (device, (remaining + 999) / 1000);
-		}
-	}
-
 	if (dwWritten != size) {
 		status = DC_STATUS_TIMEOUT;
 	}
 
 out:
-	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Write", (const unsigned char *) data, dwWritten);
-
-out_invalidargs:
 	if (actual)
 		*actual = dwWritten;
 
 	return status;
 }
 
-dc_status_t
-dc_serial_purge (dc_serial_t *device, dc_direction_t direction)
+static dc_status_t
+dc_serial_ioctl (dc_iostream_t *abstract, unsigned int request, void *data, size_t size)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+	switch (request) {
+	case DC_IOCTL_SERIAL_SET_LATENCY:
+		return DC_STATUS_SUCCESS;
+	default:
+		return DC_STATUS_UNSUPPORTED;
+	}
+}
 
-	INFO (device->context, "Purge: direction=%u", direction);
+static dc_status_t
+dc_serial_purge (dc_iostream_t *abstract, dc_direction_t direction)
+{
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	DWORD flags = 0;
 
@@ -535,48 +712,42 @@ dc_serial_purge (dc_serial_t *device, dc_direction_t direction)
 
 	if (!PurgeComm (device->hFile, flags)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_flush (dc_serial_t *device)
+static dc_status_t
+dc_serial_flush (dc_iostream_t *abstract)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Flush: none");
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	if (!FlushFileBuffers (device->hFile)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_break (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_break (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Break: value=%i", level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	if (level) {
 		if (!SetCommBreak (device->hFile)) {
 			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 	} else {
 		if (!ClearCommBreak (device->hFile)) {
 			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 	}
@@ -584,55 +755,48 @@ dc_serial_set_break (dc_serial_t *device, unsigned int level)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_dtr (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_dtr (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "DTR: value=%i", level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	int status = (level ? SETDTR : CLRDTR);
 
 	if (!EscapeCommFunction (device->hFile, status)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_rts (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_rts (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "RTS: value=%i", level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	int status = (level ? SETRTS : CLRRTS);
 
 	if (!EscapeCommFunction (device->hFile, status)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_get_available (dc_serial_t *device, size_t *value)
+static dc_status_t
+dc_serial_get_available (dc_iostream_t *abstract, size_t *value)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	COMSTAT stats;
 
 	if (!ClearCommError (device->hFile, NULL, &stats)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -642,18 +806,16 @@ dc_serial_get_available (dc_serial_t *device, size_t *value)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_get_lines (dc_serial_t *device, unsigned int *value)
+static dc_status_t
+dc_serial_get_lines (dc_iostream_t *abstract, unsigned int *value)
 {
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	unsigned int lines = 0;
 
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
 	DWORD stats = 0;
 	if (!GetCommModemStatus (device->hFile, &stats)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -672,15 +834,14 @@ dc_serial_get_lines (dc_serial_t *device, unsigned int *value)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_sleep (dc_serial_t *device, unsigned int timeout)
+static dc_status_t
+dc_serial_sleep (dc_iostream_t *abstract, unsigned int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Sleep: value=%u", timeout);
-
-	Sleep (timeout);
+	if (dc_platform_sleep (timeout) != 0) {
+		DWORD errcode = GetLastError ();
+		SYSERROR (abstract->context, errcode);
+		return syserror (errcode);
+	}
 
 	return DC_STATUS_SUCCESS;
 }
