@@ -26,6 +26,7 @@
 #include "shearwater_common.h"
 #include "context-private.h"
 #include "device-private.h"
+#include "platform.h"
 #include "array.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &shearwater_petrel_device_vtable)
@@ -33,7 +34,6 @@
 #define MANIFEST_ADDR 0xE0000000
 #define MANIFEST_SIZE 0x600
 
-#define DIVE_ADDR     0xC0000000
 #define DIVE_SIZE     0xFFFFFF
 
 #define RECORD_SIZE   0x20
@@ -77,7 +77,7 @@ str2num (unsigned char data[], unsigned int size, unsigned int offset)
 
 
 dc_status_t
-shearwater_petrel_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+shearwater_petrel_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	shearwater_petrel_device_t *device = NULL;
@@ -95,8 +95,8 @@ shearwater_petrel_device_open (dc_device_t **out, dc_context_t *context, const c
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	// Open the device.
-	status = shearwater_common_open (&device->base, context, name);
+	// Setup the device.
+	status = shearwater_common_setup (&device->base, context, iostream);
 	if (status != DC_STATUS_SUCCESS) {
 		goto error_free;
 	}
@@ -125,12 +125,6 @@ shearwater_petrel_device_close (dc_device_t *abstract)
 		dc_status_set_error(&status, rc);
 	}
 
-	// Close the device.
-	rc = shearwater_common_close (device);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
 	return status;
 }
 
@@ -218,21 +212,54 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 	unsigned int hardware = array_uint_be (dc_buffer_get_data (buffer), dc_buffer_get_size (buffer));
 	unsigned int model = 0;
 	switch (hardware) {
-	case 0x0808: // Petrel 2
-	case 0x0909: // Petrel 1
-	case 0x0B0B: // Petrel 1 (newer hardware)
+	case 0x0101:
+	case 0x0202:
+		model = PREDATOR;
+		break;
+	case 0x0404:
+	case 0x0909:
 		model = PETREL;
 		break;
-	case 0x0A0A: // Nerd 1
-	case 0x0E0D: // Nerd 2
+	case 0x0505:
+	case 0x0808:
+	case 0x0838:
+	case 0x08A5:
+	case 0x0B0B:
+	case 0x7828:
+	case 0x7B2C:
+	case 0x8838:
+		model = PETREL2;
+		break;
+	case 0xB407:
+		model = PETREL3;
+		break;
+	case 0x0606:
+	case 0x0A0A:
 		model = NERD;
 		break;
+	case 0x0E0D:
+	case 0x7E2D:
+		model = NERD2;
+		break;
 	case 0x0707:
 		model = PERDIX;
 		break;
 	case 0x0C0D:
+	case 0x7C2D:
+	case 0x8D6C:
 		model = PERDIXAI;
 		break;
+	case 0xC407:
+		model = PERDIX2;
+		break;
+	case 0x0F0F:
+	case 0x1F0A:
+	case 0x1F0F:
+		model = TERIC;
+		break;
+	case 0x1512:
+		model = PEREGRINE;
+		break;
 	default:
 		WARNING (abstract->context, "Unknown hardware type %04x.", hardware);
 	}
@@ -244,6 +271,41 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 	devinfo.serial = array_uint32_be (serial);
 	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
+	// Read the logbook type
+	rc = shearwater_common_identifier (&device->base, buffer, ID_LOGUPLOAD);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the logbook type.");
+		dc_buffer_free (buffer);
+		dc_buffer_free (manifests);
+		return rc;
+	}
+
+	if (dc_buffer_get_size (buffer) != 9) {
+		ERROR (abstract->context, "Unexpected packet size (" DC_PRINTF_SIZE " bytes).", dc_buffer_get_size(buffer));
+		dc_buffer_free (buffer);
+		dc_buffer_free (manifests);
+		return DC_STATUS_DATAFORMAT;
+	}
+
+	unsigned int base_addr = array_uint32_be (dc_buffer_get_data (buffer) + 1);
+	switch (base_addr) {
+	case 0xDD000000: // Predator - we shouldn't get here, we could give up or we can try 0xC0000000
+	case 0xC0000000: // Predator-Like Format (what we used to call the Petrel format)
+	case 0x90000000: // some firmware versions supported an earlier version of PNF without final record
+		// use the Predator-Like Format instead
+		base_addr = 0xC0000000;
+		break;
+	case 0x80000000: // new Petrel Native Format with final record
+		// that's the correct address
+		break;
+	default: // unknown format
+		ERROR (abstract->context, "Unknown logbook format %08x", base_addr);
+		dc_buffer_free (buffer);
+		dc_buffer_free (manifests);
+		return DC_STATUS_DATAFORMAT;
+	}
+
+	// Read the manifest pages
 	while (1) {
 		// Update the progress state.
 		// Assume the worst case scenario of a full manifest, and adjust the
@@ -267,11 +329,17 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 		unsigned int size = dc_buffer_get_size (buffer);
 
 		// Process the records in the manifest.
-		unsigned int count = 0;
+		unsigned int count = 0, deleted = 0;
 		unsigned int offset = 0;
 		while (offset < size) {
 			// Check for a valid dive header.
 			unsigned int header = array_uint16_be (data + offset);
+			if (header == 0x5A23) {
+				// this is a deleted dive; keep looking
+				offset += RECORD_SIZE;
+				deleted++;
+				continue;
+			}
 			if (header != 0xA5C4)
 				break;
 
@@ -285,7 +353,7 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 
 		// Update the progress state.
 		current += 1;
-		maximum -= RECORD_COUNT - count;
+		maximum -= RECORD_COUNT - count - deleted;
 
 		// Append the manifest records to the main buffer.
 		if (!dc_buffer_append (manifests, data, count * RECORD_SIZE)) {
@@ -296,7 +364,7 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 		}
 
 		// Stop downloading manifest if there are no more records.
-		if (count != RECORD_COUNT)
+		if (count + deleted != RECORD_COUNT)
 			break;
 	}
 
@@ -311,13 +379,18 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 
 	unsigned int offset = 0;
 	while (offset < size) {
+		// skip deleted dives
+		if (array_uint16_be(data + offset) == 0x5A23) {
+			offset += RECORD_SIZE;
+			continue;
+		}
 		// Get the address of the dive.
 		unsigned int address = array_uint32_be (data + offset + 20);
 
 		// Download the dive.
 		progress.current = NSTEPS * current;
 		progress.maximum = NSTEPS * maximum;
-		rc = shearwater_common_download (&device->base, buffer, DIVE_ADDR + address, DIVE_SIZE, 1, &progress);
+		rc = shearwater_common_download (&device->base, buffer, base_addr + address, DIVE_SIZE, 1, &progress);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to download the dive.");
 			dc_buffer_free (buffer);
