@@ -23,6 +23,7 @@
 #define SUUNTO_D9_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 #include <libdivecomputer/suunto_d9.h>
@@ -32,7 +33,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-suunto_d9_device_open (dc_device_t **device, dc_context_t *context, const char *name, unsigned int model);
+suunto_d9_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream, unsigned int model);
 
 dc_status_t
 suunto_d9_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
