@@ -32,21 +32,22 @@
 #define MAXPARAMS 3
 #define NGASMIXES 11
 
-#define D9       0x0E
-#define D6       0x0F
-#define VYPER2   0x10
-#define COBRA2   0x11
-#define D4       0x12
-#define VYPERAIR 0x13
-#define COBRA3   0x14
-#define HELO2    0x15
-#define D4i      0x19
-#define D6i      0x1A
-#define D9tx     0x1B
-#define DX       0x1C
-#define VYPERNOVO 0x1D
-#define ZOOPNOVO  0x1E
-#define D4F       0x20
+#define D9          0x0E
+#define D6          0x0F
+#define VYPER2      0x10
+#define COBRA2      0x11
+#define D4          0x12
+#define VYPERAIR    0x13
+#define COBRA3      0x14
+#define HELO2       0x15
+#define D4i         0x19
+#define D6i         0x1A
+#define D9tx        0x1B
+#define DX          0x1C
+#define VYPERNOVO   0x1D
+#define ZOOPNOVO_A  0x1E
+#define ZOOPNOVO_B  0x1F
+#define D4F         0x20
 
 #define ID_D6I_V1_MIX2 0x1871C062
 #define ID_D6I_V1_MIX3 0x1871C063
@@ -74,8 +75,10 @@ struct suunto_d9_parser_t {
 	unsigned int model;
 	// Cached fields.
 	unsigned int cached;
+	unsigned int id;
 	unsigned int mode;
 	unsigned int ngasmixes;
+	unsigned int nccr;
 	unsigned int oxygen[NGASMIXES];
 	unsigned int helium[NGASMIXES];
 	unsigned int gasmix;
@@ -98,6 +101,9 @@ static const dc_parser_vtable_t suunto_d9_parser_vtable = {
 	sizeof(suunto_d9_parser_t),
 	DC_FAMILY_SUUNTO_D9,
 	suunto_d9_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	suunto_d9_parser_get_datetime, /* datetime */
 	suunto_d9_parser_get_field, /* fields */
 	suunto_d9_parser_samples_foreach, /* samples_foreach */
@@ -108,7 +114,7 @@ static unsigned int
 suunto_d9_parser_find_gasmix (suunto_d9_parser_t *parser, unsigned int o2, unsigned int he)
 {
 	// Find the gasmix in the list.
-	unsigned int i = 0;
+	unsigned int i = parser->nccr;
 	while (i < parser->ngasmixes) {
 		if (o2 == parser->oxygen[i] && he == parser->helium[i])
 			break;
@@ -135,12 +141,13 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 	unsigned int gasmode_offset = 0x19;
 	unsigned int gasmix_offset = 0x21;
 	unsigned int gasmix_count = 3;
+	unsigned int ccr_count = 0;
 	if (parser->model == HELO2) {
 		gasmode_offset = 0x1F;
 		gasmix_offset = 0x54;
 		gasmix_count = 8;
-	} else if (parser->model == D4i || parser->model == ZOOPNOVO ||
-		parser->model == D4F) {
+	} else if (parser->model == D4i || parser->model == ZOOPNOVO_A ||
+		parser->model == ZOOPNOVO_B || parser->model == D4F) {
 		gasmode_offset = 0x1D;
 		if (id == ID_D4I_V2)
 			gasmix_offset = 0x67;
@@ -168,6 +175,7 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		else
 			gasmix_offset = 0xC1;
 		gasmix_count = 11;
+		ccr_count = 3;
 	}
 
 	// Offset to the configuration data.
@@ -176,8 +184,9 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		config += 1;
 	} else if (parser->model == HELO2 || parser->model == D4i ||
 		parser->model == D6i || parser->model == D9tx ||
-		parser->model == DX || parser->model == ZOOPNOVO ||
-		parser->model == VYPERNOVO || parser->model == D4F) {
+		parser->model == DX || parser->model == ZOOPNOVO_A ||
+		parser->model == ZOOPNOVO_B || parser->model == VYPERNOVO ||
+		parser->model == D4F) {
 		config = gasmix_offset + gasmix_count * 6;
 	}
 	if (config + 1 > size)
@@ -188,17 +197,21 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 	parser->gasmix = 0;
 	if (parser->mode == GAUGE || parser->mode == FREEDIVE) {
 		parser->ngasmixes = 0;
+		parser->nccr = 0;
 	} else if (parser->mode == AIR) {
 		parser->oxygen[0] = 21;
 		parser->helium[0] = 0;
 		parser->ngasmixes = 1;
+		parser->nccr = 0;
 	} else {
 		parser->ngasmixes = 0;
+		parser->nccr = ccr_count;
 		for (unsigned int i = 0; i < gasmix_count; ++i) {
 			if (parser->model == HELO2 || parser->model == D4i ||
 				parser->model == D6i || parser->model == D9tx ||
-				parser->model == DX || parser->model == ZOOPNOVO ||
-				parser->model == VYPERNOVO || parser->model == D4F) {
+				parser->model == DX || parser->model == ZOOPNOVO_A ||
+				parser->model == ZOOPNOVO_B || parser->model == VYPERNOVO ||
+				parser->model == D4F) {
 				parser->oxygen[i] = data[gasmix_offset + 6 * i + 1];
 				parser->helium[i] = data[gasmix_offset + 6 * i + 2];
 			} else {
@@ -215,16 +228,23 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 		if (parser->model == HELO2) {
 			parser->gasmix = data[0x26];
 		} else if (parser->model == D4i || parser->model == D6i ||
-			parser->model == D9tx || parser->model == ZOOPNOVO ||
-			parser->model == VYPERNOVO || parser->model == D4F) {
+			parser->model == D9tx || parser->model == ZOOPNOVO_A ||
+			parser->model == ZOOPNOVO_B || parser->model == VYPERNOVO ||
+			parser->model == D4F) {
 			if (id == ID_D4I_V2 || id == ID_D6I_V2) {
 				parser->gasmix = data[0x2D];
 			} else {
 				parser->gasmix = data[0x28];
 			}
+		} else if (parser->model == DX) {
+			parser->gasmix = data[0x31] & 0x7F;
+			if ((data[0x31] & 0x80) == 0) {
+				parser->gasmix += parser->nccr;
+			}
 		}
 	}
 	parser->config = config;
+	parser->id = id;
 	parser->cached = 1;
 
 	return DC_STATUS_SUCCESS;
@@ -248,8 +268,10 @@ suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 	// Set the default values.
 	parser->model = model;
 	parser->cached = 0;
+	parser->id = 0;
 	parser->mode = AIR;
 	parser->ngasmixes = 0;
+	parser->nccr = 0;
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->oxygen[i] = 0;
 		parser->helium[i] = 0;
@@ -270,8 +292,10 @@ suunto_d9_parser_set_data (dc_parser_t *abstract, const unsigned char *data, uns
 
 	// Reset the cache.
 	parser->cached = 0;
+	parser->id = 0;
 	parser->mode = AIR;
 	parser->ngasmixes = 0;
+	parser->nccr = 0;
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->oxygen[i] = 0;
 		parser->helium[i] = 0;
@@ -292,8 +316,9 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 	if (parser->model == HELO2 || parser->model == DX)
 		offset = 0x17;
 	else if (parser->model == D4i || parser->model == D6i ||
-		parser->model == D9tx || parser->model == ZOOPNOVO ||
-		parser->model == VYPERNOVO || parser->model == D4F)
+		parser->model == D9tx || parser->model == ZOOPNOVO_A ||
+		parser->model == ZOOPNOVO_B || parser->model == VYPERNOVO ||
+		parser->model == D4F)
 		offset = 0x13;
 
 	if (abstract->size < offset + 7)
@@ -304,8 +329,8 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 	if (datetime) {
 		if (parser->model == D4i || parser->model == D6i ||
 			parser->model == D9tx || parser->model == DX ||
-			parser->model == ZOOPNOVO || parser->model == VYPERNOVO ||
-			parser->model == D4F) {
+			parser->model == ZOOPNOVO_A || parser->model == ZOOPNOVO_B ||
+			parser->model == VYPERNOVO || parser->model == D4F) {
 			datetime->year   = p[0] + (p[1] << 8);
 			datetime->month  = p[2];
 			datetime->day    = p[3];
@@ -331,9 +356,7 @@ static dc_status_t
 suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
 {
 	suunto_d9_parser_t *parser = (suunto_d9_parser_t*) abstract;
-
 	const unsigned char *data = abstract->data;
-	unsigned int size = abstract->size;
 
 	// Cache the gas mix data.
 	dc_status_t rc = suunto_d9_parser_cache (parser);
@@ -341,6 +364,7 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 		return rc;
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
+	dc_decomodel_t *decomodel = (dc_decomodel_t *) value;
 
 	if (value) {
 		switch (type) {
@@ -349,8 +373,8 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 				*((unsigned int *) value) = array_uint16_le (data + 0x0B);
 			else if (parser->model == D4i || parser->model == D6i ||
 				parser->model == D9tx || parser->model == DX ||
-				parser->model == ZOOPNOVO || parser->model == VYPERNOVO ||
-				parser->model == D4F)
+				parser->model == ZOOPNOVO_A || parser->model == ZOOPNOVO_B ||
+				parser->model == VYPERNOVO || parser->model == D4F)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0D);
 			else if (parser->model == HELO2)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0D) * 60;
@@ -382,12 +406,26 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 				*((dc_divemode_t *) value) = DC_DIVEMODE_FREEDIVE;
 				break;
 			case CCR:
-				*((dc_divemode_t *) value) = DC_DIVEMODE_CC;
+				*((dc_divemode_t *) value) = DC_DIVEMODE_CCR;
 				break;
 			default:
 				return DC_STATUS_DATAFORMAT;
 			}
 			break;
+		case DC_FIELD_DECOMODEL:
+			decomodel->type = DC_DECOMODEL_RGBM;
+			if (parser->model == D4i ||parser->model == D6i ||
+				parser->model == D9tx || parser->model == ZOOPNOVO_A ||
+				parser->model == ZOOPNOVO_B || parser->model == VYPERNOVO ||
+				parser->model == D4F)
+				decomodel->conservatism = data[0x21] - 2;
+			else if (parser->model == HELO2)
+				decomodel->conservatism = data[0x23] - 2;
+			else if (parser->model == DX)
+				decomodel->conservatism = data[0x25] - 2;
+			else
+				decomodel->conservatism = data[0x1E];
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -461,8 +499,8 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 	unsigned int interval_sample_offset = 0x18;
 	if (parser->model == HELO2 || parser->model == D4i ||
 		parser->model == D6i || parser->model == D9tx ||
-		parser->model == ZOOPNOVO || parser->model == VYPERNOVO ||
-		parser->model == D4F)
+		parser->model == ZOOPNOVO_A || parser->model == ZOOPNOVO_B ||
+		parser->model == VYPERNOVO || parser->model == D4F)
 		interval_sample_offset = 0x1E;
 	else if (parser->model == DX)
 		interval_sample_offset = 0x22;
@@ -536,9 +574,9 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 		if ((nsamples + 1) == marker) {
 			while (offset < size) {
 				unsigned int event = data[offset++];
-				unsigned int seconds, type, unknown, heading;
+				unsigned int seconds, type, state, number, heading;
 				unsigned int current, next;
-				unsigned int he, o2, idx;
+				unsigned int he, o2, ppo2, idx;
 				unsigned int length;
 
 				sample.event.type = SAMPLE_EVENT_NONE;
@@ -565,7 +603,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
 					}
-					unknown = data[offset + 0];
+					state   = data[offset + 0];
 					seconds = data[offset + 1];
 					sample.event.type = SAMPLE_EVENT_SURFACE;
 					sample.event.time = seconds;
@@ -684,7 +722,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
 					}
-					unknown = data[offset + 0];
+					number  = data[offset + 0];
 					seconds = data[offset + 1];
 					heading = array_uint16_le (data + offset + 2);
 					if (heading == 0xFFFF) {
@@ -715,29 +753,41 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					offset += 2;
 					break;
 				case 0x06: // Gas Change
-					if (parser->model == DX || parser->model == VYPERNOVO)
+					if (parser->model == DX || parser->model == VYPERNOVO ||
+						(parser->model == D6i && parser->id == ID_D6I_V2)) {
 						length = 5;
-					else
+					} else {
 						length = 4;
+					}
 					if (offset + length > size) {
 						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
 					}
-					unknown = data[offset + 0];
+					type = data[offset + 0];
 					he = data[offset + 1];
 					o2 = data[offset + 2];
-					if (parser->model == DX || parser->model == VYPERNOVO) {
+					if (parser->model == DX || parser->model == VYPERNOVO ||
+						(parser->model == D6i && parser->id == ID_D6I_V2)) {
+						ppo2 = data[offset + 3];
 						seconds = data[offset + 4];
 					} else {
+						ppo2 = 0;
 						seconds = data[offset + 3];
 					}
-					idx = suunto_d9_parser_find_gasmix(parser, o2, he);
-					if (idx >= parser->ngasmixes) {
+					idx = type & 0x0F;
+					if ((type & 0x80) == 0) {
+						idx += parser->nccr;
+					}
+					if (idx >= parser->ngasmixes || o2 != parser->oxygen[idx] || he != parser->helium[idx]) {
 						ERROR (abstract->context, "Invalid gas mix.");
 						return DC_STATUS_DATAFORMAT;
 					}
 					sample.gasmix = idx;
 					if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+					if (type & 0x80) {
+						sample.setpoint = ppo2 / 10.0;
+						if (callback) callback (DC_SAMPLE_SETPOINT, sample, userdata);
+					}
 					offset += length;
 					break;
 				default:
