@@ -27,15 +27,16 @@
 #include "context-private.h"
 #include "device-private.h"
 #include "array.h"
-#include "usbhid.h"
 #include "platform.h"
+#include "checksum.h"
+#include "hdlc.h"
 
 #define EONSTEEL 0
 #define EONCORE  1
 
 typedef struct suunto_eonsteel_device_t {
 	dc_device_t base;
-	dc_usbhid_t *usbhid;
+	dc_iostream_t *iostream;
 	unsigned int model;
 	unsigned int magic;
 	unsigned short seq;
@@ -75,10 +76,15 @@ struct directory_entry {
 #define CMD_SET_DATE	0x0203
 #define CMD_GET_DATE	0x0303
 
+#define PACKET_SIZE 64
+#define HEADER_SIZE 12
+#define MAXDATA_SIZE 2048
+#define CRC_SIZE    4
+
 static dc_status_t suunto_eonsteel_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t suunto_eonsteel_device_timesync(dc_device_t *abstract, const dc_datetime_t *datetime);
-static dc_status_t suunto_eonsteel_device_close(dc_device_t *abstract);
+static dc_status_t suunto_eonsteel_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t suunto_eonsteel_device_vtable = {
 	sizeof(suunto_eonsteel_device_t),
@@ -94,6 +100,15 @@ static const dc_device_vtable_t suunto_eonsteel_device_vtable = {
 
 static const char dive_directory[] = "0:/dives";
 
+static void file_list_free (struct directory_entry *de)
+{
+	while (de) {
+		struct directory_entry *next = de->next;
+		free (de);
+		de = next;
+	}
+}
+
 static struct directory_entry *alloc_dirent(int type, int len, const char *name)
 {
 	struct directory_entry *res;
@@ -109,20 +124,6 @@ static struct directory_entry *alloc_dirent(int type, int len, const char *name)
 	return res;
 }
 
-static void put_le16(unsigned short val, unsigned char *p)
-{
-	p[0] = val;
-	p[1] = val >> 8;
-}
-
-static void put_le32(unsigned int val, unsigned char *p)
-{
-	p[0] = val;
-	p[1] = val >> 8;
-	p[2] = val >> 16;
-	p[3] = val >> 24;
-}
-
 /*
  * Get a single 64-byte packet from the dive computer. This handles packet
  * logging and any obvious packet-level errors, and returns the payload of
@@ -133,146 +134,146 @@ static void put_le32(unsigned int val, unsigned char *p)
  *
  * The maximum payload is 62 bytes.
  */
-#define PACKET_SIZE 64
-static int receive_packet(suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
+static dc_status_t
+suunto_eonsteel_receive_usb(suunto_eonsteel_device_t *device, unsigned char data[], unsigned int size, unsigned int *actual)
 {
-	unsigned char buf[64];
 	dc_status_t rc = DC_STATUS_SUCCESS;
+	unsigned char buf[PACKET_SIZE];
 	size_t transferred = 0;
-	int len;
+	unsigned int len = 0;
 
-	rc = dc_usbhid_read(eon->usbhid, buf, PACKET_SIZE, &transferred);
+	rc = dc_iostream_read(device->iostream, buf, sizeof(buf), &transferred);
 	if (rc != DC_STATUS_SUCCESS) {
-		ERROR(eon->base.context, "read interrupt transfer failed");
-		return -1;
+		ERROR(device->base.context, "Failed to receive the packet.");
+		return rc;
 	}
-	if (transferred != PACKET_SIZE) {
-		ERROR(eon->base.context, "incomplete read interrupt transfer (got " DC_PRINTF_SIZE ", expected %d)", transferred, PACKET_SIZE);
-		return -1;
+
+	if (transferred < 2) {
+		ERROR(device->base.context, "Invalid packet length (" DC_PRINTF_SIZE ").", transferred);
+		return DC_STATUS_PROTOCOL;
 	}
+
 	if (buf[0] != 0x3f) {
-		ERROR(eon->base.context, "read interrupt transfer returns wrong report type (%d)", buf[0]);
-		return -1;
+		ERROR(device->base.context, "Invalid report type (%02x).", buf[0]);
+		return DC_STATUS_PROTOCOL;
 	}
+
 	len = buf[1];
-	if (len > PACKET_SIZE-2) {
-		ERROR(eon->base.context, "read interrupt transfer reports bad length (%d)", len);
-		return -1;
+	if (len + 2 > transferred) {
+		ERROR(device->base.context, "Invalid payload length (%u).", len);
+		return DC_STATUS_PROTOCOL;
 	}
 	if (len > size) {
-		ERROR(eon->base.context, "receive_packet result buffer too small - truncating");
-		len = size;
+		ERROR(device->base.context, "Insufficient buffer space available.");
+		return DC_STATUS_PROTOCOL;
 	}
-	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "rcv", buf+2, len);
-	memcpy(buffer, buf+2, len);
-	return len;
+
+	HEXDUMP (device->base.context, DC_LOGLEVEL_DEBUG, "rcv", buf + 2, len);
+
+	memcpy(data, buf + 2, len);
+
+	if (actual)
+		*actual = len;
+
+	return DC_STATUS_SUCCESS;
 }
 
-static int send_cmd(suunto_eonsteel_device_t *eon,
-	unsigned short cmd,
-	unsigned int len,
-	const unsigned char *buffer)
+static dc_status_t
+suunto_eonsteel_receive_ble(suunto_eonsteel_device_t *device, unsigned char data[], unsigned int size, unsigned int *actual)
 {
-	unsigned char buf[64];
-	unsigned short seq = eon->seq;
-	unsigned int magic = eon->magic;
 	dc_status_t rc = DC_STATUS_SUCCESS;
+	unsigned char buffer[HEADER_SIZE + MAXDATA_SIZE + CRC_SIZE];
 	size_t transferred = 0;
 
+	rc = dc_iostream_read(device->iostream, buffer, sizeof(buffer), &transferred);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR(device->base.context, "Failed to receive the packet.");
+		return rc;
+	}
+
+	if (transferred < CRC_SIZE) {
+		ERROR(device->base.context, "Invalid packet length (" DC_PRINTF_SIZE ").", transferred);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	unsigned int nbytes = transferred - CRC_SIZE;
+
+	unsigned int crc = array_uint32_le(buffer + nbytes);
+	unsigned int ccrc = checksum_crc32r(buffer, nbytes);
+	if (crc != ccrc) {
+		ERROR(device->base.context, "Invalid checksum (expected %08x, received %08x).", ccrc, crc);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	if (nbytes > size) {
+		ERROR(device->base.context, "Insufficient buffer space available.");
+		return DC_STATUS_PROTOCOL;
+	}
+
+	memcpy(data, buffer, nbytes);
+
+	HEXDUMP (device->base.context, DC_LOGLEVEL_DEBUG, "rcv", buffer, nbytes);
+
+	if (actual)
+		*actual = nbytes;
+
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t
+suunto_eonsteel_send(suunto_eonsteel_device_t *device,
+	unsigned short cmd,
+	const unsigned char data[],
+	unsigned int size)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	unsigned char buf[PACKET_SIZE + CRC_SIZE];
+
 	// Two-byte packet header, followed by 12 bytes of extended header
-	if (len > sizeof(buf)-2-12) {
-		ERROR(eon->base.context, "send command with too much long");
-		return -1;
+	if (size + 2 + HEADER_SIZE + CRC_SIZE > sizeof(buf)) {
+		ERROR(device->base.context, "Insufficient buffer space available.");
+		return DC_STATUS_PROTOCOL;
 	}
 
 	memset(buf, 0, sizeof(buf));
 
 	buf[0] = 0x3f;
-	buf[1] = len + 12;
+	buf[1] = size + HEADER_SIZE;
 
 	// 2-byte LE command word
-	put_le16(cmd, buf+2);
+	array_uint16_le_set(buf + 2, cmd);
 
 	// 4-byte LE magic value (starts at 1)
-	put_le32(magic, buf+4);
+	array_uint32_le_set(buf + 4, device->magic);
 
 	// 2-byte LE sequence number;
-	put_le16(seq, buf+8);
+	array_uint16_le_set(buf + 8, device->seq);
 
 	// 4-byte LE length
-	put_le32(len, buf+10);
+	array_uint32_le_set(buf + 10, size);
 
 	// .. followed by actual data
-	if (len) {
-		memcpy(buf+14, buffer, len);
+	if (size) {
+		memcpy(buf + 14, data, size);
 	}
 
-	rc = dc_usbhid_write(eon->usbhid, buf, sizeof(buf), &transferred);
+	// 4 byte LE checksum
+	unsigned int crc = checksum_crc32r(buf + 2, size + HEADER_SIZE);
+	array_uint32_le_set(buf + 14 + size, crc);
+
+	if (dc_iostream_get_transport(device->iostream) == DC_TRANSPORT_BLE) {
+		rc = dc_iostream_write(device->iostream, buf + 2, size + HEADER_SIZE + CRC_SIZE, NULL);
+	} else {
+		rc = dc_iostream_write(device->iostream, buf, sizeof(buf) - CRC_SIZE, NULL);
+	}
 	if (rc != DC_STATUS_SUCCESS) {
-		ERROR(eon->base.context, "write interrupt transfer failed");
-		return -1;
+		ERROR(device->base.context, "Failed to send the command.");
+		return rc;
 	}
 
-	// dump every outgoing packet?
-	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "cmd", buf+2, len+12);
-	return 0;
-}
+	HEXDUMP (device->base.context, DC_LOGLEVEL_DEBUG, "cmd", buf + 2, size + HEADER_SIZE);
 
-struct eon_hdr {
-	unsigned short cmd;
-	unsigned int magic;
-	unsigned short seq;
-	unsigned int len;
-};
-
-static int receive_header(suunto_eonsteel_device_t *eon, struct eon_hdr *hdr, unsigned char *buffer, int size)
-{
-	int ret;
-	unsigned char header[64];
-
-	ret = receive_packet(eon, header, sizeof(header));
-	if (ret < 0)
-		return -1;
-	if (ret < 12) {
-		ERROR(eon->base.context, "short reply packet (%d)", ret);
-		return -1;
-	}
-
-	/* Unpack the 12-byte header */
-	hdr->cmd = array_uint16_le(header);
-	hdr->magic = array_uint32_le(header+2);
-	hdr->seq = array_uint16_le(header+6);
-	hdr->len = array_uint32_le(header+8);
-
-	ret -= 12;
-	if (ret > size) {
-		ERROR(eon->base.context, "receive_header result data buffer too small (%d vs %d)", ret, size);
-		return -1;
-	}
-	memcpy(buffer, header+12, ret);
-	return ret;
-}
-
-static int receive_data(suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
-{
-	int ret = 0;
-
-	while (size > 0) {
-		int len;
-
-		len = receive_packet(eon, buffer + ret, size);
-		if (len < 0)
-			return -1;
-
-		size -= len;
-		ret += len;
-
-		/* Was it not a full packet of data? We're done, regardless of expectations */
-		if (len < PACKET_SIZE-2)
-			break;
-	}
-
-	return ret;
+	return DC_STATUS_SUCCESS;
 }
 
 /*
@@ -282,96 +283,151 @@ static int receive_data(suunto_eonsteel_device_t *eon, unsigned char *buffer, in
  * against the command, and then only returns the actual reply
  * data itself.
  *
- * Also note that "receive_data()" itself will have removed the
- * per-packet handshake bytes, so unlike "send_cmd()", this does
- * not see the two initial 0x3f 0x?? bytes, and this the offsets
- * for the cmd/magic/seq/len are off by two compared to the
- * send_cmd() side. The offsets are the same in the actual raw
- * packet.
+ * Also note that receive() function itself will have removed the
+ * per-packet handshake bytes, so unlike the send() function, this
+ * functon does not see the two initial 0x3f 0x?? bytes, and thus the
+ * offsets for the cmd/magic/seq/len are off by two compared to the
+ * send() side. The offsets are the same in the actual raw packet.
  */
-static int send_receive(suunto_eonsteel_device_t *eon,
+static dc_status_t
+suunto_eonsteel_transfer(suunto_eonsteel_device_t *device,
 	unsigned short cmd,
-	unsigned int len_out, const unsigned char *out,
-	unsigned int len_in, unsigned char *in)
+	const unsigned char data[], unsigned int size,
+	unsigned char answer[], unsigned int asize,
+	unsigned int *actual)
 {
-	int len, actual, max;
-	unsigned char buf[2048];
-	struct eon_hdr hdr;
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	unsigned char header[HEADER_SIZE + MAXDATA_SIZE];
+	unsigned int len = 0;
 
-	if (send_cmd(eon, cmd, len_out, out) < 0)
-		return -1;
+	// Send the command.
+	rc = suunto_eonsteel_send(device, cmd, data, size);
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
 
-	/* Get the header and the first part of the data */
-	len = receive_header(eon, &hdr, in, len_in);
-	if (len < 0)
-		return -1;
+	if (dc_iostream_get_transport(device->iostream) == DC_TRANSPORT_BLE) {
+		// Receive the entire data packet.
+		rc = suunto_eonsteel_receive_ble(device, header, sizeof(header), &len);
+	} else {
+		// Receive the header and the first part of the data.
+		rc = suunto_eonsteel_receive_usb(device, header, sizeof(header), &len);
+	}
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
 
-	/* Verify the header data */
-	if (hdr.cmd != cmd) {
-		ERROR(eon->base.context, "command reply doesn't match command");
-		return -1;
-	}
-	if (hdr.magic != eon->magic + 5) {
-		ERROR(eon->base.context, "command reply doesn't match magic (got %08x, expected %08x)", hdr.magic, eon->magic + 5);
-		return -1;
-	}
-	if (hdr.seq != eon->seq) {
-		ERROR(eon->base.context, "command reply doesn't match sequence number");
-		return -1;
-	}
-	actual = hdr.len;
-	if (actual < len) {
-		ERROR(eon->base.context, "command reply length mismatch (got %d, claimed %d)", len, actual);
-		return -1;
-	}
-	if (actual > len_in) {
-		ERROR(eon->base.context, "command reply too big for result buffer - truncating");
-		actual = len_in;
+	// Verify the header length.
+	if (len < HEADER_SIZE) {
+		ERROR(device->base.context, "Invalid packet length (%u).", len);
+		return DC_STATUS_PROTOCOL;
 	}
 
-	/* Get the rest of the data */
-	len += receive_data(eon, in + len, actual - len);
-	if (len != actual) {
-		ERROR(eon->base.context, "command reply returned unexpected amoutn of data (got %d, expected %d)", len, actual);
-		return -1;
+	// Unpack the 12 byte header.
+	unsigned int reply = array_uint16_le(header);
+	unsigned int magic = array_uint32_le(header + 2);
+	unsigned int seq = array_uint16_le(header + 6);
+	unsigned int length = array_uint32_le(header + 8);
+
+	if (cmd != CMD_INIT) {
+		// Verify the command reply.
+		if (reply != cmd) {
+			ERROR(device->base.context, "Unexpected command reply (received %04x, expected %04x).", reply, cmd);
+			return DC_STATUS_PROTOCOL;
+		}
+
+		// Verify the magic value.
+		if (magic != device->magic + 5) {
+			ERROR(device->base.context, "Unexpected magic value (received %08x, expected %08x).", magic, device->magic + 5);
+			return DC_STATUS_PROTOCOL;
+		}
 	}
 
-	// Successful command - increment sequence number
-	eon->seq++;
-	return len;
+	// Verify the sequence number.
+	if (seq != device->seq) {
+		ERROR(device->base.context, "Unexpected sequence number (received %04x, expected %04x).", seq, device->seq);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	// Verify the length.
+	if (length > asize) {
+		ERROR(device->base.context, "Insufficient buffer space available.");
+		return DC_STATUS_PROTOCOL;
+	}
+
+	// Verify the initial payload length.
+	unsigned int nbytes = len - HEADER_SIZE;
+	if (nbytes > length) {
+		ERROR(device->base.context, "Unexpected number of bytes (received %u, expected %u).", nbytes, length);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	// Copy the payload data.
+	memcpy(answer, header + HEADER_SIZE, nbytes);
+
+	// Receive the remainder of the data.
+	if (dc_iostream_get_transport(device->iostream) != DC_TRANSPORT_BLE) {
+		while (nbytes < length) {
+			rc = suunto_eonsteel_receive_usb(device, answer + nbytes, length - nbytes, &len);
+			if (rc != DC_STATUS_SUCCESS)
+				return rc;
+
+			nbytes += len;
+
+			if (len < PACKET_SIZE - 2)
+				break;
+		}
+	}
+
+	// Verify the total payload length.
+	if (nbytes != length) {
+		ERROR(device->base.context, "Unexpected number of bytes (received %u, expected %u).", nbytes, length);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	// Remember the magic number.
+	if (cmd == CMD_INIT) {
+		device->magic = (magic & 0xffff0000) | 0x0005;
+	}
+
+	// Increment the sequence number.
+	device->seq++;
+
+	if (actual)
+		*actual = nbytes;
+
+	return DC_STATUS_SUCCESS;
 }
 
-static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buffer_t *buf)
+static dc_status_t
+read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buffer_t *buf)
 {
+	dc_status_t rc = DC_STATUS_SUCCESS;
 	unsigned char result[2560];
 	unsigned char cmdbuf[64];
-	unsigned int size, offset;
-	int rc, len;
+	unsigned int size, offset, len;
+	unsigned int n = 0;
 
 	memset(cmdbuf, 0, sizeof(cmdbuf));
 	len = strlen(filename) + 1;
 	if (len + 4 > sizeof(cmdbuf)) {
 		ERROR(eon->base.context, "too long filename: %s", filename);
-		return -1;
+		return DC_STATUS_PROTOCOL;
 	}
 	memcpy(cmdbuf+4, filename, len);
-	rc = send_receive(eon, CMD_FILE_OPEN,
-		len+4, cmdbuf,
-		sizeof(result), result);
-	if (rc < 0) {
+	rc = suunto_eonsteel_transfer(eon, CMD_FILE_OPEN,
+		cmdbuf, len + 4, result, sizeof(result), &n);
+	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "unable to look up %s", filename);
-		return -1;
+		return rc;
 	}
-	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "lookup", result, rc);
+	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "lookup", result, n);
 
-	rc = send_receive(eon, CMD_FILE_STAT,
-		0, NULL,
-		sizeof(result), result);
-	if (rc < 0) {
+	rc = suunto_eonsteel_transfer(eon, CMD_FILE_STAT,
+		NULL, 0, result, sizeof(result), &n);
+	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "unable to stat %s", filename);
-		return -1;
+		return rc;
 	}
-	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "stat", result, rc);
+	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "stat", result, n);
 
 	size = array_uint32_le(result+4);
 	offset = 0;
@@ -382,53 +438,77 @@ static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buf
 		ask = size;
 		if (ask > 1024)
 			ask = 1024;
-		put_le32(1234, cmdbuf+0);	// Not file offset, after all
-		put_le32(ask, cmdbuf+4);	// Size of read
-		rc = send_receive(eon, CMD_FILE_READ,
-			8, cmdbuf,
-			sizeof(result), result);
-		if (rc < 0) {
+		array_uint32_le_set(cmdbuf + 0, 1234);	// Not file offset, after all
+		array_uint32_le_set(cmdbuf + 4, ask);	// Size of read
+		rc = suunto_eonsteel_transfer(eon, CMD_FILE_READ,
+			cmdbuf, 8, result, sizeof(result), &n);
+		if (rc != DC_STATUS_SUCCESS) {
 			ERROR(eon->base.context, "unable to read %s", filename);
-			return -1;
+			return rc;
 		}
-		if (rc < 8) {
+		if (n < 8) {
 			ERROR(eon->base.context, "got short read reply for %s", filename);
-			return -1;
+			return DC_STATUS_PROTOCOL;
 		}
 
 		// Not file offset, just stays unmodified.
 		at = array_uint32_le(result);
 		if (at != 1234) {
 			ERROR(eon->base.context, "read of %s returned different offset than asked for (%d vs %d)", filename, at, offset);
-			return -1;
+			return DC_STATUS_PROTOCOL;
 		}
 
 		// Number of bytes actually read
 		got = array_uint32_le(result+4);
 		if (!got)
 			break;
-		if (rc < 8 + got) {
+		if (n < 8 + got) {
 			ERROR(eon->base.context, "odd read size reply for offset %d of file %s", offset, filename);
-			return -1;
+			return DC_STATUS_PROTOCOL;
 		}
 
 		if (got > size)
 			got = size;
-		dc_buffer_append(buf, result+8, got);
+		if (!dc_buffer_append (buf, result + 8, got)) {
+			ERROR (eon->base.context, "Insufficient buffer space available.");
+			return DC_STATUS_NOMEMORY;
+		}
 		offset += got;
 		size -= got;
 	}
 
-	rc = send_receive(eon, CMD_FILE_CLOSE,
-		0, NULL,
-		sizeof(result), result);
-	if (rc < 0) {
+	rc = suunto_eonsteel_transfer(eon, CMD_FILE_CLOSE,
+		NULL, 0, result, sizeof(result), &n);
+	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "cmd CMD_FILE_CLOSE failed");
-		return -1;
+		return rc;
 	}
-	HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "close", result, rc);
+	HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "close", result, n);
 
-	return offset;
+	return DC_STATUS_SUCCESS;
+}
+
+/*
+ * Insert a directory entry in the sorted list, most recent entry
+ * first.
+ *
+ * The directory entry names are the timestamps as hex, so ordering
+ * in alphabetical order ends up also ordering in date order!
+ */
+static struct directory_entry *insert_dirent(struct directory_entry *entry, struct directory_entry *list)
+{
+	struct directory_entry **pos = &list, *next;
+
+	while ((next = *pos) != NULL) {
+		/* Is this bigger (more recent) than the next entry? We're good! */
+		if (strcmp(entry->name, next->name) > 0)
+			break;
+		pos = &next->next;
+	}
+	entry->next = next;
+	*pos = entry;
+
+	return list;
 }
 
 /*
@@ -436,7 +516,7 @@ static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buf
  * with the last dirent first. That's intentional: for dives,
  * we will want to look up the last dive first.
  */
-static struct directory_entry *parse_dirent(suunto_eonsteel_device_t *eon, int nr, const unsigned char *p, int len, struct directory_entry *old)
+static struct directory_entry *parse_dirent(suunto_eonsteel_device_t *eon, int nr, const unsigned char *p, unsigned int len, struct directory_entry *list)
 {
 	while (len > 8) {
 		unsigned int type = array_uint32_le(p);
@@ -457,108 +537,88 @@ static struct directory_entry *parse_dirent(suunto_eonsteel_device_t *eon, int n
 			ERROR(eon->base.context, "out of memory");
 			break;
 		}
-		entry->next = old;
-		old = entry;
+		list = insert_dirent(entry, list);
 	}
-	return old;
+	return list;
 }
 
-static int get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry **res)
+static dc_status_t
+get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry **res)
 {
+	dc_status_t rc = DC_STATUS_SUCCESS;
 	struct directory_entry *de = NULL;
 	unsigned char cmd[64];
 	unsigned char result[2048];
-	int rc, cmdlen;
+	unsigned int n = 0;
+	unsigned int cmdlen;
 
-
-	*res = NULL;
-	put_le32(0, cmd);
+	array_uint32_le_set(cmd, 0);
 	memcpy(cmd + 4, dive_directory, sizeof(dive_directory));
 	cmdlen = 4 + sizeof(dive_directory);
-	rc = send_receive(eon, CMD_DIR_OPEN,
-		cmdlen, cmd,
-		sizeof(result), result);
-	if (rc < 0) {
+	rc = suunto_eonsteel_transfer(eon, CMD_DIR_OPEN,
+		cmd, cmdlen, result, sizeof(result), &n);
+	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "cmd DIR_LOOKUP failed");
+		return rc;
 	}
-	HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "DIR_LOOKUP", result, rc);
+	HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "DIR_LOOKUP", result, n);
 
 	for (;;) {
 		unsigned int nr, last;
 
-		rc = send_receive(eon, CMD_DIR_READDIR,
-			0, NULL,
-			sizeof(result), result);
-		if (rc < 0) {
+		rc = suunto_eonsteel_transfer(eon, CMD_DIR_READDIR,
+			NULL, 0, result, sizeof(result), &n);
+		if (rc != DC_STATUS_SUCCESS) {
 			ERROR(eon->base.context, "readdir failed");
-			return -1;
+			file_list_free(de);
+			return rc;
 		}
-		if (rc < 8) {
+		if (n < 8) {
 			ERROR(eon->base.context, "short readdir result");
-			return -1;
+			file_list_free(de);
+			return DC_STATUS_PROTOCOL;
 		}
 		nr = array_uint32_le(result);
 		last = array_uint32_le(result+4);
 		HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "dir packet", result, 8);
 
-		de = parse_dirent(eon, nr, result+8, rc-8, de);
+		de = parse_dirent(eon, nr, result+8, n-8, de);
 		if (last)
 			break;
 	}
 
-	rc = send_receive(eon, CMD_DIR_CLOSE,
-		0, NULL,
-		sizeof(result), result);
-	if (rc < 0) {
+	rc = suunto_eonsteel_transfer(eon, CMD_DIR_CLOSE,
+		NULL, 0, result, sizeof(result), NULL);
+	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "dir close failed");
+		file_list_free(de);
+		return rc;
 	}
 
 	*res = de;
-	return 0;
+
+	return DC_STATUS_SUCCESS;
 }
 
-static int initialize_eonsteel(suunto_eonsteel_device_t *eon)
+static int
+count_file_list(struct directory_entry *list)
 {
-	const unsigned char init[] = {0x02, 0x00, 0x2a, 0x00};
-	unsigned char buf[64];
-	struct eon_hdr hdr;
+	int count = 0;
 
-	dc_usbhid_set_timeout(eon->usbhid, 10);
-
-	/* Get rid of any pending stale input first */
-	for (;;) {
-		size_t transferred = 0;
-
-		dc_status_t rc = dc_usbhid_read(eon->usbhid, buf, sizeof(buf), &transferred);
-		if (rc != DC_STATUS_SUCCESS)
-			break;
-		if (!transferred)
-			break;
+	while (list) {
+		count++;
+		list = list->next;
 	}
 
-	dc_usbhid_set_timeout(eon->usbhid, 5000);
-
-	if (send_cmd(eon, CMD_INIT, sizeof(init), init)) {
-		ERROR(eon->base.context, "Failed to send initialization command");
-		return -1;
-	}
-	if (receive_header(eon, &hdr, eon->version, sizeof(eon->version)) < 0) {
-		ERROR(eon->base.context, "Failed to receive initial reply");
-		return -1;
-	}
-
-	// Don't ask
-	eon->magic = (hdr.magic & 0xffff0000) | 0x0005;
-	// Increment the sequence number for every command sent
-	eon->seq++;
-	return 0;
+	return count;
 }
 
 dc_status_t
-suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, unsigned int model)
+suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream, unsigned int model)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_eonsteel_device_t *eon = NULL;
+	dc_transport_t transport = dc_iostream_get_transport (iostream);
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
@@ -574,35 +634,55 @@ suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, unsigned i
 	memset (eon->version, 0, sizeof (eon->version));
 	memset (eon->fingerprint, 0, sizeof (eon->fingerprint));
 
-	unsigned int vid = 0x1493, pid = 0;
-	if (model == EONCORE) {
-		pid = 0x0033;
+	if (transport == DC_TRANSPORT_BLE) {
+		status = dc_hdlc_open (&eon->iostream, context, iostream, 20, 20);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (context, "Failed to create the HDLC stream.");
+			goto error_free;
+		}
 	} else {
-		pid = 0x0030;
-	}
-	status = dc_usbhid_open(&eon->usbhid, context, vid, pid);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR(context, "unable to open device");
-		goto error_free;
+		eon->iostream = iostream;
 	}
 
-	if (initialize_eonsteel(eon) < 0) {
+	status = dc_iostream_set_timeout(eon->iostream, 5000);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to set the timeout.");
+		goto error_free_iostream;
+	}
+
+	const unsigned char init[] = {0x02, 0x00, 0x2a, 0x00};
+	status = suunto_eonsteel_transfer(eon, CMD_INIT,
+		init, sizeof(init), eon->version, sizeof(eon->version), NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR(context, "unable to initialize device");
-		status = DC_STATUS_IO;
-		goto error_close;
+		goto error_free_iostream;
 	}
 
 	*out = (dc_device_t *) eon;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_usbhid_close(eon->usbhid);
+error_free_iostream:
+	if (transport == DC_TRANSPORT_BLE) {
+		dc_iostream_close (eon->iostream);
+	}
 error_free:
-	free(eon);
+	dc_device_deallocate ((dc_device_t *) eon);
 	return status;
 }
 
+static dc_status_t
+suunto_eonsteel_device_close (dc_device_t *abstract)
+{
+	suunto_eonsteel_device_t *device = (suunto_eonsteel_device_t *) abstract;
+
+	if (dc_iostream_get_transport (device->iostream) == DC_TRANSPORT_BLE) {
+		return dc_iostream_close (device->iostream);
+	}
+
+	return DC_STATUS_SUCCESS;
+}
+
 static dc_status_t
 suunto_eonsteel_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
@@ -622,13 +702,14 @@ suunto_eonsteel_device_set_fingerprint (dc_device_t *abstract, const unsigned ch
 static dc_status_t
 suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
-	int skip = 0, rc;
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	int skip = 0;
 	struct directory_entry *de;
 	suunto_eonsteel_device_t *eon = (suunto_eonsteel_device_t *) abstract;
 	dc_buffer_t *file;
 	char pathname[64];
 	unsigned int time;
-	unsigned int count = 0;
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 
 	// Emit a device info event.
@@ -638,47 +719,25 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 	devinfo.serial = array_convert_str2num(eon->version + 0x10, 16);
 	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
-	if (get_file_list(eon, &de) < 0)
-		return DC_STATUS_IO;
+	rc = get_file_list(eon, &de);
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
 
 	if (de == NULL) {
 		return DC_STATUS_SUCCESS;
 	}
 
-	// Locate the most recent dive.
-	// The filename represent the time of the dive, encoded as a hexadecimal
-	// number. Thus the most recent dive can be found by simply sorting the
-	// filenames alphabetically.
-	struct directory_entry *head = de, *tail = de, *latest = de;
-	while (de) {
-		if (strcmp (de->name, latest->name) > 0) {
-			latest = de;
-		}
-		tail = de;
-		count++;
-		de = de->next;
+	file = dc_buffer_new (16384);
+	if (file == NULL) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		file_list_free(de);
+		return DC_STATUS_NOMEMORY;
 	}
 
-	// Make the most recent dive the head of the list.
-	// The linked list is made circular, by attaching the head to the tail and
-	// then cut open again just before the most recent dive.
-	de = head;
-	while (de) {
-		if (de->next == latest) {
-			de->next = NULL;
-			tail->next = head;
-			break;
-		}
-
-		de = de->next;
-	}
-
-	file = dc_buffer_new(0);
-	progress.maximum = count;
+	progress.maximum = count_file_list(de);
 	progress.current = 0;
 	device_event_emit(abstract, DC_EVENT_PROGRESS, &progress);
 
-	de = latest;
 	while (de) {
 		int len;
 		struct directory_entry *next = de->next;
@@ -686,8 +745,10 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 		const unsigned char *data = NULL;
 		unsigned int size = 0;
 
-		if (device_is_cancelled(abstract))
+		if (device_is_cancelled(abstract)) {
+			dc_status_set_error(&status, DC_STATUS_CANCELLED);
 			skip = 1;
+		}
 
 		switch (de->type) {
 		case DIRTYPE_DIR:
@@ -696,30 +757,39 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 		case DIRTYPE_FILE:
 			if (skip)
 				break;
-			if (sscanf(de->name, "%x.LOG", &time) != 1)
+
+			if (sscanf(de->name, "%x.LOG", &time) != 1) {
+				dc_status_set_error(&status, DC_STATUS_PROTOCOL);
 				break;
-			len = snprintf(pathname, sizeof(pathname), "%s/%s", dive_directory, de->name);
-			if (len >= sizeof(pathname))
+			}
+
+			array_uint32_le_set(buf, time);
+
+			if (memcmp (buf, eon->fingerprint, sizeof (eon->fingerprint)) == 0) {
+				skip = 1;
 				break;
+			}
+
+			len = dc_platform_snprintf(pathname, sizeof(pathname), "%s/%s", dive_directory, de->name);
+			if (len < 0 || (unsigned int) len >= sizeof(pathname)) {
+				dc_status_set_error(&status, DC_STATUS_PROTOCOL);
+				break;
+			}
 
 			// Reset the membuffer, put the 4-byte length at the head.
 			dc_buffer_clear(file);
-			put_le32(time, buf);
 			dc_buffer_append(file, buf, 4);
 
 			// Then read the filename into the rest of the buffer
 			rc = read_file(eon, pathname, file);
-			if (rc < 0)
+			if (rc != DC_STATUS_SUCCESS) {
+				dc_status_set_error(&status, rc);
 				break;
+			}
 
 			data = dc_buffer_get_data(file);
 			size = dc_buffer_get_size(file);
 
-			if (memcmp (data, eon->fingerprint, sizeof (eon->fingerprint)) == 0) {
-				skip = 1;
-				break;
-			}
-
 			if (callback && !callback(data, size, data, sizeof(eon->fingerprint), userdata))
 				skip = 1;
 		}
@@ -731,16 +801,16 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 	}
 	dc_buffer_free(file);
 
-	return device_is_cancelled(abstract) ? DC_STATUS_CANCELLED : DC_STATUS_SUCCESS;
+	return status;
 }
 
 static dc_status_t suunto_eonsteel_device_timesync(dc_device_t *abstract, const dc_datetime_t *datetime)
 {
 	suunto_eonsteel_device_t *eon = (suunto_eonsteel_device_t *) abstract;
+	dc_status_t rc = DC_STATUS_SUCCESS;
 	unsigned char result[64], cmd[8];
 	unsigned int year, month, day;
 	unsigned int hour, min, msec;
-	int rc;
 
 	year = datetime->year;
 	month = datetime->month;
@@ -758,20 +828,15 @@ static dc_status_t suunto_eonsteel_device_timesync(dc_device_t *abstract, const
 	cmd[6] = msec & 0xFF;
 	cmd[7] = msec >> 8;
 
-	rc = send_receive(eon, CMD_SET_TIME, sizeof(cmd), cmd, sizeof(result), result);
-	if (rc < 0) {
-		return DC_STATUS_IO;
+	rc = suunto_eonsteel_transfer(eon, CMD_SET_TIME, cmd, sizeof(cmd), result, sizeof(result), NULL);
+	if (rc != DC_STATUS_SUCCESS) {
+		return rc;
+	}
+
+	rc = suunto_eonsteel_transfer(eon, CMD_SET_DATE, cmd, sizeof(cmd), result, sizeof(result), NULL);
+	if (rc != DC_STATUS_SUCCESS) {
+		return rc;
 	}
 
 	return DC_STATUS_SUCCESS;
 }
-
-static dc_status_t
-suunto_eonsteel_device_close(dc_device_t *abstract)
-{
-	suunto_eonsteel_device_t *eon = (suunto_eonsteel_device_t *) abstract;
-
-	dc_usbhid_close(eon->usbhid);
-
-	return DC_STATUS_SUCCESS;
-}
