@@ -30,8 +30,6 @@
 #include "array.h"
 #include "platform.h"
 
-#define C_ARRAY_SIZE(a) (sizeof(a) / sizeof(*(a)))
-
 enum eon_sample {
 	ES_none = 0,
 	ES_dtime,		// duint16,precision=3 (time delta in ms)
@@ -64,7 +62,7 @@ enum eon_sample {
 #define EON_MAX_GROUP 16
 
 struct type_desc {
-	const char *desc, *format, *mod;
+	char *desc, *format, *mod;
 	unsigned int size;
 	enum eon_sample type[EON_MAX_GROUP];
 };
@@ -92,10 +90,11 @@ typedef struct suunto_eonsteel_parser_t {
 		dc_tankvolume_t tankinfo[MAXGASES];
 		double tanksize[MAXGASES];
 		double tankworkingpressure[MAXGASES];
+		dc_decomodel_t decomodel;
 	} cache;
 } suunto_eonsteel_parser_t;
 
-typedef int (*eon_data_cb_t)(unsigned short type, const struct type_desc *desc, const unsigned char *data, int len, void *user);
+typedef int (*eon_data_cb_t)(unsigned short type, const struct type_desc *desc, const unsigned char *data, unsigned int len, void *user);
 
 typedef struct eon_event_t {
 	const char *name;
@@ -137,7 +136,6 @@ static const struct {
 
 static enum eon_sample lookup_descriptor_type(suunto_eonsteel_parser_t *eon, struct type_desc *desc)
 {
-	int i;
 	const char *name = desc->desc;
 
 	// Not a sample type? Skip it
@@ -160,7 +158,7 @@ static enum eon_sample lookup_descriptor_type(suunto_eonsteel_parser_t *eon, str
 	name += 8;
 
 	// .. and look it up in the table of sample type strings
-	for (i = 0; i < C_ARRAY_SIZE(type_translation); i++) {
+	for (size_t i = 0; i < C_ARRAY_SIZE(type_translation); i++) {
 		if (!strcmp(name, type_translation[i].name))
 			return type_translation[i].type;
 	}
@@ -179,8 +177,7 @@ static parser_sample_event_t lookup_event(const char *name, const eon_event_t ev
 
 static const char *desc_type_name(enum eon_sample type)
 {
-	int i;
-	for (i = 0; i < C_ARRAY_SIZE(type_translation); i++) {
+	for (size_t i = 0; i < C_ARRAY_SIZE(type_translation); i++) {
 		if (type == type_translation[i].type)
 			return type_translation[i].name;
 	}
@@ -222,7 +219,7 @@ static int fill_in_group_details(suunto_eonsteel_parser_t *eon, struct type_desc
 		long index;
 
 		index = strtol(grp, &end, 10);
-		if (index < 0 || index > MAXTYPE || end == grp) {
+		if (index < 0 || index >= MAXTYPE || end == grp) {
 			ERROR(eon->base.context, "Group type descriptor '%s' does not parse", desc->desc);
 			break;
 		}
@@ -290,9 +287,9 @@ static void
 desc_free (struct type_desc desc[], unsigned int count)
 {
 	for (unsigned int i = 0; i < count; ++i) {
-		free((void *)desc[i].desc);
-		free((void *)desc[i].format);
-		free((void *)desc[i].mod);
+		free(desc[i].desc);
+		free(desc[i].format);
+		free(desc[i].mod);
 	}
 }
 
@@ -349,7 +346,7 @@ static int record_type(suunto_eonsteel_parser_t *eon, unsigned short type, const
 		}
 	} while ((name = next) != NULL);
 
-	if (type > MAXTYPE) {
+	if (type >= MAXTYPE) {
 		ERROR(eon->base.context, "Type out of range (%04x: '%s' '%s' '%s')",
 			type,
 			desc.desc ? desc.desc : "",
@@ -366,10 +363,10 @@ static int record_type(suunto_eonsteel_parser_t *eon, unsigned short type, const
 	return 0;
 }
 
-static int traverse_entry(suunto_eonsteel_parser_t *eon, const unsigned char *p, int len, eon_data_cb_t callback, void *user)
+static int traverse_entry(suunto_eonsteel_parser_t *eon, const unsigned char *p, int size, eon_data_cb_t callback, void *user)
 {
-	const unsigned char *name, *data, *end, *last, *one_past_end = p + len;
-	int textlen, type;
+	const unsigned char *name, *data, *end, *last, *one_past_end = p + size;
+	int textlen, id;
 	int rc;
 
 	// First two bytes: zero and text length
@@ -388,7 +385,7 @@ static int traverse_entry(suunto_eonsteel_parser_t *eon, const unsigned char *p,
 
 	// Two bytes of 'type' followed by the name/descriptor, followed by the data
 	data = name + textlen;
-	type = array_uint16_le(name);
+	id = array_uint16_le(name);
 	name += 2;
 
 	if (*name != '<') {
@@ -396,7 +393,7 @@ static int traverse_entry(suunto_eonsteel_parser_t *eon, const unsigned char *p,
 		return -1;
 	}
 
-	record_type(eon, type, (const char *) name, textlen-3);
+	record_type(eon, id, (const char *) name, textlen-3);
 
 	end = data;
 	last = data;
@@ -418,7 +415,7 @@ static int traverse_entry(suunto_eonsteel_parser_t *eon, const unsigned char *p,
 			end += 4;
 		}
 
-		if (type > MAXTYPE || !eon->type_desc[type].desc) {
+		if (type >= MAXTYPE || !eon->type_desc[type].desc) {
 			HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "last", last, 16);
 			HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "this", begin, 16);
 		} else {
@@ -463,8 +460,8 @@ struct sample_data {
 	dc_sample_callback_t callback;
 	void *userdata;
 	unsigned int time;
-	const char *state_type, *notify_type;
-	const char *warning_type, *alarm_type;
+	char *state_type, *notify_type;
+	char *warning_type, *alarm_type;
 
 	/* We gather up deco and cylinder pressure information */
 	int gasnr;
@@ -496,7 +493,7 @@ static void sample_temp(struct sample_data *info, short temp)
 {
 	dc_sample_value_t sample = {0};
 
-	if (temp < -3000)
+	if (temp <= -3000)
 		return;
 
 	sample.temperature = temp / 10.0;
@@ -551,7 +548,8 @@ static void sample_gastime(struct sample_data *info, short gastime)
 	if (gastime < 0)
 		return;
 
-	// Hmm. We have no good way to report airtime remaining
+	sample.rbt = gastime / 60;
+	if (info->callback) info->callback (DC_SAMPLE_RBT, sample, info->userdata);
 }
 
 /*
@@ -613,7 +611,7 @@ static void sample_gas_switch_event(struct sample_data *info, unsigned short idx
  *
  * "enum:0=NoFly Time,1=Depth,2=Surface Time,3=..."
  */
-static const char *lookup_enum(const struct type_desc *desc, unsigned char value)
+static char *lookup_enum(const struct type_desc *desc, unsigned char value)
 {
 	const char *str = desc->format;
 	unsigned char c;
@@ -674,6 +672,7 @@ static const char *lookup_enum(const struct type_desc *desc, unsigned char value
  */
 static void sample_event_state_type(const struct type_desc *desc, struct sample_data *info, unsigned char type)
 {
+	free(info->state_type);
 	info->state_type = lookup_enum(desc, type);
 }
 
@@ -705,6 +704,7 @@ static void sample_event_state_value(const struct type_desc *desc, struct sample
 
 static void sample_event_notify_type(const struct type_desc *desc, struct sample_data *info, unsigned char type)
 {
+	free(info->notify_type);
 	info->notify_type = lookup_enum(desc, type);
 }
 
@@ -747,6 +747,7 @@ static void sample_event_notify_value(const struct type_desc *desc, struct sampl
 
 static void sample_event_warning_type(const struct type_desc *desc, struct sample_data *info, unsigned char type)
 {
+	free(info->warning_type);
 	info->warning_type = lookup_enum(desc, type);
 }
 
@@ -785,6 +786,7 @@ static void sample_event_warning_value(const struct type_desc *desc, struct samp
 
 static void sample_event_alarm_type(const struct type_desc *desc, struct sample_data *info, unsigned char type)
 {
+	free(info->alarm_type);
 	info->alarm_type = lookup_enum(desc, type);
 }
 
@@ -819,7 +821,7 @@ static void sample_event_alarm_value(const struct type_desc *desc, struct sample
 static void sample_setpoint_type(const struct type_desc *desc, struct sample_data *info, unsigned char value)
 {
 	dc_sample_value_t sample = {0};
-	const char *type = lookup_enum(desc, value);
+	char *type = lookup_enum(desc, value);
 
 	if (!type) {
 		DEBUG(info->eon->base.context, "sample_setpoint_type(%u) did not match anything in %s", value, desc->format);
@@ -827,17 +829,19 @@ static void sample_setpoint_type(const struct type_desc *desc, struct sample_dat
 	}
 
 	if (!strcasecmp(type, "Low"))
-		sample.ppo2 = info->eon->cache.lowsetpoint;
+		sample.setpoint = info->eon->cache.lowsetpoint;
 	else if (!strcasecmp(type, "High"))
-		sample.ppo2 = info->eon->cache.highsetpoint;
+		sample.setpoint = info->eon->cache.highsetpoint;
 	else if (!strcasecmp(type, "Custom"))
-		sample.ppo2 = info->eon->cache.customsetpoint;
+		sample.setpoint = info->eon->cache.customsetpoint;
 	else {
 		DEBUG(info->eon->base.context, "sample_setpoint_type(%u) unknown type '%s'", value, type);
+		free(type);
 		return;
 	}
 
 	if (info->callback) info->callback(DC_SAMPLE_SETPOINT, sample, info->userdata);
+	free(type);
 }
 
 // uint32
@@ -853,7 +857,7 @@ static void sample_setpoint_automatic(struct sample_data *info, unsigned char va
 	DEBUG(info->eon->base.context, "sample_setpoint_automatic(%u)", value);
 }
 
-static int handle_sample_type(const struct type_desc *desc, struct sample_data *info, enum eon_sample type, const unsigned char *data)
+static unsigned int handle_sample_type(const struct type_desc *desc, struct sample_data *info, enum eon_sample type, const unsigned char *data)
 {
 	switch (type) {
 	case ES_dtime:
@@ -961,7 +965,7 @@ static int handle_sample_type(const struct type_desc *desc, struct sample_data *
 	}
 }
 
-static int traverse_samples(unsigned short type, const struct type_desc *desc, const unsigned char *data, int len, void *user)
+static int traverse_samples(unsigned short type, const struct type_desc *desc, const unsigned char *data, unsigned int len, void *user)
 {
 	struct sample_data *info = (struct sample_data *) user;
 	suunto_eonsteel_parser_t *eon = info->eon;
@@ -975,8 +979,7 @@ static int traverse_samples(unsigned short type, const struct type_desc *desc, c
 	info->ceiling = 0.0;
 
 	for (i = 0; i < EON_MAX_GROUP; i++) {
-		enum eon_sample type = desc->type[i];
-		int bytes = handle_sample_type(desc, info, type, data);
+		unsigned int bytes = handle_sample_type(desc, info, desc->type[i], data);
 
 		if (!bytes)
 			break;
@@ -1011,6 +1014,12 @@ suunto_eonsteel_parser_samples_foreach(dc_parser_t *abstract, dc_sample_callback
 	struct sample_data data = { eon, callback, userdata, 0 };
 
 	traverse_data(eon, traverse_samples, &data);
+
+	free(data.state_type);
+	free(data.notify_type);
+	free(data.warning_type);
+	free(data.alarm_type);
+
 	return DC_STATUS_SUCCESS;
 }
 
@@ -1087,6 +1096,9 @@ suunto_eonsteel_parser_get_field(dc_parser_t *parser, dc_field_type_t type, unsi
 				tank->type = DC_TANKVOLUME_IMPERIAL;
 		}
 		break;
+	case DC_FIELD_DECOMODEL:
+		field_value(value, eon->cache.decomodel);
+		break;
 	default:
 		return DC_STATUS_UNSUPPORTED;
 	}
@@ -1146,7 +1158,7 @@ static int add_gas_type(suunto_eonsteel_parser_t *eon, const struct type_desc *d
 {
 	int idx = eon->cache.ngases;
 	dc_tankvolume_t tankinfo = DC_TANKVOLUME_METRIC;
-	const char *name;
+	char *name;
 
 	if (idx >= MAXGASES)
 		return 0;
@@ -1168,6 +1180,7 @@ static int add_gas_type(suunto_eonsteel_parser_t *eon, const struct type_desc *d
 
 	eon->cache.initialized |= 1 << DC_FIELD_GASMIX_COUNT;
 	eon->cache.initialized |= 1 << DC_FIELD_TANK_COUNT;
+	free(name);
 	return 0;
 }
 
@@ -1232,8 +1245,6 @@ static float get_le32_float(const unsigned char *src)
 static int traverse_device_fields(suunto_eonsteel_parser_t *eon, const struct type_desc *desc,
                                   const unsigned char *data, int len)
 {
-	const char *name = desc->desc + strlen("sml.DeviceLog.Device.");
-
 	return 0;
 }
 
@@ -1327,13 +1338,30 @@ static int traverse_diving_fields(suunto_eonsteel_parser_t *eon, const struct ty
 	}
 
 	if (!strcmp(name, "DiveMode")) {
-		if (!strncmp((const char *)data, "CCR", 3)) {
-			eon->cache.divemode = DC_DIVEMODE_CC;
+		if (!strncmp((const char *)data, "Air", 3) || !strncmp((const char *)data, "Nitrox", 6)) {
+			eon->cache.divemode = DC_DIVEMODE_OC;
+			eon->cache.initialized |= 1 << DC_FIELD_DIVEMODE;
+		} else if (!strncmp((const char *)data, "CCR", 3)) {
+			eon->cache.divemode = DC_DIVEMODE_CCR;
 			eon->cache.initialized |= 1 << DC_FIELD_DIVEMODE;
 		}
 		return 0;
 	}
 
+	if (!strcmp(name, "Algorithm")) {
+		if (!strcmp((const char *)data, "Suunto Fused RGBM")) {
+			eon->cache.decomodel.type = DC_DECOMODEL_RGBM;
+			eon->cache.initialized |= 1 << DC_FIELD_DECOMODEL;
+		}
+		return 0;
+	}
+
+	if (!strcmp(name, "Conservatism")) {
+		eon->cache.decomodel.conservatism = *(const signed char *)data;
+		eon->cache.initialized |= 1 << DC_FIELD_DECOMODEL;
+		return 0;
+	}
+
 	if (!strcmp(name, "LowSetPoint")) {
 		unsigned int pressure = array_uint32_le(data); // in SI units - Pascal
 		eon->cache.lowsetpoint = pressure / 100000.0; // bar
@@ -1415,13 +1443,15 @@ static int traverse_sample_fields(suunto_eonsteel_parser_t *eon, const struct ty
 			set_depth_field(eon, array_uint16_le(data));
 			data += 2;
 			continue;
+		default:
+			break;
 		}
 		break;
 	}
 	return 0;
 }
 
-static int traverse_fields(unsigned short type, const struct type_desc *desc, const unsigned char *data, int len, void *user)
+static int traverse_fields(unsigned short type, const struct type_desc *desc, const unsigned char *data, unsigned int len, void *user)
 {
 	suunto_eonsteel_parser_t *eon = (suunto_eonsteel_parser_t *) user;
 
@@ -1498,6 +1528,9 @@ static const dc_parser_vtable_t suunto_eonsteel_parser_vtable = {
 	sizeof(suunto_eonsteel_parser_t),
 	DC_FAMILY_SUUNTO_EONSTEEL,
 	suunto_eonsteel_parser_set_data, /* set_data */
+	NULL, /* set_clock */
+	NULL, /* set_atmospheric */
+	NULL, /* set_density */
 	suunto_eonsteel_parser_get_datetime, /* datetime */
 	suunto_eonsteel_parser_get_field, /* fields */
 	suunto_eonsteel_parser_samples_foreach, /* samples_foreach */
