@@ -27,7 +27,6 @@
 #include "context-private.h"
 #include "device-private.h"
 #include "ringbuffer.h"
-#include "serial.h"
 #include "array.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &suunto_solution_device_vtable)
@@ -39,12 +38,11 @@
 
 typedef struct suunto_solution_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 } suunto_solution_device_t;
 
 static dc_status_t suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t suunto_solution_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t suunto_solution_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t suunto_solution_device_vtable = {
 	sizeof(suunto_solution_device_t),
@@ -55,14 +53,14 @@ static const dc_device_vtable_t suunto_solution_device_vtable = {
 	suunto_solution_device_dump, /* dump */
 	suunto_solution_device_foreach, /* foreach */
 	NULL, /* timesync */
-	suunto_solution_device_close /* close */
+	NULL /* close */
 };
 
 static dc_status_t
 suunto_solution_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
-suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+suunto_solution_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_solution_device_t *device = NULL;
@@ -78,74 +76,47 @@ suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const cha
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (1200 8N2).
+	status = dc_iostream_configure (device->iostream, 1200, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (1200 8N2).
-	status = dc_serial_configure (device->port, 1200, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (1000ms).
-	status = dc_serial_set_timeout (device->port, 1000);
+	status = dc_iostream_set_timeout (device->iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Clear the RTS line.
-	status = dc_serial_set_rts (device->port, 0);
+	status = dc_iostream_set_rts (device->iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR/RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
 
-static dc_status_t
-suunto_solution_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	suunto_solution_device_t *device = (suunto_solution_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
-
 static dc_status_t
 suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_solution_device_t *device = (suunto_solution_device_t*) abstract;
 
-	// Erase the current contents of the buffer and
-	// allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, SZ_MEMORY)) {
+	// Allocate the required amount of memory.
+	if (!dc_buffer_resize (buffer, SZ_MEMORY)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
@@ -161,14 +132,18 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	unsigned char answer[3] = {0};
 
 	// Assert DTR
-	dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr(device->iostream, 1);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to set the DTR line.");
+		return status;
+	}
 
 	// Send: 0xFF
 	command[0] = 0xFF;
-	dc_serial_write (device->port, command, 1, NULL);
+	dc_iostream_write (device->iostream, command, 1, NULL);
 
 	// Receive: 0x3F
-	status = dc_serial_read (device->port, answer, 1, NULL);
+	status = dc_iostream_read (device->iostream, answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x3F)
@@ -178,7 +153,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	command[0] = 0x4D;
 	command[1] = 0x01;
 	command[2] = 0x01;
-	dc_serial_write (device->port, command, 3, NULL);
+	dc_iostream_write (device->iostream, command, 3, NULL);
 
 	// Update and emit a progress event.
 	progress.current += 1;
@@ -187,7 +162,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	data[0] = 0x00;
 	for (unsigned int i = 1; i < SZ_MEMORY; ++i) {
 		// Receive: 0x01, i, data[i]
-		status = dc_serial_read (device->port, answer, 3, NULL);
+		status = dc_iostream_read (device->iostream, answer, 3, NULL);
 		if (status != DC_STATUS_SUCCESS)
 			return status;
 		if (answer[0] != 0x01 || answer[1] != i)
@@ -195,10 +170,10 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 		// Send: i
 		command[0] = i;
-		dc_serial_write (device->port, command, 1, NULL);
+		dc_iostream_write (device->iostream, command, 1, NULL);
 
 		// Receive: data[i]
-		status = dc_serial_read (device->port, data + i, 1, NULL);
+		status = dc_iostream_read (device->iostream, data + i, 1, NULL);
 		if (status != DC_STATUS_SUCCESS)
 			return status;
 		if (data[i] != answer[2])
@@ -206,7 +181,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 		// Send: 0x0D
 		command[0] = 0x0D;
-		dc_serial_write (device->port, command, 1, NULL);
+		dc_iostream_write (device->iostream, command, 1, NULL);
 
 		// Update and emit a progress event.
 		progress.current += 1;
@@ -214,7 +189,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	}
 
 	// Receive: 0x02, 0x00, 0x80
-	status = dc_serial_read (device->port, answer, 3, NULL);
+	status = dc_iostream_read (device->iostream, answer, 3, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x02 || answer[1] != 0x00 || answer[2] != 0x80)
@@ -222,10 +197,10 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send: 0x80
 	command[0] = 0x80;
-	dc_serial_write (device->port, command, 1, NULL);
+	dc_iostream_write (device->iostream, command, 1, NULL);
 
 	// Receive: 0x80
-	status = dc_serial_read (device->port, answer, 1, NULL);
+	status = dc_iostream_read (device->iostream, answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x80)
@@ -233,10 +208,10 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send: 0x20
 	command[0] = 0x20;
-	dc_serial_write (device->port, command, 1, NULL);
+	dc_iostream_write (device->iostream, command, 1, NULL);
 
 	// Receive: 0x3F
-	status = dc_serial_read (device->port, answer, 1, NULL);
+	status = dc_iostream_read (device->iostream, answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x3F)
@@ -246,6 +221,13 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	progress.current += 1;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
+	// Emit a device info event.
+	dc_event_devinfo_t devinfo;
+	devinfo.model = 0;
+	devinfo.firmware = 0;
+	devinfo.serial = array_convert_bcd2dec (data + 0x1D, 3);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
 	return DC_STATUS_SUCCESS;
 }
 
@@ -263,18 +245,6 @@ suunto_solution_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 		return rc;
 	}
 
-	// Emit a device info event.
-	unsigned char *data = dc_buffer_get_data (buffer);
-	dc_event_devinfo_t devinfo;
-	devinfo.model = 0;
-	devinfo.firmware = 0;
-	devinfo.serial = 0;
-	for (unsigned int i = 0; i < 3; ++i) {
-		devinfo.serial *= 100;
-		devinfo.serial += bcd2dec (data[0x1D + i]);
-	}
-	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
-
 	rc = suunto_solution_extract_dives (abstract,
 		dc_buffer_get_data (buffer), dc_buffer_get_size (buffer), callback, userdata);
 
