@@ -25,7 +25,6 @@
 #include "uwatec_aladin.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "ringbuffer.h"
 #include "checksum.h"
 #include "array.h"
@@ -43,7 +42,7 @@
 
 typedef struct uwatec_aladin_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned int timestamp;
 	unsigned int devtime;
 	dc_ticks_t systime;
@@ -52,7 +51,6 @@ typedef struct uwatec_aladin_device_t {
 static dc_status_t uwatec_aladin_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t uwatec_aladin_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t uwatec_aladin_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t uwatec_aladin_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t uwatec_aladin_device_vtable = {
 	sizeof(uwatec_aladin_device_t),
@@ -63,14 +61,14 @@ static const dc_device_vtable_t uwatec_aladin_device_vtable = {
 	uwatec_aladin_device_dump, /* dump */
 	uwatec_aladin_device_foreach, /* foreach */
 	NULL, /* timesync */
-	uwatec_aladin_device_close /* close */
+	NULL /* close */
 };
 
 static dc_status_t
 uwatec_aladin_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
-uwatec_aladin_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+uwatec_aladin_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	uwatec_aladin_device_t *device = NULL;
@@ -86,75 +84,49 @@ uwatec_aladin_device_open (dc_device_t **out, dc_context_t *context, const char
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 	device->timestamp = 0;
 	device->systime = (dc_ticks_t) -1;
 	device->devtime = 0;
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (19200 8N1).
+	status = dc_iostream_configure (device->iostream, 19200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (19200 8N1).
-	status = dc_serial_configure (device->port, 19200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
-	// Set the timeout for receiving data (INFINITE).
-	status = dc_serial_set_timeout (device->port, -1);
+	// Set the timeout for receiving data (3000ms).
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Clear the RTS line.
-	status = dc_serial_set_rts (device->port, 0);
+	status = dc_iostream_set_rts (device->iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to clear the RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
 
-static dc_status_t
-uwatec_aladin_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	uwatec_aladin_device_t *device = (uwatec_aladin_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
-
 static dc_status_t
 uwatec_aladin_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
@@ -178,9 +150,8 @@ uwatec_aladin_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	dc_status_t status = DC_STATUS_SUCCESS;
 	uwatec_aladin_device_t *device = (uwatec_aladin_device_t*) abstract;
 
-	// Erase the current contents of the buffer and
-	// pre-allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, SZ_MEMORY)) {
+	// Pre-allocate the required amount of memory.
+	if (!dc_buffer_reserve (buffer, SZ_MEMORY)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
@@ -193,20 +164,24 @@ uwatec_aladin_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	unsigned char answer[SZ_MEMORY + 2] = {0};
 
 	// Receive the header of the package.
-	for (unsigned int i = 0; i < 4;) {
+	unsigned int i = 0;
+	while (i < 4) {
 		if (device_is_cancelled (abstract))
 			return DC_STATUS_CANCELLED;
 
-		status = dc_serial_read (device->port, answer + i, 1, NULL);
+		status = dc_iostream_read (device->iostream, answer + i, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
-			return status;
+			if (status != DC_STATUS_TIMEOUT)
+				return status;
 		}
-		if (answer[i] == (i < 3 ? 0x55 : 0x00)) {
-			i++; // Continue.
-		} else {
-			i = 0; // Reset.
+
+		const unsigned char expected = i < 3 ? 0x55 : 0x00;
+		if (status != DC_STATUS_SUCCESS || answer[i] != expected) {
 			device_event_emit (abstract, DC_EVENT_WAITING, NULL);
+			i = 0;
+		} else {
+			i++;
 		}
 	}
 
@@ -218,7 +193,7 @@ uwatec_aladin_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Receive the remaining part of the package.
-	status = dc_serial_read (device->port, answer + 4, sizeof (answer) - 4, NULL);
+	status = dc_iostream_read (device->iostream, answer + 4, sizeof (answer) - 4, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Unexpected EOF in answer.");
 		return status;
