@@ -27,10 +27,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 
 dc_status_t
-uwatec_aladin_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+uwatec_aladin_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 #ifdef __cplusplus
 }
