@@ -23,22 +23,16 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
-#include <libdivecomputer/zeagle_n2ition3.h>
-
+#include "zeagle_n2ition3.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "checksum.h"
 #include "array.h"
 #include "ringbuffer.h"
+#include "rbstream.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &zeagle_n2ition3_device_vtable)
 
-#define EXITCODE(rc) \
-( \
-	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
-)
-
 #define SZ_MEMORY 0x8000
 #define SZ_PACKET 64
 
@@ -51,7 +45,7 @@
 
 typedef struct zeagle_n2ition3_device_t {
 	dc_device_t base;
-	serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned char fingerprint[16];
 } zeagle_n2ition3_device_t;
 
@@ -59,22 +53,24 @@ static dc_status_t zeagle_n2ition3_device_set_fingerprint (dc_device_t *abstract
 static dc_status_t zeagle_n2ition3_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size);
 static dc_status_t zeagle_n2ition3_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t zeagle_n2ition3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t zeagle_n2ition3_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t zeagle_n2ition3_device_vtable = {
+	sizeof(zeagle_n2ition3_device_t),
 	DC_FAMILY_ZEAGLE_N2ITION3,
 	zeagle_n2ition3_device_set_fingerprint, /* set_fingerprint */
 	zeagle_n2ition3_device_read, /* read */
 	NULL, /* write */
 	zeagle_n2ition3_device_dump, /* dump */
 	zeagle_n2ition3_device_foreach, /* foreach */
-	zeagle_n2ition3_device_close /* close */
+	NULL, /* timesync */
+	NULL /* close */
 };
 
 
 static dc_status_t
 zeagle_n2ition3_packet (zeagle_n2ition3_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
 
 	assert (asize >= csize + 5);
@@ -83,17 +79,17 @@ zeagle_n2ition3_packet (zeagle_n2ition3_device_t *device, const unsigned char co
 		return DC_STATUS_CANCELLED;
 
 	// Send the command to the device.
-	int n = serial_write (device->port, command, csize);
-	if (n != csize) {
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	// Receive the answer of the device.
-	n = serial_read (device->port, answer, asize);
-	if (n != asize) {
+	status = dc_iostream_read (device->iostream, answer, asize, NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	// Verify the echo.
@@ -135,51 +131,41 @@ zeagle_n2ition3_init (zeagle_n2ition3_device_t *device)
 }
 
 dc_status_t
-zeagle_n2ition3_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+zeagle_n2ition3_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
+	zeagle_n2ition3_device_t *device = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	zeagle_n2ition3_device_t *device = (zeagle_n2ition3_device_t *) malloc (sizeof (zeagle_n2ition3_device_t));
+	device = (zeagle_n2ition3_device_t *) dc_device_allocate (context, &zeagle_n2ition3_device_vtable);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base, context, &zeagle_n2ition3_device_vtable);
-
 	// Set the default values.
-	device->port = NULL;
-
-	// Open the device.
-	int rc = serial_open (&device->port, context, name);
-	if (rc == -1) {
-		ERROR (context, "Failed to open the serial port.");
-		free (device);
-		return DC_STATUS_IO;
-	}
+	device->iostream = iostream;
+	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
 	// Set the serial communication protocol (4800 8N1).
-	rc = serial_configure (device->port, 4800, 8, SERIAL_PARITY_NONE, 1, SERIAL_FLOWCONTROL_NONE);
-	if (rc == -1) {
+	status = dc_iostream_configure (device->iostream, 4800, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_free;
 	}
 
 	// Set the timeout for receiving data (1000 ms).
-	if (serial_set_timeout (device->port, 1000) == -1) {
+	status = dc_iostream_set_timeout (device->iostream, 1000);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_free;
 	}
 
 	// Make sure everything is in a sane state.
-	serial_flush (device->port, SERIAL_QUEUE_BOTH);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Send the init commands.
 	zeagle_n2ition3_init (device);
@@ -187,24 +173,10 @@ zeagle_n2ition3_device_open (dc_device_t **out, dc_context_t *context, const cha
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
-}
 
-
-static dc_status_t
-zeagle_n2ition3_device_close (dc_device_t *abstract)
-{
-	zeagle_n2ition3_device_t *device = (zeagle_n2ition3_device_t*) abstract;
-
-	// Close the device.
-	if (serial_close (device->port) == -1) {
-		free (device);
-		return DC_STATUS_IO;
-	}
-
-	// Free memory.
-	free (device);
-
-	return DC_STATUS_SUCCESS;
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
@@ -263,14 +235,13 @@ zeagle_n2ition3_device_read (dc_device_t *abstract, unsigned int address, unsign
 static dc_status_t
 zeagle_n2ition3_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
-	// Erase the current contents of the buffer and
-	// allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, SZ_MEMORY)) {
+	// Allocate the required amount of memory.
+	if (!dc_buffer_resize (buffer, SZ_MEMORY)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	return device_dump_read (abstract, dc_buffer_get_data (buffer),
+	return device_dump_read (abstract, 0, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), SZ_PACKET);
 }
 
@@ -301,7 +272,7 @@ zeagle_n2ition3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 		last < RB_LOGBOOK_BEGIN || last >= RB_LOGBOOK_END) {
 		if (last == 0xFF)
 			return DC_STATUS_SUCCESS;
-		ERROR (abstract->context, "Invalid ringbuffer pointer detected.");
+		ERROR (abstract->context, "Invalid ringbuffer pointer detected (0x%02x 0x%02x).", first, last);
 		return DC_STATUS_DATAFORMAT;
 	}
 
@@ -311,10 +282,10 @@ zeagle_n2ition3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 	// Get the profile pointer.
 	unsigned int eop = array_uint16_le (config + 0x7E);
 	if (eop < RB_PROFILE_BEGIN || eop >= RB_PROFILE_END) {
-		ERROR (abstract->context, "Invalid ringbuffer pointer detected.");
+		ERROR (abstract->context, "Invalid ringbuffer pointer detected (0x%04x).", eop);
 		return DC_STATUS_DATAFORMAT;
 	}
-	
+
 	// The logbook ringbuffer can store at most 60 dives, even if the profile
 	// data could store more (e.g. many small dives). But it's also possible
 	// that the profile ringbuffer is filled faster than the logbook ringbuffer
@@ -326,13 +297,13 @@ zeagle_n2ition3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 		// Get the pointer to the profile data.
 		unsigned int current = array_uint16_le (config + 2 * idx);
 		if (current < RB_PROFILE_BEGIN || current >= RB_PROFILE_END) {
-			ERROR (abstract->context, "Invalid ringbuffer pointer detected.");
+			ERROR (abstract->context, "Invalid ringbuffer pointer detected (0x%04x).", current);
 			return DC_STATUS_DATAFORMAT;
 		}
 
 		// Get the profile length.
 		unsigned int length = ringbuffer_distance (current, previous, 1, RB_PROFILE_BEGIN, RB_PROFILE_END);
-		
+
 		// Check for a ringbuffer overflow.
 		if (total + length > RB_PROFILE_END - RB_PROFILE_BEGIN) {
 			count = i;
@@ -353,16 +324,21 @@ zeagle_n2ition3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 	progress.maximum = (RB_LOGBOOK_END - RB_LOGBOOK_BEGIN) * 2 + 8 + total;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
+	// Create the ringbuffer stream.
+	dc_rbstream_t *rbstream = NULL;
+	rc = dc_rbstream_new (&rbstream, abstract, 1, SZ_PACKET, RB_PROFILE_BEGIN, RB_PROFILE_END, eop);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to create the ringbuffer stream.");
+		return rc;
+	}
+
 	// Memory buffer for the profile data.
 	unsigned char buffer[RB_PROFILE_END - RB_PROFILE_BEGIN] = {0};
 
-	unsigned int available = 0;
-	unsigned int remaining = total;
 	unsigned int offset = RB_PROFILE_END - RB_PROFILE_BEGIN;
-	
+
 	idx = last;
 	previous = eop;
-	unsigned int address = previous;
 	for (unsigned int i = 0; i < count; ++i) {
 		// Get the pointer to the profile data.
 		unsigned int current = array_uint16_le (config + 2 * idx);
@@ -370,50 +346,37 @@ zeagle_n2ition3_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 		// Get the profile length.
 		unsigned int length = ringbuffer_distance (current, previous, 1, RB_PROFILE_BEGIN, RB_PROFILE_END);
 
-		unsigned nbytes = available;
-		while (nbytes < length) {
-			if (address == RB_PROFILE_BEGIN)
-				address = RB_PROFILE_END;
-			
-			unsigned int len = SZ_PACKET;
-			if (RB_PROFILE_BEGIN + len > address)
-				len = address - RB_PROFILE_BEGIN; // End of ringbuffer.
-			if (nbytes + len > remaining)
-				len = remaining - nbytes; // End of profile.
+		// Move to the begin of the current dive.
+		offset -= length;
 
-			address -= len;
-			offset -= len;
+		// Read the dive.
+		rc = dc_rbstream_read (rbstream, &progress, buffer + offset, length);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to read the dive.");
+			dc_rbstream_free (rbstream);
+			return rc;
+		}
 
-			// Read the memory page.
-			rc = zeagle_n2ition3_device_read (abstract, address, buffer + offset, len);
-			if (rc != DC_STATUS_SUCCESS) {
-				ERROR (abstract->context, "Failed to read the memory page.");
-				return rc;
-			}
+		unsigned char *p = buffer + offset;
 
-			// Update and emit a progress event.
-			progress.current += len;
-			device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+		if (memcmp (p, device->fingerprint, sizeof (device->fingerprint)) == 0) {
+			dc_rbstream_free (rbstream);
+			return DC_STATUS_SUCCESS;
+		}
 
-			nbytes += len;
+		if (callback && !callback (p, length, p, sizeof (device->fingerprint), userdata)) {
+			dc_rbstream_free (rbstream);
+			return DC_STATUS_SUCCESS;
 		}
 
-		remaining -= length;
-		available = nbytes - length;
 		previous = current;
 
-		unsigned char *p = buffer + offset + available;
-
-		if (memcmp (p, device->fingerprint, sizeof (device->fingerprint)) == 0)
-			return DC_STATUS_SUCCESS;
-
-		if (callback && !callback (p, length, p, sizeof (device->fingerprint), userdata))
-			return DC_STATUS_SUCCESS;
-
 		if (idx == RB_LOGBOOK_BEGIN)
 			idx = RB_LOGBOOK_END;
 		idx--;
 	}
 
+	dc_rbstream_free (rbstream);
+
 	return DC_STATUS_SUCCESS;
 }
